@@ -232,7 +232,7 @@ public:
     QMargins drawMargins() const;
     void invalidateDrawMargins();
 
-    QMargins computeLayerOffsetMargins() const;
+    void adjustBoundingRectForOffsetsAndImageLayers(QRect &boundingRect) const;
 
     int layerCount() const;
     int layerCount(Layer::TypeFlag type) const;
@@ -313,7 +313,8 @@ public:
     Layer *findLayerById(int layerId) const;
     MapObject *findObjectById(int objectId) const;
 
-    QRegion tileRegion() const;
+    QRect tileBoundingRect() const;
+    QRegion modifiedTileRegion() const;
 
 private:
     friend class GroupLayer;    // so it can call adoptLayer
@@ -790,3 +791,5 @@ Q_DECLARE_METATYPE(Tiled::Map*)
 Q_DECLARE_METATYPE(Tiled::Map::Orientation)
 Q_DECLARE_METATYPE(Tiled::Map::LayerDataFormat)
 Q_DECLARE_METATYPE(Tiled::Map::RenderOrder)
+Q_DECLARE_METATYPE(Tiled::Map::StaggerAxis)
+Q_DECLARE_METATYPE(Tiled::Map::StaggerIndex)
