@@ -28,10 +28,11 @@
 
 #pragma once
 
-#include "objecttypes.h"
 #include "properties.h"
 #include "propertytype.h"
 
+#include <QPointer>
+
 namespace Tiled {
 
 /**
@@ -40,18 +41,23 @@ namespace Tiled {
 class TILEDSHARED_EXPORT Object
 {
 public:
+    // Keep values synchronized with ClassPropertyType::ClassUsageFlag
     enum TypeId {
-        LayerType,
-        MapObjectType,
-        MapType,
-        ObjectTemplateType,
-        TilesetType,
-        TileType,
-        WangSetType,
-        WangColorType
+        LayerType           = 0x002,
+        MapObjectType       = 0x004,
+        MapType             = 0x008,
+        TilesetType         = 0x010,
+        TileType            = 0x020,
+        WangSetType         = 0x040,
+        WangColorType       = 0x080,
+        ProjectType         = 0x100,
+        WorldType           = 0x200,
     };
 
-    explicit Object(TypeId typeId) : mTypeId(typeId) {}
+    explicit Object(TypeId typeId, const QString &className = QString())
+        : mTypeId(typeId)
+        , mClassName(className)
+    {}
 
     /**
      * Virtual destructor.
@@ -63,11 +69,21 @@ public:
      */
     TypeId typeId() const { return mTypeId; }
 
+    const QString &className() const;
+    void setClassName(const QString &className);
+
+    const ClassPropertyType *classType() const;
+
     /**
      * Returns the properties of this object.
      */
     const Properties &properties() const { return mProperties; }
 
+    /**
+     * Returns the properties of this object.
+     */
+    Properties &properties() { return mProperties; }
+
     /**
      * Replaces all existing properties with a new set of properties.
      */
@@ -97,6 +113,7 @@ public:
 
     QVariant resolvedProperty(const QString &name) const;
     QVariantMap resolvedProperties() const;
+    QVariantMap inheritedProperties() const;
 
     /**
      * Returns the value of the object's \a name property, as a string.
@@ -125,6 +142,18 @@ public:
     void setProperty(const QString &name, const QVariant &value)
     { mProperties.insert(name, value); }
 
+    /**
+     * Sets the value of an object's property identified the given \a path
+     * to \a value.
+     *
+     * The \a path is a list of property names, where each name identifies
+     * a member of the previous member's value. The last name in the list
+     * identifies the property to set.
+     *
+     * Returns whether the property was set.
+     */
+    bool setProperty(const QStringList &path, const QVariant &value);
+
     /**
      * Removes the property with the given \a name.
      */
@@ -133,21 +162,40 @@ public:
 
     bool isPartOfTileset() const;
 
-    static void setObjectTypes(const ObjectTypes &objectTypes);
     static void setPropertyTypes(const SharedPropertyTypes &propertyTypes);
-    static const ObjectTypes &objectTypes()
-    { return mObjectTypes; }
     static const PropertyTypes &propertyTypes();
 
 private:
     const TypeId mTypeId;
+    QString mClassName;
     Properties mProperties;
 
-    static ObjectTypes mObjectTypes;
+    /**
+     * The editable wrapper created for this object.
+     */
+    QPointer<QObject> mEditable;
+    friend class EditableObject;
+
     static SharedPropertyTypes mPropertyTypes;
 };
 
 
+/**
+ * Returns the class of this object. The class usually says something about
+ * how the object is meant to be interpreted by the engine.
+ *
+ * Tile objects that do not have a class explicitly set on them are assumed to
+ * be of the class set on their tile (see MapObject::effectiveClassName).
+ */
+inline const QString &Object::className() const
+{ return mClassName; }
+
+/**
+ * Sets the class of this object.
+ */
+inline void Object::setClassName(const QString &className)
+{ mClassName = className; }
+
 /**
  * Returns whether this object is stored as part of a tileset.
  */
