@@ -32,36 +32,12 @@
 #include "mapobject.h"
 #include "tile.h"
 #include "tilelayer.h"
-#include "tileset.h"
 #include "objectgroup.h"
 
 #include <QtCore/qmath.h>
 
 using namespace Tiled;
 
-QRect OrthogonalRenderer::mapBoundingRect() const
-{
-    if (!map()->infinite()) {
-        return QRect(0, 0,
-                     map()->width() * map()->tileWidth(),
-                     map()->height() * map()->tileHeight());
-    }
-
-    QRect mapBounds;
-
-    LayerIterator iterator(map(), Layer::TileLayerType);
-    while (TileLayer *tileLayer = static_cast<TileLayer*>(iterator.next()))
-        mapBounds = mapBounds.united(tileLayer->bounds());
-
-    if (mapBounds.size() == QSize(0, 0))
-        mapBounds.setSize(QSize(1, 1));
-
-    return QRect(mapBounds.x() * map()->tileWidth(),
-                 mapBounds.y() * map()->tileHeight(),
-                 mapBounds.width() * map()->tileWidth(),
-                 mapBounds.height() * map()->tileHeight());
-}
-
 QRect OrthogonalRenderer::boundingRect(const QRect &rect) const
 {
     const int tileWidth = map()->tileWidth();
@@ -156,10 +132,25 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
 
     switch (object->shape()) {
     case MapObject::Rectangle: {
-        if (bounds.isNull())
+        if (bounds.isNull()) {
             path.addRect(object->x() - 10, object->y() - 10, 20, 20);
-        else
+        } else {
+            if (const Tile *tile = object->cell().tile()) {
+                QPointF tileOffset = tile->offset();
+                const QSize tileSize = tile->size();
+                if (!tileSize.isNull()) {
+                    const QSizeF scale {
+                        bounds.width() / tileSize.width(),
+                        bounds.height() / tileSize.height()
+                    };
+                    tileOffset.rx() *= scale.width();
+                    tileOffset.ry() *= scale.height();
+                }
+                bounds.translate(tileOffset);
+            }
+
             path.addRect(bounds);
+        }
         break;
     }
     case MapObject::Polygon:
@@ -176,7 +167,7 @@ QPainterPath OrthogonalRenderer::shape(const MapObject *object) const
     }
     case MapObject::Ellipse: {
         if (bounds.isNull())
-            path.addEllipse(bounds.topLeft(), 20, 20);
+            path.addEllipse(bounds.topLeft(), 10, 10);
         else
             path.addEllipse(bounds);
         break;
@@ -245,7 +236,7 @@ void OrthogonalRenderer::drawGrid(QPainter *painter, const QRectF &rect,
     }
 
     QPen gridPen, majorGridPen;
-    setupGridPens(painter->device(), gridColor, gridPen, majorGridPen, tileWidth, gridMajor);
+    setupGridPens(painter->device(), gridColor, gridPen, majorGridPen, qMin(tileWidth, tileHeight), gridMajor);
 
     if (startY < endY) {
         gridPen.setDashOffset(startY * tileHeight);
@@ -321,12 +312,7 @@ void OrthogonalRenderer::drawTileSelection(QPainter *painter,
 {
     QPainterPath path;
 
-#if QT_VERSION < 0x050800
-    const auto rects = region.rects();
-    for (const QRect &r : rects) {
-#else
     for (const QRect &r : region) {
-#endif
         const QRectF toFill = QRectF(boundingRect(r));
         if (toFill.intersects(exposed))
             path.addRect(toFill);
@@ -346,7 +332,7 @@ void OrthogonalRenderer::drawTileSelection(QPainter *painter,
 
 void OrthogonalRenderer::drawMapObject(QPainter *painter,
                                        const MapObject *object,
-                                       const QColor &color) const
+                                       const MapObjectColors &colors) const
 {
     painter->save();
 
@@ -384,7 +370,7 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             painter->setPen(pen);
             painter->drawRect(bounds);
             pen.setStyle(Qt::DotLine);
-            pen.setColor(color);
+            pen.setColor(colors.main);
             painter->setPen(pen);
             painter->drawRect(bounds);
         }
@@ -395,14 +381,12 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
         const QPointF shadowOffset = QPointF(shadowDist * 0.5,
                                              shadowDist * 0.5);
 
-        QPen linePen(color, lineWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
+        QPen linePen(colors.main, lineWidth, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
         linePen.setCosmetic(true);
         QPen shadowPen(linePen);
         shadowPen.setColor(Qt::black);
 
-        QColor brushColor = color;
-        brushColor.setAlpha(50);
-        const QBrush fillBrush(brushColor);
+        const QBrush fillBrush = colors.fill.isValid() ? QBrush(colors.fill) : QBrush(Qt::NoBrush);
 
         painter->setRenderHint(QPainter::Antialiasing);
 
@@ -430,8 +414,11 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             break;
         }
 
+        case MapObject::Polygon:
         case MapObject::Polyline: {
-            QPolygonF screenPolygon = pixelToScreenCoords(object->polygon());
+            const QPolygonF screenPolygon = pixelToScreenCoords(object->polygon());
+            const QPointF pointPos = screenPolygon.isEmpty() ? QPointF()
+                                                             : screenPolygon.first();
 
             QPen thickShadowPen(shadowPen);
             QPen thickLinePen(linePen);
@@ -439,36 +426,21 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             thickLinePen.setWidthF(thickLinePen.widthF() * 4);
 
             painter->setPen(shadowPen);
-            painter->drawPolyline(screenPolygon.translated(shadowOffset));
+            if (shape == MapObject::Polygon)
+                painter->drawPolygon(screenPolygon.translated(shadowOffset));
+            else
+                painter->drawPolyline(screenPolygon.translated(shadowOffset));
             painter->setPen(thickShadowPen);
-            painter->drawPoint(screenPolygon.first() + shadowOffset);
+            painter->drawPoint(pointPos + shadowOffset);
 
             painter->setPen(linePen);
             painter->setBrush(fillBrush);
-            painter->drawPolyline(screenPolygon);
+            if (shape == MapObject::Polygon)
+                painter->drawPolygon(screenPolygon);
+            else
+                painter->drawPolyline(screenPolygon);
             painter->setPen(thickLinePen);
-            painter->drawPoint(screenPolygon.first());
-            break;
-        }
-
-        case MapObject::Polygon: {
-            QPolygonF screenPolygon = pixelToScreenCoords(object->polygon());
-
-            QPen thickShadowPen(shadowPen);
-            QPen thickLinePen(linePen);
-            thickShadowPen.setWidthF(thickShadowPen.widthF() * 4);
-            thickLinePen.setWidthF(thickLinePen.widthF() * 4);
-
-            painter->setPen(shadowPen);
-            painter->drawPolygon(screenPolygon.translated(shadowOffset));
-            painter->setPen(thickShadowPen);
-            painter->drawPoint(screenPolygon.first() + shadowOffset);
-
-            painter->setPen(linePen);
-            painter->setBrush(fillBrush);
-            painter->drawPolygon(screenPolygon);
-            painter->setPen(thickLinePen);
-            painter->drawPoint(screenPolygon.first());
+            painter->drawPoint(pointPos);
             break;
         }
 
@@ -494,7 +466,7 @@ void OrthogonalRenderer::drawMapObject(QPainter *painter,
             break;
         }
         case MapObject::Point: {
-            drawPointObject(painter, color);
+            drawPointObject(painter, colors.main);
             break;
         }
         }
