@@ -1,6 +1,6 @@
 /*
  * tiled.h
- * Copyright 2013, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2013-2022, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  *
  * This file is part of libtiled.
  *
@@ -69,6 +69,15 @@ enum LoadingStatus {
     LoadingError
 };
 
+enum CompatibilityVersion {
+    UnknownVersion  = 0,
+    Tiled_1_8       = 1080,
+    Tiled_1_9       = 1090,
+    Tiled_1_10      = 1100,
+    Tiled_Current   = Tiled_1_10,
+    Tiled_Latest    = 65535,
+};
+
 const int CHUNK_SIZE = 16;
 const int CHUNK_BITS = 4;
 const int CHUNK_SIZE_MIN = 4;
@@ -79,7 +88,11 @@ static const char FRAMES_MIMETYPE[] = "application/vnd.frame.list";
 static const char LAYERS_MIMETYPE[] = "application/vnd.layer.list";
 static const char PROPERTIES_MIMETYPE[] = "application/vnd.properties.list";
 
-TILEDSHARED_EXPORT QPointF alignmentOffset(const QRectF &r, Alignment alignment);
+TILEDSHARED_EXPORT QPointF alignmentOffset(const QSizeF &size, Alignment alignment);
+TILEDSHARED_EXPORT Alignment flipAlignment(Alignment alignment, FlipDirection direction);
+
+inline QPointF alignmentOffset(const QRectF &r, Alignment alignment)
+{ return alignmentOffset(r.size(), alignment); }
 
 TILEDSHARED_EXPORT QString toFileReference(const QUrl &url, const QString &path = QString());
 TILEDSHARED_EXPORT QUrl toUrl(const QString &filePathOrUrl, const QString &path = QString());
@@ -115,6 +128,10 @@ inline QMargins maxMargins(const QMargins &a,
 TILEDSHARED_EXPORT QString alignmentToString(Alignment);
 TILEDSHARED_EXPORT Alignment alignmentFromString(const QString &);
 
+TILEDSHARED_EXPORT CompatibilityVersion versionFromString(const QString &);
+
+TILEDSHARED_EXPORT void increaseImageAllocationLimit(int mbLimit = 4096);
+
 } // namespace Tiled
 
 Q_DECLARE_METATYPE(Tiled::Alignment);
