@@ -63,6 +63,12 @@ using SharedTileset = QSharedPointer<Tileset>;
 class TILEDSHARED_EXPORT Tileset : public Object, public QEnableSharedFromThis<Tileset>
 {
 public:
+    // Used by TilesetChangeEvent
+    enum Property {
+        FillModeProperty,
+        TileRenderSizeProperty,
+    };
+
     /**
      * The orientation of the tileset determines the projection used in the
      * TileCollisionDock and for the Wang color overlay of the TilesetView.
@@ -72,6 +78,23 @@ public:
         Isometric,
     };
 
+    /**
+     * The size to use when rendering tiles from this tileset on a tile layer.
+     */
+    enum TileRenderSize {
+        TileSize,
+        GridSize,
+    };
+
+    /**
+     * The fill mode to use when rendering tiles from this tileset. Only
+     * relevant when the tiles are not rendered at their native size.
+     */
+    enum FillMode {
+        Stretch,
+        PreserveAspectFit
+    };
+
     /**
      * Creates a new tileset with the given parameters.
      *
@@ -130,6 +153,12 @@ public:
     Alignment objectAlignment() const;
     void setObjectAlignment(Alignment objectAlignment);
 
+    TileRenderSize tileRenderSize() const;
+    void setTileRenderSize(TileRenderSize tileRenderSize);
+
+    FillMode fillMode() const;
+    void setFillMode(FillMode fillMode);
+
     QPoint tileOffset() const;
     void setTileOffset(QPoint offset);
 
@@ -169,6 +198,7 @@ public:
     bool loadFromImage(const QImage &image, const QString &source);
     bool loadFromImage(const QString &fileName);
     bool loadImage();
+    bool initializeTilesetTiles();
 
     SharedTileset findSimilarTileset(const QVector<SharedTileset> &tilesets) const;
 
@@ -177,6 +207,8 @@ public:
     void setImageSource(const QString &url);
     QString imageSourceString() const;
 
+    const QPixmap &image() const;
+
     bool isCollection() const;
 
     int columnCountForWidth(int width) const;
@@ -190,13 +222,14 @@ public:
     void insertWangSet(int index, std::unique_ptr<WangSet> wangSet);
     std::unique_ptr<WangSet> takeWangSetAt(int index);
 
-    Tile *addTile(const QPixmap &image, const QUrl &source = QUrl());
+    Tile *addTile(const QPixmap &image, const QUrl &source = QUrl(), const QRect &rect = QRect());
     void addTiles(const QList<Tile*> &tiles);
     void removeTiles(const QList<Tile *> &tiles);
     void deleteTile(int id);
     QList<int> relocateTiles(const QList<Tile *> &tiles, int location);
 
     bool anyTileOutOfOrder() const;
+    void resetTileOrder();
 
     void setNextTileId(int nextId);
     int nextTileId() const;
@@ -205,6 +238,8 @@ public:
     void setTileImage(Tile *tile,
                       const QPixmap &image,
                       const QUrl &source = QUrl());
+    void setTileImageRect(Tile *tile, const QRect &imageRect);
+
     /**
      * @deprecated Only kept around for the Python API!
      */
@@ -252,12 +287,20 @@ public:
      */
     static Orientation orientationFromString(const QString &);
 
+    static QString tileRenderSizeToString(TileRenderSize tileRenderSize);
+    static TileRenderSize tileRenderSizeFromString(const QString &);
+
+    static QString fillModeToString(FillMode fillMode);
+    static FillMode fillModeFromString(const QString &);
+
 private:
+    void maybeUpdateTileSize(QSize oldSize, QSize newSize);
     void updateTileSize();
 
     QString mName;
     QString mFileName;
     ImageReference mImageReference;
+    QPixmap mImage;
     int mTileWidth;
     int mTileHeight;
     int mTileSpacing;
@@ -265,6 +308,8 @@ private:
     QPoint mTileOffset;
     Alignment mObjectAlignment = Unspecified;
     Orientation mOrientation = Orthogonal;
+    TileRenderSize mTileRenderSize = TileSize;
+    FillMode mFillMode = Stretch;
     QSize mGridSize;
     int mColumnCount = 0;
     int mExpectedColumnCount = 0;
@@ -379,6 +424,26 @@ inline void Tileset::setObjectAlignment(Alignment objectAlignment)
     mObjectAlignment = objectAlignment;
 }
 
+inline Tileset::TileRenderSize Tileset::tileRenderSize() const
+{
+    return mTileRenderSize;
+}
+
+inline void Tileset::setTileRenderSize(TileRenderSize tileRenderSize)
+{
+    mTileRenderSize = tileRenderSize;
+}
+
+inline Tileset::FillMode Tileset::fillMode() const
+{
+    return mFillMode;
+}
+
+inline void Tileset::setFillMode(FillMode fillMode)
+{
+    mFillMode = fillMode;
+}
+
 /**
  * Returns the offset that is applied when drawing the tiles in this
  * tileset.
@@ -569,13 +634,19 @@ inline QString Tileset::imageSourceString() const
     return url.isLocalFile() ? url.toLocalFile() : url.toString();
 }
 
+inline const QPixmap &Tileset::image() const
+{
+    return mImage;
+}
+
 /**
  * Returns whether this tileset is a collection of images. In this case, the
- * tileset itself has no image source.
+ * tileset itself has no image source and the tileset image is also not
+ * embedded.
  */
 inline bool Tileset::isCollection() const
 {
-    return imageSource().isEmpty();
+    return imageSource().isEmpty() && image().isNull();
 }
 
 inline const QList<WangSet*> &Tileset::wangSets() const
@@ -668,5 +739,9 @@ inline void Tileset::setTransformationFlags(TransformationFlags flags)
 
 Q_DECLARE_METATYPE(Tiled::Tileset*)
 Q_DECLARE_METATYPE(Tiled::SharedTileset)
+Q_DECLARE_METATYPE(Tiled::Tileset::Orientation)
+Q_DECLARE_METATYPE(Tiled::Tileset::TileRenderSize)
+Q_DECLARE_METATYPE(Tiled::Tileset::FillMode)
+Q_DECLARE_METATYPE(Tiled::Tileset::TransformationFlags)
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Tiled::Tileset::TransformationFlags)
