@@ -38,10 +38,10 @@ class DROIDCRAFTSHARED_EXPORT DroidcraftPlugin : public Tiled::MapFormat
 public:
     DroidcraftPlugin();
 
-    Tiled::Map *read(const QString &fileName) override;
+    std::unique_ptr<Tiled::Map> read(const QString &fileName) override;
     bool supportsFile(const QString &fileName) const override;
 
-    bool write(const Tiled::Map *map, const QString &fileName) override;
+    bool write(const Tiled::Map *map, const QString &fileName, Options options) override;
     QString nameFilter() const override;
     QString shortName() const override;
     QString errorString() const override;
