@@ -30,11 +30,14 @@ public:
     void setAutoFormattingIndent(int spaceOrTabs);
     int autoFormattingIndent() const;
 
+    void setAutoFormattingWrapArrayCount(int count);
+
 private:
     void stringify(const QVariant &variant, int depth);
 
     QString m_result;
     QString m_errorString;
-    bool m_autoFormatting;
+    bool m_autoFormatting = false;
     QString m_autoFormattingIndent;
+    int m_autoFormattingWrapArrayCount = 0;
 };
