@@ -7,14 +7,14 @@ Note:
 - only supports bg1 of bgmode 1
 http://snesemu.black-ship.net/misc/techdocs/snesdoc.html#GraphicsFormat
 """
-import sys, re, string
-from tiled import *
-from tiled.qt import *
 from os.path import dirname
 from struct import pack,unpack,Struct
 from collections import namedtuple
+import sys, re, string
+import tiled as T
 
-class ZST(Plugin):
+
+class ZST(T.Plugin):
   @classmethod
   def nameFilter(cls):
     return "zSNES Save State (*.zs?)"
@@ -37,7 +37,7 @@ class ZST(Plugin):
         (2, 2, 0, 0, 0, 0, 0, 0),
         (2, 0, 0, 0, 0, 0, 0, 0))
 
-    img = QImage(32*8, 32*8, QImage.Format_Indexed8)
+    img = T.qt.QImage(32*8, 32*8, T.qt.QImage.Format_Indexed8)
     #img.fill(255)
     cmap = []
     with open(f, 'rb') as fh:
@@ -83,7 +83,7 @@ class ZST(Plugin):
             """
             la.setCell(x, y, Tiled.Cell(tile))
           except:
-            print 'out of range %i,%i: %i' % (x,y,t.idx)
+            print('out of range %i,%i: %i' % (x,y,t.idx))
 
       if la.width() > 32:
         for y in range(la.height()):
@@ -93,7 +93,7 @@ class ZST(Plugin):
               tile = tsets[t.pal].data().tileAt(t.idx)
               la.setCell(x, y, Tiled.Cell(tile))
             except:
-              print 'out of range %i,%i: %i' % (x,y,t.idx)
+              print('out of range %i,%i: %i' % (x,y,t.idx))
 
       for pal in range(7):
         m.addTileset(tsets[pal])
@@ -148,7 +148,7 @@ def parseColors(cgram):
     g = ((col >> 5) << 3) & 0xf8
     b = ((col >> 10) << 3) & 0xf8
     #print (r,g,b),
-    yield QColor(r,g,b).rgb()
+    yield T.qt.QColor(r,g,b).rgb()
 
 def parseTile(t):
   ret = namedtuple('Tile', 'idx pal prio flipx flipy')
