@@ -40,8 +40,6 @@
 
 #include <vector>
 
-#include "qtcompat_p.h"
-
 using namespace Tiled;
 using namespace Yy;
 
@@ -687,7 +685,7 @@ static void fillTileLayer(GMRTileLayer &gmrTileLayer, const TileLayer *tileLayer
     gmrTileLayer.SerialiseHeight = tileLayer->height();
     gmrTileLayer.SerialiseWidth = tileLayer->width();
 
-    constexpr unsigned Unintialized         = 0x80000000;
+    constexpr unsigned Uninitialized        = 0x80000000;
     constexpr unsigned FlippedHorizontally  = 0x10000000;
     constexpr unsigned FlippedVertically    = 0x20000000;
     constexpr unsigned Rotated90            = 0x40000000;
@@ -696,7 +694,7 @@ static void fillTileLayer(GMRTileLayer &gmrTileLayer, const TileLayer *tileLayer
         for (int x = 0; x < tileLayer->width(); ++x) {
             const Cell &cell = tileLayer->cellAt(x, y);
             if (cell.tileset() != tileset) {
-                gmrTileLayer.tiles.push_back(Unintialized);
+                gmrTileLayer.tiles.push_back(Uninitialized);
                 continue;
             }
 
@@ -781,8 +779,7 @@ static void createAssetsFromTiles(std::vector<GMRGraphic> &assets,
 
         const bool isSprite = !tile->imageSource().isEmpty();
 
-        assets.emplace_back(isSprite);
-        GMRGraphic &g = assets.back();
+        GMRGraphic &g = assets.emplace_back(isSprite);
 
         QSize size = tile->size();
         QPointF origin(optionalProperty(tile, "originX", 0.0),
@@ -867,7 +864,7 @@ static std::unique_ptr<GMRLayer> processTileLayer(const TileLayer *tileLayer,
             return a->name() < b->name();
         });
 
-        for (const auto &tileset : qAsConst(tilesets)) {
+        for (const auto &tileset : std::as_const(tilesets)) {
             if (tileset->isCollection())
                 continue;
             if (tileset->tileSize() != tileLayer->map()->tileSize())
@@ -928,10 +925,10 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
                          [](const MapObject *a, const MapObject *b) { return a->y() < b->y(); });
     }
 
-    for (const MapObject *mapObject : qAsConst(objects)) {
-        const QString type = mapObject->effectiveType();
+    for (const MapObject *mapObject : std::as_const(objects)) {
+        const QString &className = mapObject->effectiveClassName();
 
-        if (type == QLatin1String("view")) {
+        if (className == QLatin1String("view")) {
             // GM only has 8 views so drop anything more than that
             if (context.views.size() > 7) {
                 Tiled::ERROR(QLatin1String("YY plugin: Can't export more than 8 views."),
@@ -939,8 +936,7 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
                 continue;
             }
 
-            context.views.emplace_back();
-            GMRView &view = context.views.back();
+            GMRView &view = context.views.emplace_back();
 
             view.inherit = optionalProperty(mapObject, "inherit", false);
             view.visible = mapObject->isVisible();
@@ -950,7 +946,7 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
             view.yview = qRound(mapObject->y());
             view.wview = qRound(mapObject->width());
             view.hview = qRound(mapObject->height());
-            // Round these incase user adds properties as floats and not ints
+            // Round these in case user adds properties as floats and not ints
             view.xport = qRound(optionalProperty(mapObject, "xport", 0.0));
             view.yport = qRound(optionalProperty(mapObject, "yport", 0.0));
             view.wport = qRound(optionalProperty(mapObject, "wport", 1024.0));
@@ -961,16 +957,15 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
             view.vspeed = qRound(optionalProperty(mapObject, "vspeed", -1.0));
             view.objectId = optionalProperty(mapObject, "objectId", QString());
         }
-        else if (!type.isEmpty())
+        else if (!className.isEmpty())
         {
-            instances.emplace_back();
-            GMRInstance &instance = instances.back();
+            GMRInstance &instance = instances.emplace_back();
 
             auto props = mapObject->resolvedProperties();
 
             // The type is used to refer to the name of the object
             instance.isDnd = takeProperty(props, "isDnd", instance.isDnd);
-            instance.objectId = sanitizeName(type);
+            instance.objectId = sanitizeName(className);
 
             QPointF origin(takeProperty(props, "originX", 0.0),
                            takeProperty(props, "originY", 0.0));
@@ -1001,17 +996,19 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
                 // Tile objects don't necessarily have top-left origin in Tiled,
                 // so the position needs to be translated for top-left origin in
                 // GameMaker, taking into account the rotation.
-                origin -= alignmentOffset(mapObject->bounds(), mapObject->alignment());
+                origin -= alignmentOffset(mapObject->size(), mapObject->alignment());
             }
 
             // Allow overriding the scale using custom properties
             instance.scaleX = takeProperty(props, "scaleX", instance.scaleX);
             instance.scaleY = takeProperty(props, "scaleY", instance.scaleY);
 
+            QPointF pos = context.renderer->pixelToScreenCoords(mapObject->position());
+
             // Adjust the position based on the origin
             QTransform transform;
             transform.rotate(mapObject->rotation());
-            const QPointF pos = mapObject->position() + transform.map(origin);
+            pos += transform.map(origin);
 
             // TODO: Support creation code - takeProperty(props, "code", QString());
             // Would need to be written out as a separate file
@@ -1031,16 +1028,14 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
             instance.tags = readTags(mapObject);
             props.remove(QStringLiteral("tags"));
 
-            context.instanceCreationOrder.emplace_back();
-            InstanceCreation &instanceCreation = context.instanceCreationOrder.back();
+            InstanceCreation &instanceCreation = context.instanceCreationOrder.emplace_back();
             instanceCreation.name = instance.name;
             instanceCreation.creationOrder = takeProperty(props, "creationOrder", 0);
 
             // Remaining unknown custom properties are assumed to
             // override properties defined on the GameMaker object.
             for (auto it = props.constBegin(); it != props.constEnd(); ++it) {
-                instance.properties.emplace_back();
-                GMOverriddenProperty &prop = instance.properties.back();
+                GMOverriddenProperty &prop = instance.properties.emplace_back();
                 prop.propertyId = it.key();
                 prop.objectId = instance.objectId;
                 prop.value = toOverriddenPropertyValue(it.value(), context);
@@ -1055,8 +1050,7 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
 
             const bool isSprite = !tile->imageSource().isEmpty();
 
-            assets.emplace_back(isSprite);
-            GMRGraphic &g = assets.back();
+            GMRGraphic &g = assets.emplace_back(isSprite);
 
             QPointF origin(optionalProperty(mapObject, "originX", 0.0),
                            optionalProperty(mapObject, "originY", 0.0));
@@ -1099,12 +1093,14 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
             // Tile objects don't necessarily have top-left origin in Tiled,
             // so the position needs to be translated for top-left origin in
             // GameMaker, taking into account the rotation.
-            origin -= alignmentOffset(mapObject->bounds(), mapObject->alignment());
+            origin -= alignmentOffset(mapObject->size(), mapObject->alignment());
+
+            QPointF pos = context.renderer->pixelToScreenCoords(mapObject->position());
 
             // Adjust the position based on the origin
             QTransform transform;
             transform.rotate(mapObject->rotation());
-            const QPointF pos = mapObject->position() + transform.map(origin);
+            pos += transform.map(origin);
 
             g.colour = optionalProperty(mapObject, "colour", color);
             // TODO: g.inheritedItemId
@@ -1123,9 +1119,7 @@ static std::unique_ptr<GMRLayer> processObjectGroup(const ObjectGroup *objectGro
         }
         else if (mapObject->shape() == MapObject::Polygon || mapObject->shape() == MapObject::Polyline)
         {
-            paths.emplace_back();
-            GMPath &p = paths.back();
-
+            GMPath &p = paths.emplace_back();
             p.kind = optionalProperty(mapObject, "smooth", false);
             p.closed = mapObject->shape() == MapObject::Polygon;
             p.precision = optionalProperty(mapObject, "precision", 4);
@@ -1392,7 +1386,7 @@ bool YyPlugin::write(const Map *map, const QString &fileName, Options options)
     // Last view in Object layer is the first view in the room
     json.writeStartArray("views");
     context.views.resize(8);    // GameMaker always stores 8 views
-    for (const GMRView &view : qAsConst(context.views)) {
+    for (const GMRView &view : std::as_const(context.views)) {
         json.prepareNewLine();
         json.writeStartObject();
         const bool wasMinimize = json.minimize();
