@@ -49,10 +49,10 @@ public:
                          const QString &name,
                          const QIcon &icon,
                          const QKeySequence &shortcut,
-                         BrushItem *brushItem = nullptr,
                          QObject *parent = nullptr);
     ~AbstractTileFillTool() override;
 
+    void activate(MapScene *scene) override;
     void deactivate(MapScene *scene) override;
 
     void mousePressed(QGraphicsSceneMouseEvent *event) override;
