@@ -23,6 +23,7 @@
 #include "preferences.h"
 
 #include <QMenu>
+#include <QScopedValueRollback>
 
 namespace Tiled {
 
@@ -53,15 +54,15 @@ void ActionManager::registerAction(QAction *action, Id id)
 
     Q_ASSERT_X(!d->mIdToActions.contains(id, action), "ActionManager::registerAction", "action already registered");
     d->mIdToActions.insert(id, action);
-    d->mLastKnownShortcuts.insert(id, action->shortcut());
+    d->mLastKnownShortcuts.insert(id, action->shortcuts());
 
     connect(action, &QAction::changed, d, [=] {
         if (d->mApplyingToolTipWithShortcut)
             return;
 
-        if (!d->mApplyingShortcut && d->mDefaultShortcuts.contains(id) && d->mLastKnownShortcuts.value(id) != action->shortcut()) {
+        if (!d->mApplyingShortcut && d->mDefaultShortcuts.contains(id) && d->mLastKnownShortcuts.value(id) != action->shortcuts()) {
             // Update remembered default shortcut
-            d->mDefaultShortcuts.insert(id, action->shortcut());
+            d->mDefaultShortcuts.insert(id, action->shortcuts());
 
             // Reset back to user-defined shortcut if set
             if (d->mCustomShortcuts.contains(id)) {
@@ -70,7 +71,7 @@ void ActionManager::registerAction(QAction *action, Id id)
             }
         }
 
-        d->mLastKnownShortcuts.insert(id, action->shortcut());
+        d->mLastKnownShortcuts.insert(id, action->shortcuts());
 
         d->updateToolTipWithShortcut(action);
 
@@ -78,7 +79,7 @@ void ActionManager::registerAction(QAction *action, Id id)
     });
 
     if (d->hasCustomShortcut(id)) {
-        d->mDefaultShortcuts.insert(id, action->shortcut());
+        d->mDefaultShortcuts.insert(id, action->shortcuts());
         d->applyShortcut(action, d->mCustomShortcuts.value(id));
     }
 
@@ -160,6 +161,19 @@ QAction *ActionManager::findAction(Id id)
     return d->mIdToActions.value(id);
 }
 
+QAction *ActionManager::findEnabledAction(Id id)
+{
+    auto d = instance();
+
+    const auto [start, end] = std::as_const(d->mIdToActions).equal_range(id);
+    for (auto it = start; it != end; ++it) {
+        if (it.value()->isEnabled())
+            return it.value();
+    }
+
+    return nullptr;
+}
+
 bool ActionManager::hasMenu(Id id)
 {
     return instance()->mIdToMenu.contains(id);
@@ -183,7 +197,7 @@ void ActionManager::setCustomShortcut(Id id, const QKeySequence &keySequence)
     Q_ASSERT_X(!actions.isEmpty(), "ActionManager::setCustomShortcut", "unknown id");
 
     if (!hasCustomShortcut(id))
-        mDefaultShortcuts.insert(id, actions.first()->shortcut());
+        mDefaultShortcuts.insert(id, actions.first()->shortcuts());
 
     mCustomShortcuts.insert(id, keySequence);
 
@@ -207,26 +221,24 @@ void ActionManager::resetCustomShortcut(Id id)
     const auto actions = mIdToActions.values(id);
     Q_ASSERT_X(!actions.isEmpty(), "ActionManager::resetCustomShortcut", "unknown id");
 
-    mResettingShortcut = true;
+    QScopedValueRollback<bool> resettingShortcut(mResettingShortcut, true);
 
-    const QKeySequence defaultShortcut = mDefaultShortcuts.take(id);
+    const QList<QKeySequence> defaultShortcuts = mDefaultShortcuts.take(id);
     for (QAction *a : actions)
-        applyShortcut(a, defaultShortcut);
+        applyShortcuts(a, defaultShortcuts);
     mCustomShortcuts.remove(id);
 
-    mResettingShortcut = false;
-
     Preferences::instance()->remove(QLatin1String("CustomShortcuts/") + id.toString());
 }
 
 void ActionManager::resetAllCustomShortcuts()
 {
-    QHashIterator<Id, QKeySequence> iterator(mDefaultShortcuts);
+    QHashIterator<Id, QList<QKeySequence>> iterator(mDefaultShortcuts);
     while (iterator.hasNext()) {
         iterator.next();
         const auto actions = mIdToActions.values(iterator.key());
         for (QAction *a : actions)
-            applyShortcut(a, iterator.value());
+            applyShortcuts(a, iterator.value());
     }
     mDefaultShortcuts.clear();
     mCustomShortcuts.clear();
@@ -234,13 +246,13 @@ void ActionManager::resetAllCustomShortcuts()
     Preferences::instance()->remove(QLatin1String("CustomShortcuts"));
 }
 
-QKeySequence ActionManager::defaultShortcut(Id id)
+QList<QKeySequence> ActionManager::defaultShortcuts(Id id) const
 {
     if (mDefaultShortcuts.contains(id))
         return mDefaultShortcuts.value(id);
     if (auto a = findAction(id))
-        return a->shortcut();
-    return QKeySequence();
+        return a->shortcuts();
+    return {};
 }
 
 /**
@@ -248,22 +260,24 @@ QKeySequence ActionManager::defaultShortcut(Id id)
  *
  * Shortcuts that are the same as the default ones will be reset.
  */
-void ActionManager::setCustomShortcuts(const QHash<Id, QKeySequence> &shortcuts)
+void ActionManager::setCustomShortcuts(const QHash<Id, QList<QKeySequence>> &shortcuts)
 {
-    QHashIterator<Id, QKeySequence> iterator(shortcuts);
+    QHashIterator<Id, QList<QKeySequence>> iterator(shortcuts);
     while (iterator.hasNext()) {
         iterator.next();
 
         const Id id = iterator.key();
-        const QKeySequence &shortcut = iterator.value();
+        const QList<QKeySequence> &shortcuts = iterator.value();
 
         if (auto a = findAction(id)) {
-            if (mDefaultShortcuts.contains(id)
-                    ? mDefaultShortcuts.value(id) == shortcut
-                    : a->shortcut() == shortcut) {
+            const auto defaultShortcuts = mDefaultShortcuts.contains(id) ?
+                        mDefaultShortcuts.value(id) : a->shortcuts();
+
+            if (defaultShortcuts == shortcuts) {
                 resetCustomShortcut(id);
             } else {
-                setCustomShortcut(id, shortcut);
+                setCustomShortcut(id, shortcuts.isEmpty() ? QKeySequence()
+                                                          : shortcuts.first());
             }
         }
     }
@@ -285,14 +299,20 @@ void ActionManager::readCustomShortcuts()
 
 void ActionManager::applyShortcut(QAction *action, const QKeySequence &shortcut)
 {
-    mApplyingShortcut = true;
+    QScopedValueRollback<bool> applyingShortcut(mApplyingShortcut, true);
     action->setShortcut(shortcut);
+}
+
+void ActionManager::applyShortcuts(QAction *action, const QList<QKeySequence> &shortcuts)
+{
+    QScopedValueRollback<bool> applyingShortcut(mApplyingShortcut, true);
+    action->setShortcuts(shortcuts);
     mApplyingShortcut = false;
 }
 
 void ActionManager::updateToolTipWithShortcut(QAction *action)
 {
-    mApplyingToolTipWithShortcut = true;
+    QScopedValueRollback<bool> applyingToolTipWithShortcut(mApplyingToolTipWithShortcut, true);
 
     QString toolTip = action->toolTip();
 
@@ -308,8 +328,6 @@ void ActionManager::updateToolTipWithShortcut(QAction *action)
     }
 
     action->setToolTip(toolTip);
-
-    mApplyingToolTipWithShortcut = false;
 }
 
 void ActionManager::applyMenuExtension(QMenu *menu, const ActionManager::MenuExtension &extension)
