@@ -54,10 +54,6 @@ AddPropertyDialog::AddPropertyDialog(const ClassPropertyType *parentClassType, Q
 
 void AddPropertyDialog::initialize(const Tiled::ClassPropertyType *parentClassType)
 {
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
-#endif
-
     mUi->setupUi(this);
     resize(Utils::dpiScaled(size()));
 
@@ -77,6 +73,11 @@ void AddPropertyDialog::initialize(const Tiled::ClassPropertyType *parentClassTy
         if (parentClassType && !parentClassType->canAddMemberOfType(propertyType))
             continue;
 
+        // Avoid suggesting classes not meant to be used as property value
+        if (propertyType->isClass())
+            if (!static_cast<const ClassPropertyType*>(propertyType)->isPropertyValueType())
+                continue;
+
         const QVariant var = propertyType->wrap(propertyType->defaultValue());
         const QIcon icon = PropertyTypesModel::iconForPropertyType(propertyType->type);
         mUi->typeBox->addItem(icon, propertyType->name, var);
@@ -91,6 +92,8 @@ void AddPropertyDialog::initialize(const Tiled::ClassPropertyType *parentClassTy
             this, &AddPropertyDialog::nameChanged);
     connect(mUi->typeBox, &QComboBox::currentTextChanged,
             this, &AddPropertyDialog::typeChanged);
+
+    mUi->name->setFocus();
 }
 
 AddPropertyDialog::~AddPropertyDialog()
