@@ -24,6 +24,10 @@
 #include <QDialog>
 #include <QVariant>
 
+namespace Tiled {
+class ClassPropertyType;
+}
+
 namespace Ui {
 class AddPropertyDialog;
 }
@@ -34,12 +38,15 @@ class AddPropertyDialog : public QDialog
 
 public:
     explicit AddPropertyDialog(QWidget *parent = nullptr);
-    ~AddPropertyDialog();
+    AddPropertyDialog(const Tiled::ClassPropertyType *parentClassType, QWidget *parent = nullptr);
+    ~AddPropertyDialog() override;
 
     QString propertyName() const;
     QVariant propertyValue() const;
 
 private:
+    void initialize(const Tiled::ClassPropertyType *parentClassType);
+
     void nameChanged(const QString &text);
     void typeChanged(const QString &text);
 
