@@ -25,7 +25,6 @@
 #include "changetile.h"
 #include "changetileanimation.h"
 #include "changetileobjectgroup.h"
-#include "changetileprobability.h"
 #include "changetilewangid.h"
 #include "changewangcolordata.h"
 #include "changewangsetdata.h"
@@ -81,34 +80,22 @@ AdjustTileIndexes::AdjustTileIndexes(MapDocument *mapDocument,
             const QRegion region = tileLayer->region(isFromTileset).translated(-layer->position());
 
             if (!region.isEmpty()) {
-                const QRect boundingRect(region.boundingRect());
-                auto changedLayer = new TileLayer(QString(), 0, 0,
-                                                  boundingRect.width(),
-                                                  boundingRect.height());
+                TileLayer adjustedTileLayer;
 
-#if QT_VERSION < 0x050800
-                const auto rects = region.rects();
-                for (const QRect &rect : rects) {
-#else
                 for (const QRect &rect : region) {
-#endif
                     for (int x = rect.left(); x <= rect.right(); ++x) {
                         for (int y = rect.top(); y <= rect.bottom(); ++y) {
-                            Cell cell = adjustCell(tileLayer->cellAt(x, y));
-                            changedLayer->setCell(x - boundingRect.x(),
-                                                  y - boundingRect.y(),
-                                                  cell);
+                            const Cell cell = adjustCell(tileLayer->cellAt(x, y));
+                            adjustedTileLayer.setCell(x, y, cell);
                         }
                     }
                 }
 
                 new PaintTileLayer(mapDocument, tileLayer,
-                                   boundingRect.x() + tileLayer->x(),
-                                   boundingRect.y() + tileLayer->y(),
-                                   changedLayer,
+                                   0, 0,
+                                   &adjustedTileLayer,
+                                   region.translated(tileLayer->position()),
                                    this);
-
-                delete changedLayer;
             }
 
             break;
@@ -159,9 +146,9 @@ AdjustTileMetaData::AdjustTileMetaData(TilesetDocument *tilesetDocument)
     };
 
     // Adjust tile meta data
-    QMap<QString, QList<Tile*>> tilesChangingTypeByType;
+    QMap<QString, QList<Object*>> tilesChangingClassByClass;
     QList<Tile*> tilesChangingProbability;
-    QList<qreal> tileProbabilities;
+    QVector<qreal> tileProbabilities;
     QSet<Tile*> tilesToReset;
 
     auto adjustAnimationFrames = [&](const QVector<Frame> &frames) -> QVector<Frame> {
@@ -177,7 +164,7 @@ AdjustTileMetaData::AdjustTileMetaData(TilesetDocument *tilesetDocument)
 
     auto applyMetaData = [&](Tile *toTile,
                              const Properties &properties,
-                             const QString &type,
+                             const QString &className,
                              qreal probability,
                              std::unique_ptr<ObjectGroup> objectGroup,
                              const QVector<Frame> &frames)
@@ -190,8 +177,8 @@ AdjustTileMetaData::AdjustTileMetaData(TilesetDocument *tilesetDocument)
                                  this);
         }
 
-        if (type != toTile->type())
-            tilesChangingTypeByType[type].append(toTile);
+        if (className != toTile->className())
+            tilesChangingClassByClass[className].append(toTile);
 
         if (probability != toTile->probability()) {
             tilesChangingProbability.append(toTile);
@@ -225,7 +212,7 @@ AdjustTileMetaData::AdjustTileMetaData(TilesetDocument *tilesetDocument)
 
         applyMetaData(toTile,
                       fromTile->properties(),
-                      fromTile->type(),
+                      fromTile->className(),
                       fromTile->probability(),
                       std::move(objectGroup),
                       adjustAnimationFrames(fromTile->frames()));
@@ -304,10 +291,10 @@ AdjustTileMetaData::AdjustTileMetaData(TilesetDocument *tilesetDocument)
             new ChangeTileWangId(tilesetDocument, wangSet, changes, this);
     }
 
-    QMapIterator<QString, QList<Tile*>> it(tilesChangingTypeByType);
+    QMapIterator<QString, QList<Object*>> it(tilesChangingClassByClass);
     while (it.hasNext()) {
         it.next();
-        new ChangeTileType(tilesetDocument, it.value(), it.key(), this);
+        new ChangeClassName(tilesetDocument, it.value(), it.key(), this);
     }
 
     if (!tilesChangingProbability.isEmpty()) {
