@@ -1,6 +1,7 @@
 /*
  * automappingmanager.h
- * Copyright 2010-2011, Stefan Beller, stefanbeller@googlemail.com
+ * Copyright 2010-2012, Stefan Beller <stefanbeller@googlemail.com>
+ * Copyright 2013-2022, Thorbjørn Lindeijer <bjorn@lindeijer.nl>
  *
  * This file is part of Tiled.
  *
@@ -22,10 +23,11 @@
 
 #include "session.h"
 
+#include <QFileSystemWatcher>
 #include <QObject>
 #include <QRegion>
+#include <QRegularExpression>
 #include <QString>
-#include <QFileSystemWatcher>
 
 #include <memory>
 #include <vector>
@@ -131,6 +133,7 @@ private:
     QFileSystemWatcher mWatcher;
 
     QString mRulesFile;
+    QRegularExpression mMapNameFilter;
     bool mRulesFileOverride = false;
 };
 
