@@ -23,6 +23,7 @@
 #include "imagelayer.h"
 #include "map.h"
 #include "mapobject.h"
+#include "tileset.h"
 #include "wangset.h"
 
 #include <QList>
@@ -36,6 +37,9 @@ class ChangeEvent
 {
 public:
     enum Type {
+        DocumentAboutToReload,
+        DocumentReloaded,
+        ObjectsChanged,
         MapChanged,
         LayerChanged,
         TileLayerChanged,
@@ -50,12 +54,14 @@ public:
         MapObjectsRemoved,
         ObjectGroupChanged,
         TilesAboutToBeRemoved,
+        TilesetChanged,
         WangSetAboutToBeAdded,
         WangSetAboutToBeRemoved,
         WangSetAdded,
         WangSetRemoved,
         WangSetChanged,
         WangColorAboutToBeRemoved,
+        WangColorChanged,
     } type;
 
 protected:
@@ -68,6 +74,39 @@ protected:
     {}
 };
 
+class AboutToReloadEvent : public ChangeEvent
+{
+public:
+    AboutToReloadEvent()
+        : ChangeEvent(DocumentAboutToReload)
+    {}
+};
+
+class ReloadEvent : public ChangeEvent
+{
+public:
+    ReloadEvent()
+        : ChangeEvent(DocumentReloaded)
+    {}
+};
+
+class ObjectsChangeEvent : public ChangeEvent
+{
+public:
+    enum ObjectProperty {
+        ClassProperty           = 1 << 0,
+    };
+
+    ObjectsChangeEvent(const QList<Object *> &objects, int properties)
+        : ChangeEvent(ObjectsChanged)
+        , objects(objects)
+        , properties(properties)
+    {}
+
+    QList<Object *> objects;
+    int properties;
+};
+
 class MapChangeEvent : public ChangeEvent
 {
 public:
@@ -126,7 +165,7 @@ public:
 class ImageLayerChangeEvent : public LayerChangeEvent
 {
 public:
-    enum TileLayerProperty {
+    enum ImageLayerProperty {
         TransparentColorProperty    = 1 << 7,
         ImageSourceProperty         = 1 << 8,
         RepeatProperty              = 1 << 9,
@@ -198,6 +237,19 @@ public:
     int index;
 };
 
+class TilesetChangeEvent : public ChangeEvent
+{
+public:
+    TilesetChangeEvent(Tileset *tileset, Tileset::Property property)
+        : ChangeEvent(TilesetChanged)
+        , tileset(tileset)
+        , property(property)
+    {}
+
+    Tileset *tileset;
+    Tileset::Property property;
+};
+
 class TilesEvent : public ChangeEvent
 {
 public:
@@ -226,17 +278,20 @@ class WangSetChangeEvent : public ChangeEvent
 {
 public:
     enum WangSetProperty {
-        TypeProperty            = 1 << 0,
+        NameProperty,
+        TypeProperty,
+        ImageProperty,
+        ColorCountProperty,
     };
 
-    WangSetChangeEvent(WangSet *wangSet, int properties)
+    WangSetChangeEvent(WangSet *wangSet, WangSetProperty property)
         : ChangeEvent(WangSetChanged)
         , wangSet(wangSet)
-        , properties(properties)
+        , property(property)
     {}
 
     WangSet *wangSet;
-    int properties;
+    WangSetProperty property;
 };
 
 class WangColorEvent : public ChangeEvent
@@ -252,4 +307,24 @@ public:
     int color;
 };
 
+class WangColorChangeEvent : public ChangeEvent
+{
+public:
+    enum WangColorProperty {
+        NameProperty,
+        ColorProperty,
+        ImageProperty,
+        ProbabilityProperty,
+    };
+
+    WangColorChangeEvent(WangColor *wangColor, WangColorProperty property)
+        : ChangeEvent(WangColorChanged)
+        , wangColor(wangColor)
+        , property(property)
+    {}
+
+    WangColor *wangColor;
+    WangColorProperty property;
+};
+
 } // namespace Tiled
