@@ -28,7 +28,6 @@
 
 #include "addremovetileset.h"
 #include "changeevents.h"
-#include "qtcompat_p.h"
 
 using namespace Tiled;
 
@@ -61,9 +60,7 @@ bool ChangeMapObject::mergeWith(const QUndoCommand *other)
 {
     auto o = static_cast<const ChangeMapObject*>(other);
     if (mDocument == o->mDocument && mMapObject == o->mMapObject && mProperty == o->mProperty) {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
         setObsolete(mMapObject->mapObjectProperty(mProperty) == mValue);
-#endif
         return true;
     }
     return false;
@@ -103,9 +100,7 @@ static QList<MapObject*> objectList(const QVector<MapObjectCell> &changes)
 
 void ChangeMapObjectCells::swap()
 {
-    for (int i = 0; i < mChanges.size(); ++i) {
-        MapObjectCell &change = mChanges[i];
-
+    for (auto &change : mChanges) {
         auto cell = change.object->cell();
         change.object->setCell(change.cell);
         change.cell = cell;
@@ -129,7 +124,7 @@ ChangeMapObjectsTile::ChangeMapObjectsTile(Document *document,
     , mMapObjects(mapObjects)
     , mTile(tile)
 {
-    for (MapObject *object : qAsConst(mMapObjects)) {
+    for (MapObject *object : std::as_const(mMapObjects)) {
         Cell cell = object->cell();
         mOldCells.append(cell);
         Tile *tile = cell.tile();
@@ -207,6 +202,7 @@ DetachObjects::DetachObjects(Document *document,
 {
     for (const MapObject *object : mapObjects) {
         mObjectTemplates.append(object->objectTemplate());
+        mClassNames.append(object->className());
         mProperties.append(object->properties());
     }
 }
@@ -215,7 +211,7 @@ void DetachObjects::redo()
 {
     QUndoCommand::redo(); // redo child commands
 
-    for (MapObject *object : qAsConst(mMapObjects))
+    for (MapObject *object : std::as_const(mMapObjects))
         object->detachFromTemplate();
 
     emit mDocument->changed(MapObjectsChangeEvent(mMapObjects, MapObject::TemplateProperty));
@@ -226,6 +222,7 @@ void DetachObjects::undo()
     for (int i = 0; i < mMapObjects.size(); ++i) {
         MapObject *object = mMapObjects.at(i);
         object->setObjectTemplate(mObjectTemplates.at(i));
+        object->setClassName(mClassNames.at(i));
         object->setProperties(mProperties.at(i));
         object->syncWithTemplate();
     }
