@@ -28,6 +28,7 @@
 #include "mapscene.h"
 #include "mapview.h"
 #include "objectgroup.h"
+#include "objectreferenceshelper.h"
 #include "snaphelper.h"
 #include "tile.h"
 #include "tilelayer.h"
@@ -35,15 +36,14 @@
 
 #include <QApplication>
 #include <QClipboard>
+#include <QHash>
 #include <QJsonDocument>
 #include <QMimeData>
 #include <QSet>
 #include <QUndoStack>
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
 #include <QCborArray>
 #include <QCborValue>
-#endif
 
 #include <algorithm>
 
@@ -233,16 +233,23 @@ void ClipboardManager::pasteObjectGroup(const ObjectGroup *objectGroup,
     QVector<AddMapObjects::Entry> objectsToAdd;
     objectsToAdd.reserve(objectGroup->objectCount());
 
+    Map *map = mapDocument->map();
+    ObjectReferencesHelper objectRefs(map);
+
     for (const MapObject *mapObject : objectGroup->objects()) {
         if (flags & PasteNoTileObjects && !mapObject->cell().isEmpty())
             continue;
 
         MapObject *objectClone = mapObject->clone();
-        objectClone->resetId();
         objectClone->setPosition(objectClone->position() + insertPos);
+
+        objectRefs.reassignId(objectClone);
+
         objectsToAdd.append(AddMapObjects::Entry { objectClone, currentObjectGroup });
     }
 
+    objectRefs.rewire();
+
     auto command = new AddMapObjects(mapDocument, objectsToAdd);
     command->setText(tr("Paste Objects"));
 
