@@ -23,7 +23,6 @@
 #include "createobjecttool.h"
 
 namespace Tiled {
-namespace Internal {
 
 class CreatePointObjectTool : public CreateObjectTool
 {
@@ -35,17 +34,10 @@ public:
     void languageChanged() override;
 
 protected:
-    void mouseMovedWhileCreatingObject(const QPointF &pos,
-                                       Qt::KeyboardModifiers modifiers) override;
-    void mousePressedWhileCreatingObject(QGraphicsSceneMouseEvent *event) override;
-    void mouseReleasedWhileCreatingObject(QGraphicsSceneMouseEvent *event) override;
-
     MapObject *createNewMapObject() override;
-    bool startNewMapObject(const QPointF &pos, ObjectGroup *objectGroup) override;
 
 private:
     void languageChangedImpl();
 };
 
-} // namespace Internal
 } // namespace Tiled
