@@ -46,7 +46,7 @@
 #include "tilesetmanager.h"
 #include "tmxmapformat.h"
 #include "utils.h"
-#include "wangset.h"
+#include "world.h"
 #include "worlddocument.h"
 #include "worldmanager.h"
 #include "zoomable.h"
@@ -62,13 +62,10 @@
 #include <QScrollBar>
 #include <QStackedLayout>
 #include <QTabBar>
-#include <QTabWidget>
 #include <QUndoGroup>
 #include <QUndoStack>
 #include <QVBoxLayout>
 
-#include "qtcompat_p.h"
-
 using namespace Tiled;
 
 
@@ -87,6 +84,7 @@ DocumentManager *DocumentManager::maybeInstance()
 
 DocumentManager::DocumentManager(QObject *parent)
     : QObject(parent)
+    , mLockedIcon(QLatin1String(":images/16/locked.png"))
     , mTilesetDocumentsModel(new TilesetDocumentsModel(this))
     , mWidget(new QWidget)
     , mNoEditorWidget(new NoEditorWidget(mWidget))
@@ -102,10 +100,13 @@ DocumentManager::DocumentManager(QObject *parent)
     Q_ASSERT(!mInstance);
     mInstance = this;
 
+    mLockedIcon.addFile(QStringLiteral(":images/24/locked.png"));
+
     mBrokenLinksWidget->setVisible(false);
 
     mTabBar->setExpanding(false);
     mTabBar->setDocumentMode(true);
+    mTabBar->setUsesScrollButtons(true);    // defaults to false on macOS
     mTabBar->setTabsClosable(true);
     mTabBar->setMovable(true);
     mTabBar->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -115,6 +116,9 @@ DocumentManager::DocumentManager(QObject *parent)
     connect(mFileChangedWarning, &FileChangedWarning::reload, this, &DocumentManager::reloadCurrentDocument);
     connect(mFileChangedWarning, &FileChangedWarning::ignore, this, &DocumentManager::hideChangedWarning);
 
+    connect(this, &DocumentManager::templateTilesetReplaced,
+            mBrokenLinksModel, &BrokenLinksModel::refresh);
+
     QVBoxLayout *vertical = new QVBoxLayout(mWidget);
     vertical->addWidget(mTabBar);
     vertical->addWidget(mFileChangedWarning);
@@ -169,7 +173,7 @@ DocumentManager::DocumentManager(QObject *parent)
     JumpToObject::activated = [this] (const JumpToObject &jump) {
         if (auto mapDocument = openMapFile(jump.mapFile)) {
             if (auto object = mapDocument->map()->findObjectById(jump.objectId)) {
-                mapDocument->focusMapObjectRequested(object);
+                emit mapDocument->focusMapObjectRequested(object);
                 mapDocument->setSelectedObjects({ object });
             }
         }
@@ -205,7 +209,7 @@ DocumentManager::DocumentManager(QObject *parent)
                 break;
             case Object::MapObjectType:
                 if (auto object = mapDocument->map()->findObjectById(select.id)) {
-                    mapDocument->focusMapObjectRequested(object);
+                    emit mapDocument->focusMapObjectRequested(object);
                     mapDocument->setSelectedObjects({ object });
                     obj = object;
                 }
@@ -213,10 +217,6 @@ DocumentManager::DocumentManager(QObject *parent)
             case Object::MapType:
                 obj = mapDocument->map();
                 break;
-            case Object::ObjectTemplateType:
-                emit templateOpenRequested(select.fileName);
-                // todo: can't access Object pointer
-                break;
             }
             break;
         }
@@ -244,14 +244,11 @@ DocumentManager::DocumentManager(QObject *parent)
             case Object::WangColorType:
                 // todo: can't select just by color index
                 break;
-            case Object::ObjectTemplateType:
-                emit templateOpenRequested(select.fileName);
-                // todo: can't access Object pointer
-                break;
             }
             break;
         }
         case Document::WorldDocumentType:
+        case Document::ProjectDocumentType:
             break;
         }
 
@@ -283,7 +280,9 @@ DocumentManager::DocumentManager(QObject *parent)
         }
     };
 
-    WorldManager& worldManager = WorldManager::instance();
+    WorldManager &worldManager = WorldManager::instance();
+    connect(&worldManager, &WorldManager::worldLoaded,
+            this, &DocumentManager::onWorldLoaded);
     connect(&worldManager, &WorldManager::worldUnloaded,
             this, &DocumentManager::onWorldUnloaded);
 }
@@ -357,7 +356,7 @@ void DocumentManager::restoreState()
 }
 
 /**
- * Returns the current map document, or 0 when there is none.
+ * Returns the current document, or nullptr when there is none.
  */
 Document *DocumentManager::currentDocument() const
 {
@@ -386,7 +385,7 @@ MapView *DocumentManager::viewForDocument(MapDocument *mapDocument) const
 }
 
 /**
- * Searches for a document with the given \a fileName and returns its
+ * Searches for an open document with the given \a fileName and returns its
  * index. Returns -1 when the document isn't open.
  */
 int DocumentManager::findDocument(const QString &fileName) const
@@ -428,7 +427,7 @@ bool DocumentManager::switchToDocument(const QString &fileName)
 
 /**
  * Switches to the given \a document, if there is already a tab open for it.
- * \return whether the switch was succesful
+ * \return whether the switch was successful
  */
 bool DocumentManager::switchToDocument(Document *document)
 {
@@ -457,7 +456,7 @@ void DocumentManager::switchToDocument(MapDocument *mapDocument, QPointF viewCen
 }
 
 /**
- * Switches to the given \a mapDocument, taking tilesets into accout
+ * Switches to the given \a mapDocument, taking tilesets into account
  */
 void DocumentManager::switchToDocumentAndHandleSimiliarTileset(MapDocument *mapDocument, QPointF viewCenter, qreal scale)
 {
@@ -538,14 +537,19 @@ void DocumentManager::saveFile()
 }
 
 /**
- * Adds the new or opened \a document to the document manager.
+ * Adds the new or opened \a document to the document manager and makes sure
+ * it is the current document.
  */
 void DocumentManager::addDocument(const DocumentPtr &document)
 {
-    insertDocument(mDocuments.size(), document);
+    const int index = insertDocument(mDocuments.size(), document);
+    switchToDocument(index);
+
+    if (mBrokenLinksModel->hasBrokenLinks())
+        mBrokenLinksWidget->show();
 }
 
-void DocumentManager::insertDocument(int index, const DocumentPtr &document)
+int DocumentManager::insertDocument(int index, const DocumentPtr &document)
 {
     Q_ASSERT(document);
     Q_ASSERT(!mDocuments.contains(document));
@@ -566,23 +570,21 @@ void DocumentManager::insertDocument(int index, const DocumentPtr &document)
         }
     }
 
-    if (!document->fileName().isEmpty())
-        mFileSystemWatcher->addPath(document->fileName());
-
     if (Editor *editor = mEditorForType.value(document->type()))
         editor->addDocument(documentPtr);
 
-    QString tabText = document->displayName();
-    if (document->isModified())
-        tabText.prepend(QLatin1Char('*'));
-
-    const int documentIndex = mTabBar->insertTab(index, tabText);
-    mTabBar->setTabToolTip(documentIndex, document->fileName());
-
+    // Connect before adding the tab, so that we handle the 'changed' signal
+    // first, since we may be creating TilesetDocument instances for tilesets
+    // used by a reloaded map (design not ideal...).
     connect(documentPtr, &Document::fileNameChanged, this, &DocumentManager::fileNameChanged);
     connect(documentPtr, &Document::modifiedChanged, this, [=] { updateDocumentTab(documentPtr); });
+    connect(documentPtr, &Document::isReadOnlyChanged, this, [=] { updateDocumentTab(documentPtr); });
+    connect(documentPtr, &Document::changed, this, &DocumentManager::onDocumentChanged);
     connect(documentPtr, &Document::saved, this, &DocumentManager::onDocumentSaved);
 
+    mTabBar->insertTab(index, QString());
+    updateDocumentTab(documentPtr);
+
     if (auto *mapDocument = qobject_cast<MapDocument*>(documentPtr)) {
         connect(mapDocument, &MapDocument::tilesetAdded, this, &DocumentManager::tilesetAdded);
         connect(mapDocument, &MapDocument::tilesetRemoved, this, &DocumentManager::tilesetRemoved);
@@ -591,12 +593,9 @@ void DocumentManager::insertDocument(int index, const DocumentPtr &document)
     if (auto *tilesetDocument = qobject_cast<TilesetDocument*>(documentPtr))
         connect(tilesetDocument, &TilesetDocument::tilesetNameChanged, this, &DocumentManager::tilesetNameChanged);
 
-    switchToDocument(documentIndex);
-
-    if (mBrokenLinksModel->hasBrokenLinks())
-        mBrokenLinksWidget->show();
-
     emit documentOpened(documentPtr);
+
+    return index;
 }
 
 /**
@@ -637,7 +636,7 @@ DocumentPtr DocumentManager::loadDocument(const QString &fileName,
 {
     // Try to find it in already loaded documents
     QString canonicalFilePath = QFileInfo(fileName).canonicalFilePath();
-    if (Document *doc = Document::documentInstances().value(canonicalFilePath))
+    if (Document *doc = mDocumentByFileName.value(canonicalFilePath))
         return doc->sharedFromThis();
 
     if (!fileFormat) {
@@ -829,7 +828,7 @@ void DocumentManager::closeOtherDocuments(int index)
 
     for (int i = mTabBar->count() - 1; i >= 0; --i) {
         if (i != index)
-            documentCloseRequested(i);
+            emit documentCloseRequested(i);
 
         if (!mMultiDocumentClose)
             return;
@@ -847,7 +846,7 @@ void DocumentManager::closeDocumentsToRight(int index)
     mMultiDocumentClose = true;
 
     for (int i = mTabBar->count() - 1; i > index; --i) {
-        documentCloseRequested(i);
+        emit documentCloseRequested(i);
 
         if (!mMultiDocumentClose)
             return;
@@ -869,14 +868,11 @@ void DocumentManager::closeDocumentAt(int index)
     mDocuments.removeAt(index);
     mTabBar->removeTab(index);
 
+    document->disconnect(this);
+
     if (Editor *editor = mEditorForType.value(document->type()))
         editor->removeDocument(document.data());
 
-    if (!document->fileName().isEmpty()) {
-        mFileSystemWatcher->removePath(document->fileName());
-        document->setChangedOnDisk(false);
-    }
-
     if (auto mapDocument = qobject_cast<MapDocument*>(document.data())) {
         for (const SharedTileset &tileset : mapDocument->map()->tilesets())
             removeFromTilesetDocument(tileset, mapDocument);
@@ -884,8 +880,6 @@ void DocumentManager::closeDocumentAt(int index)
         if (tilesetDocument->mapDocuments().isEmpty()) {
             mTilesetDocumentsModel->remove(tilesetDocument);
             emit tilesetDocumentRemoved(tilesetDocument);
-        } else {
-            tilesetDocument->disconnect(this);
         }
     }
 
@@ -909,57 +903,82 @@ bool DocumentManager::reloadCurrentDocument()
 }
 
 /**
- * Reloads the document at the given \a index. It will lose any undo
- * history and current selections. Will not ask the user whether to save
- * any changes!
+ * Reloads the document at the given \a index. Will not ask the user whether to
+ * save any changes!
  *
- * Returns whether the document loaded successfully.
+ * Returns whether the document reloaded successfully.
  */
 bool DocumentManager::reloadDocumentAt(int index)
 {
-    const auto oldDocument = mDocuments.at(index);
+    const auto document = mDocuments.at(index);
+    return reloadDocument(document.data());
+}
+
+/**
+ * Reloads the given \a document.
+ *
+ * The document may not actually be open in any editor. It might be a map that
+ * is loaded as part of a world, or a tileset that is loaded as part of a map.
+ *
+ * Returns whether the document reloaded successfully.
+ */
+bool DocumentManager::reloadDocument(Document *document)
+{
     QString error;
 
-    if (auto mapDocument = oldDocument.objectCast<MapDocument>()) {
-        auto readerFormat = mapDocument->readerFormat();
-        if (!readerFormat)
-            return false;
-
-        // TODO: Consider fixing the reload to avoid recreating the MapDocument
-        auto newDocument = MapDocument::load(oldDocument->fileName(),
-                                             readerFormat,
-                                             &error);
-        if (!newDocument) {
-            emit reloadError(tr("%1:\n\n%2").arg(oldDocument->fileName(), error));
+    switch (document->type()) {
+    case Document::MapDocumentType: {
+        auto mapDocument = static_cast<MapDocument*>(document);
+        if (!mapDocument->reload(&error)) {
+            emit reloadError(tr("%1:\n\n%2").arg(document->fileName(), error));
             return false;
         }
 
-        // Save the document state, to ensure the new document will match it
-        static_cast<MapEditor*>(editor(Document::MapDocumentType))->saveDocumentState(mapDocument.data());
+        const bool isCurrent = document == currentDocument();
+        if (isCurrent) {
+            if (mBrokenLinksModel->hasBrokenLinks())
+                mBrokenLinksWidget->show();
+        }
 
-        // Replace old tab
-        insertDocument(index, newDocument); // also selects the new document
-        closeDocumentAt(index + 1);
+        // Only check tileset columns for open maps since for other maps we
+        // may not have TilesetDocument instances created for their tilesets.
+        if (findDocument(document) != -1)
+            checkTilesetColumns(mapDocument);
 
-        checkTilesetColumns(newDocument.data());
-
-    } else if (auto tilesetDocument = qobject_cast<TilesetDocument*>(oldDocument)) {
+        break;
+    }
+    case Document::TilesetDocumentType: {
+        auto tilesetDocument = static_cast<TilesetDocument*>(document);
         if (tilesetDocument->isEmbedded()) {
             // For embedded tilesets, we need to reload the map
-            index = findDocument(tilesetDocument->mapDocuments().first());
-            if (!reloadDocumentAt(index))
+            if (!reloadDocument(tilesetDocument->mapDocuments().first()))
                 return false;
         } else if (!tilesetDocument->reload(&error)) {
-            emit reloadError(tr("%1:\n\n%2").arg(oldDocument->fileName(), error));
+            emit reloadError(tr("%1:\n\n%2").arg(document->fileName(), error));
             return false;
         }
 
         tilesetDocument->setChangedOnDisk(false);
+        break;
+    }
+    case Document::WorldDocumentType: {
+        auto worldDocument = static_cast<WorldDocument*>(document);
+        if (!worldDocument->reload(&error)) {
+            emit reloadError(tr("%1:\n\n%2").arg(document->fileName(), error));
+            return false;
+        }
+        break;
+    }
+    case Document::ProjectDocumentType:
+        // We don't support reloading the project at the moment
+        break;
     }
 
     if (!isDocumentChangedOnDisk(currentDocument()))
         mFileChangedWarning->setVisible(false);
 
+    emit documentReloaded(document);
+
     return true;
 }
 
@@ -993,16 +1012,12 @@ void DocumentManager::currentIndexChanged()
     emit currentDocumentChanged(document);
 }
 
-void DocumentManager::fileNameChanged(const QString &fileName,
-                                      const QString &oldFileName)
+void DocumentManager::fileNameChanged(const QString &/* fileName */,
+                                      const QString &/* oldFileName */)
 {
-    if (!fileName.isEmpty())
-        mFileSystemWatcher->addPath(fileName);
-    if (!oldFileName.isEmpty())
-        mFileSystemWatcher->removePath(oldFileName);
+    Document *document = static_cast<Document*>(sender());
 
     // Update the tabs for all opened embedded tilesets
-    Document *document = static_cast<Document*>(sender());
     if (MapDocument *mapDocument = qobject_cast<MapDocument*>(document)) {
         for (const SharedTileset &tileset : mapDocument->map()->tilesets()) {
             if (auto tilesetDocument = findTilesetDocument(tileset))
@@ -1019,12 +1034,39 @@ void DocumentManager::updateDocumentTab(Document *document)
     if (index == -1)
         return;
 
+    QIcon tabIcon = document->isReadOnly() ? mLockedIcon : QIcon();
     QString tabText = document->displayName();
+    QString tabToolTip = document->fileName();
+
     if (document->isModified())
         tabText.prepend(QLatin1Char('*'));
+    if (document->isReadOnly())
+        tabToolTip = tr("%1 [read-only]").arg(tabToolTip);
 
+    mTabBar->setTabIcon(index, tabIcon);
     mTabBar->setTabText(index, tabText);
-    mTabBar->setTabToolTip(index, document->fileName());
+    mTabBar->setTabToolTip(index, tabToolTip);
+}
+
+void DocumentManager::onDocumentChanged(const ChangeEvent &event)
+{
+    auto mapDocument = qobject_cast<MapDocument*>(sender());
+    if (!mapDocument)
+        return;
+
+    // In case a map is reloaded, the set of used tilesets might have changed
+    switch (event.type) {
+    case ChangeEvent::DocumentAboutToReload:
+        for (const SharedTileset &tileset : mapDocument->map()->tilesets())
+            removeFromTilesetDocument(tileset, mapDocument);
+        break;
+    case ChangeEvent::DocumentReloaded:
+        for (const SharedTileset &tileset : mapDocument->map()->tilesets())
+            addToTilesetDocument(tileset, mapDocument);
+        break;
+    default:
+        break;
+    }
 }
 
 void DocumentManager::onDocumentSaved()
@@ -1107,21 +1149,24 @@ void DocumentManager::filesChanged(const QStringList &fileNames)
 
 void DocumentManager::fileChanged(const QString &fileName)
 {
-    const int index = findDocument(fileName);
-
-    // Most likely the file was removed
-    if (index == -1)
+    const auto document = mDocumentByFileName.value(fileName);
+    if (!document) {
+        qWarning() << "Document not found for changed file:" << fileName;
         return;
+    }
 
-    const auto &document = mDocuments.at(index);
+    const QFileInfo fileInfo { fileName };
+
+    // Always update potentially changed read-only state
+    document->setReadOnly(fileInfo.exists() && !fileInfo.isWritable());
 
     // Ignore change event when it seems to be our own save
-    if (QFileInfo(fileName).lastModified() == document->lastSaved())
+    if (fileInfo.lastModified() == document->lastSaved())
         return;
 
     // Automatically reload when there are no unsaved changes
-    if (!isDocumentModified(document.data())) {
-        reloadDocumentAt(index);
+    if (!isDocumentModified(document)) {
+        reloadDocument(document);
         return;
     }
 
@@ -1245,32 +1290,47 @@ TilesetDocument *DocumentManager::openTilesetFile(const QString &path)
     return i == -1 ? nullptr : qobject_cast<TilesetDocument*>(mDocuments.at(i).data());
 }
 
-WorldDocument *DocumentManager::ensureWorldDocument(const QString &fileName)
+void DocumentManager::registerDocument(Document *document)
 {
-    if (!mWorldDocuments.contains(fileName)) {
-        WorldDocument* worldDocument = new WorldDocument(fileName);
-        mWorldDocuments.insert(fileName, worldDocument);
-        mUndoGroup->addStack(worldDocument->undoStack());
+    const QString &canonicalPath = document->canonicalFilePath();
+    if (canonicalPath.isEmpty())
+        return;
+
+    // Always add path because FileSystemWatcher handles duplicates
+    mFileSystemWatcher->addPath(canonicalPath);
+
+    const auto i = mDocumentByFileName.constFind(canonicalPath);
+    if (i != mDocumentByFileName.constEnd()) {
+        qWarning() << "Document already registered:" << canonicalPath;
+        return;
     }
-    return mWorldDocuments[fileName];
+
+    mDocumentByFileName.insert(canonicalPath, document);
+}
+
+void DocumentManager::unregisterDocument(Document *document)
+{
+    const QString &canonicalPath = document->canonicalFilePath();
+    if (canonicalPath.isEmpty())
+        return;
+
+    // Always remove path because FileSystemWatcher handles duplicates
+    mFileSystemWatcher->removePath(canonicalPath);
+
+    const auto i = mDocumentByFileName.constFind(canonicalPath);
+    if (i != mDocumentByFileName.constEnd() && *i == document)
+        mDocumentByFileName.erase(i);
 }
 
 bool DocumentManager::isAnyWorldModified() const
 {
-    for (const World *world : WorldManager::instance().worlds())
-        if (isWorldModified(world->fileName))
+    for (auto &worldDocument : WorldManager::instance().worlds())
+        if (worldDocument->isModified())
             return true;
 
     return false;
 }
 
-bool DocumentManager::isWorldModified(const QString &fileName) const
-{
-    if (const auto worldDocument = mWorldDocuments.value(fileName))
-        return !worldDocument->undoStack()->isClean();
-    return false;
-}
-
 /**
  * Returns a logical start location for a file dialog to open a file, based on
  * the currently selected file, a recent file, the project path or finally, the
@@ -1295,9 +1355,15 @@ QString DocumentManager::fileDialogStartLocation() const
     return Preferences::homeLocation();
 }
 
-void DocumentManager::onWorldUnloaded(const QString &worldFile)
+void DocumentManager::onWorldLoaded(WorldDocument *worldDocument)
 {
-    delete mWorldDocuments.take(worldFile);
+    mUndoGroup->addStack(worldDocument->undoStack());
+    emit documentOpened(worldDocument);
+}
+
+void DocumentManager::onWorldUnloaded(WorldDocument *worldDocument)
+{
+    mUndoGroup->removeStack(worldDocument->undoStack());
 }
 
 static bool mayNeedColumnCountAdjustment(const Tileset &tileset)
@@ -1324,7 +1390,7 @@ void DocumentManager::tilesetImagesChanged(Tileset *tileset)
     SharedTileset sharedTileset = tileset->sharedFromThis();
     QList<Document*> affectedDocuments;
 
-    for (const auto &document : qAsConst(mDocuments)) {
+    for (const auto &document : std::as_const(mDocuments)) {
         if (auto mapDocument = qobject_cast<MapDocument*>(document.data())) {
             if (mapDocument->map()->tilesets().contains(sharedTileset))
                 affectedDocuments.append(document.data());
@@ -1335,7 +1401,7 @@ void DocumentManager::tilesetImagesChanged(Tileset *tileset)
         affectedDocuments.append(tilesetDocument);
 
     if (!affectedDocuments.isEmpty() && askForAdjustment(*tileset)) {
-        for (Document *document : qAsConst(affectedDocuments)) {
+        for (Document *document : std::as_const(affectedDocuments)) {
             if (auto mapDocument = qobject_cast<MapDocument*>(document)) {
                 auto command = new AdjustTileIndexes(mapDocument, *tileset);
                 document->undoStack()->push(command);
