@@ -26,55 +26,39 @@
 #include "automappingmanager.h"
 #include "changeevents.h"
 #include "changemapproperty.h"
-#include "changeselectedarea.h"
-#include "editablegrouplayer.h"
-#include "editableimagelayer.h"
 #include "editablelayer.h"
-#include "editablemanager.h"
 #include "editablemapobject.h"
-#include "editableobjectgroup.h"
 #include "editableselectedarea.h"
 #include "editabletilelayer.h"
+#include "editabletileset.h"
 #include "grouplayer.h"
-#include "imagelayer.h"
 #include "mapobject.h"
 #include "maprenderer.h"
+#include "minimaprenderer.h"
 #include "objectgroup.h"
 #include "replacetileset.h"
 #include "resizemap.h"
 #include "scriptmanager.h"
 #include "tilelayer.h"
 #include "tileset.h"
-#include "tilesetdocument.h"
 
 #include <QCoreApplication>
 #include <QQmlEngine>
 #include <QUndoStack>
 
-#include "qtcompat_p.h"
-
 namespace Tiled {
 
 EditableMap::EditableMap(QObject *parent)
-    : EditableAsset(nullptr, new Map(), parent)
+    : EditableAsset(new Map(), parent)
 {
     mDetachedMap.reset(map());
 }
 
 EditableMap::EditableMap(MapDocument *mapDocument, QObject *parent)
-    : EditableAsset(mapDocument, mapDocument->map(), parent)
+    : EditableAsset(mapDocument->map(), parent)
     , mSelectedArea(new EditableSelectedArea(mapDocument, this))
 {
-    connect(mapDocument, &Document::fileNameChanged, this, &EditableAsset::fileNameChanged);
-    connect(mapDocument, &Document::changed, this, &EditableMap::documentChanged);
-    connect(mapDocument, &MapDocument::layerAdded, this, &EditableMap::attachLayer);
-    connect(mapDocument, &MapDocument::layerRemoved, this, &EditableMap::detachLayer);
-
-    connect(mapDocument, &MapDocument::currentLayerChanged, this, &EditableMap::currentLayerChanged);
-    connect(mapDocument, &MapDocument::selectedLayersChanged, this, &EditableMap::selectedLayersChanged);
-    connect(mapDocument, &MapDocument::selectedObjectsChanged, this, &EditableMap::selectedObjectsChanged);
-
-    connect(mapDocument, &MapDocument::regionEdited, this, &EditableMap::onRegionEdited);
+    setDocument(mapDocument);
 }
 
 /**
@@ -83,14 +67,13 @@ EditableMap::EditableMap(MapDocument *mapDocument, QObject *parent)
  * The map's lifetime must exceed that of the EditableMap instance.
  */
 EditableMap::EditableMap(const Map *map, QObject *parent)
-    : EditableAsset(nullptr, const_cast<Map*>(map), parent)
+    : EditableAsset(const_cast<Map*>(map), parent)
     , mReadOnly(true)
-    , mSelectedArea(nullptr)
 {
 }
 
 EditableMap::EditableMap(std::unique_ptr<Map> map, QObject *parent)
-    : EditableAsset(nullptr, map.get(), parent)
+    : EditableAsset(map.get(), parent)
     , mDetachedMap(std::move(map))
 {
 }
@@ -99,15 +82,18 @@ EditableMap::~EditableMap()
 {
     for (Layer *layer : map()->layers())
         detachLayer(layer);
+
+    // Prevent owned object from trying to delete us again
+    if (mDetachedMap)
+        setObject(nullptr);
 }
 
 QList<QObject *> EditableMap::tilesets() const
 {
     QList<QObject *> editableTilesets;
-    auto &editableManager = EditableManager::instance();
 
     for (const SharedTileset &tileset : map()->tilesets())
-        editableTilesets.append(editableManager.editableTileset(tileset.data()));
+        editableTilesets.append(EditableTileset::get(tileset.data()));
 
     return editableTilesets;
 }
@@ -115,10 +101,9 @@ QList<QObject *> EditableMap::tilesets() const
 QList<QObject *> EditableMap::layers()
 {
     QList<QObject *> editables;
-    auto &editableManager = EditableManager::instance();
 
     for (const auto layer : map()->layers())
-        editables.append(editableManager.editableLayer(this, layer));
+        editables.append(EditableLayer::get(this, layer));
 
     return editables;
 }
@@ -126,7 +111,7 @@ QList<QObject *> EditableMap::layers()
 EditableLayer *EditableMap::currentLayer()
 {
     if (auto document = mapDocument())
-        return EditableManager::instance().editableLayer(this, document->currentLayer());
+        return EditableLayer::get(this, document->currentLayer());
     return nullptr;
 }
 
@@ -137,10 +122,9 @@ QList<QObject *> EditableMap::selectedLayers()
 
     QList<QObject*> selectedLayers;
 
-    auto &editableManager = EditableManager::instance();
     const auto selectedLayersOrdered = mapDocument()->selectedLayersOrdered();
     for (Layer *layer : selectedLayersOrdered)
-        selectedLayers.append(editableManager.editableLayer(this, layer));
+        selectedLayers.append(EditableLayer::get(this, layer));
 
     return selectedLayers;
 }
@@ -152,10 +136,9 @@ QList<QObject *> EditableMap::selectedObjects()
 
     QList<QObject*> selectedObjects;
 
-    auto &editableManager = EditableManager::instance();
     const auto selectedObjectsOrdered = mapDocument()->selectedObjectsOrdered();
     for (MapObject *object : selectedObjectsOrdered)
-        selectedObjects.append(editableManager.editableMapObject(this, object));
+        selectedObjects.append(EditableMapObject::get(this, object));
 
     return selectedObjects;
 }
@@ -168,7 +151,7 @@ EditableLayer *EditableMap::layerAt(int index)
     }
 
     Layer *layer = map()->layerAt(index);
-    return EditableManager::instance().editableLayer(this, layer);
+    return EditableLayer::get(this, layer);
 }
 
 void EditableMap::removeLayerAt(int index)
@@ -182,7 +165,7 @@ void EditableMap::removeLayerAt(int index)
         push(new RemoveLayer(doc, index, nullptr));
     } else if (!checkReadOnly()) {
         auto layer = map()->takeLayerAt(index);
-        EditableManager::instance().release(layer);
+        EditableLayer::release(layer);
     }
 }
 
@@ -202,6 +185,18 @@ void EditableMap::removeLayer(EditableLayer *editableLayer)
     removeLayerAt(index);
 }
 
+static void initializeSize(Layer *layer, QSize size)
+{
+    if (layer->isTileLayer()) {
+        auto tileLayer = static_cast<TileLayer*>(layer);
+        if (tileLayer->size().isNull())
+            tileLayer->setSize(size);
+    } else if (layer->isGroupLayer()) {
+        for (Layer *childLayer : static_cast<GroupLayer*>(layer)->layers())
+            initializeSize(childLayer, size);
+    }
+}
+
 void EditableMap::insertLayerAt(int index, EditableLayer *editableLayer)
 {
     if (index < 0 || index > layerCount()) {
@@ -210,22 +205,19 @@ void EditableMap::insertLayerAt(int index, EditableLayer *editableLayer)
     }
 
     if (!editableLayer) {
-        ScriptManager::instance().throwNullArgError(0);
+        ScriptManager::instance().throwNullArgError(1);
         return;
     }
 
-    if (editableLayer->map()) {
-        ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Layer already part of a map"));
+    if (!editableLayer->isOwning()) {
+        ScriptManager::instance().throwError(QCoreApplication::translate("Script Errors", "Layer is in use"));
         return;
     }
 
     // If this map has a valid size but the tile layer that's getting added
     // doesn't, default the layer's size to the map size.
-    if (editableLayer->isTileLayer()) {
-        auto editableTileLayer = static_cast<EditableTileLayer*>(editableLayer);
-        if (editableTileLayer->size().isNull() && !size().isNull())
-            editableTileLayer->setSize(size());
-    }
+    if (!size().isNull())
+        initializeSize(editableLayer->layer(), size());
 
     const auto tilesets = editableLayer->layer()->usedTilesets();
 
@@ -241,12 +233,17 @@ void EditableMap::insertLayerAt(int index, EditableLayer *editableLayer)
         map()->addTilesets(tilesets);
 
         // ownership moves to the map
-        map()->insertLayer(index, editableLayer->release());
+        map()->insertLayer(index, editableLayer->attach(this));
     }
 }
 
 void EditableMap::addLayer(EditableLayer *editableLayer)
 {
+    if (!editableLayer) {
+        ScriptManager::instance().throwNullArgError(0);
+        return;
+    }
+
     insertLayerAt(layerCount(), editableLayer);
 }
 
@@ -330,7 +327,7 @@ QList<QObject *> EditableMap::usedTilesets() const
 
     QList<QObject *> editableTilesets;
     for (const SharedTileset &tileset : tilesets)
-        editableTilesets.append(EditableManager::instance().editableTileset(tileset.data()));
+        editableTilesets.append(EditableTileset::get(tileset.data()));
     return editableTilesets;
 }
 
@@ -358,9 +355,9 @@ void EditableMap::removeObjects(const QList<QObject*> &objects)
     if (auto doc = mapDocument()) {
         asset()->push(new RemoveMapObjects(doc, mapObjects));
     } else {
-        for (MapObject *mapObject : mapObjects) {
+        for (MapObject *mapObject : std::as_const(mapObjects)) {
             mapObject->objectGroup()->removeObject(mapObject);
-            EditableManager::instance().release(mapObject);
+            EditableMapObject::release(mapObject);
         }
     }
 }
@@ -442,6 +439,19 @@ void EditableMap::autoMap(const RegionValueType &region, const QString &rulesFil
         manager.autoMapRegion(region.region());
 }
 
+Tiled::ScriptImage *EditableMap::toImage(QSize size) const
+{
+    const MiniMapRenderer miniMapRenderer(map());
+    const QSize imageSize = size.isValid() ? size : miniMapRenderer.mapSize();
+    const MiniMapRenderer::RenderFlags renderFlags(MiniMapRenderer::DrawTileLayers |
+                                                   MiniMapRenderer::DrawMapObjects |
+                                                   MiniMapRenderer::DrawImageLayers |
+                                                   MiniMapRenderer::IgnoreInvisibleLayer |
+                                                   MiniMapRenderer::DrawBackground);
+
+    return new ScriptImage(miniMapRenderer.render(imageSize, renderFlags));
+}
+
 QPointF EditableMap::screenToTile(qreal x, qreal y) const
 {
     return renderer()->screenToTileCoords(x, y);
@@ -655,14 +665,49 @@ QSharedPointer<Document> EditableMap::createDocument()
     auto document = MapDocumentPtr::create(std::move(mDetachedMap));
     document->setEditable(std::unique_ptr<EditableAsset>(this));
 
-    QQmlEngine::setObjectOwnership(this, QQmlEngine::CppOwnership);
+    mSelectedArea = new EditableSelectedArea(document.data(), this);
+
+    moveOwnershipToCpp();
 
     return document;
 }
 
+void EditableMap::setDocument(Document *document)
+{
+    Q_ASSERT(!document || document->type() == Document::MapDocumentType);
+
+    if (this->document() == document)
+        return;
+
+    EditableAsset::setDocument(document);
+
+    if (auto doc = mapDocument()) {
+        connect(doc, &Document::fileNameChanged, this, &EditableAsset::fileNameChanged);
+        connect(doc, &Document::changed, this, &EditableMap::documentChanged);
+        connect(doc, &MapDocument::layerAdded, this, &EditableMap::attachLayer);
+        connect(doc, &MapDocument::layerRemoved, this, &EditableMap::detachLayer);
+
+        connect(doc, &MapDocument::currentLayerChanged, this, &EditableMap::currentLayerChanged);
+        connect(doc, &MapDocument::selectedLayersChanged, this, &EditableMap::selectedLayersChanged);
+        connect(doc, &MapDocument::selectedObjectsChanged, this, &EditableMap::selectedObjectsChanged);
+
+        connect(doc, &MapDocument::regionEdited, this, &EditableMap::onRegionEdited);
+    }
+}
+
 void EditableMap::documentChanged(const ChangeEvent &change)
 {
     switch (change.type) {
+    case ChangeEvent::DocumentAboutToReload:
+        for (Layer *layer : map()->layers())
+            detachLayer(layer);
+
+        mRenderer.reset();
+        setObject(nullptr);
+        break;
+    case ChangeEvent::DocumentReloaded:
+        setObject(mapDocument()->map());
+        break;
     case ChangeEvent::MapChanged:
         if (static_cast<const MapChangeEvent&>(change).property == Map::OrientationProperty)
             mRenderer.reset();
@@ -680,7 +725,7 @@ void EditableMap::documentChanged(const ChangeEvent &change)
 
 void EditableMap::attachLayer(Layer *layer)
 {
-    if (EditableLayer *editable = EditableManager::instance().find(layer))
+    if (auto editable = EditableLayer::find(layer))
         editable->attach(this);
 
     if (GroupLayer *groupLayer = layer->asGroupLayer()) {
@@ -693,7 +738,7 @@ void EditableMap::attachLayer(Layer *layer)
 
 void EditableMap::detachLayer(Layer *layer)
 {
-    auto editableLayer = EditableManager::instance().find(layer);
+    auto editableLayer = EditableLayer::find(layer);
     if (editableLayer && editableLayer->map() == this)
         editableLayer->detach();
 
@@ -707,18 +752,16 @@ void EditableMap::detachLayer(Layer *layer)
 
 void EditableMap::attachMapObjects(const QList<MapObject *> &mapObjects)
 {
-    const auto &editableManager = EditableManager::instance();
     for (MapObject *mapObject : mapObjects) {
-        if (EditableMapObject *editable = editableManager.find(mapObject))
+        if (auto editable = EditableMapObject::find(mapObject))
             editable->attach(this);
     }
 }
 
 void EditableMap::detachMapObjects(const QList<MapObject *> &mapObjects)
 {
-    const auto &editableManager = EditableManager::instance();
     for (MapObject *mapObject : mapObjects) {
-        if (EditableMapObject *editable = editableManager.find(mapObject)) {
+        if (auto editable = EditableMapObject::find(mapObject)) {
             Q_ASSERT(editable->map() == this);
             editable->detach();
         }
@@ -727,8 +770,7 @@ void EditableMap::detachMapObjects(const QList<MapObject *> &mapObjects)
 
 void EditableMap::onRegionEdited(const QRegion &region, TileLayer *layer)
 {
-    auto &editableManager = EditableManager::instance();
-    const auto editableLayer = static_cast<EditableTileLayer*>(editableManager.editableLayer(this, layer));
+    const auto editableLayer = static_cast<EditableTileLayer*>(EditableLayer::get(this, layer));
     emit regionEdited(RegionValueType(region), editableLayer);
 }
 
