@@ -99,4 +99,27 @@ QString Id::toString() const
     return QString::fromUtf8(name());
 }
 
+
+QStringList idsToNames(const QList<Id> &ids)
+{
+    QStringList names;
+    names.reserve(ids.size());
+    for (const Id &id : ids)
+        names.append(id.toString());
+
+    names.sort();
+
+    return names;
+}
+
+QList<Id> namesToIds(const QStringList &names)
+{
+    QList<Id> ids;
+    ids.reserve(names.size());
+    for (const QString &name : names)
+        ids.append(Id(name.toUtf8()));
+
+    return ids;
+}
+
 } // namespace Tiled
