@@ -23,12 +23,12 @@
 
 #pragma once
 
-#include "clipboardmanager.h"
 #include "document.h"
 #include "preferences.h"
 #include "preferencesdialog.h"
 #include "project.h"
 #include "session.h"
+#include "tilededitor_global.h"
 
 #include <QMainWindow>
 #include <QPointer>
@@ -36,6 +36,7 @@
 
 class QComboBox;
 class QLabel;
+class QToolButton;
 
 namespace Ui {
 class MainWindow;
@@ -51,18 +52,19 @@ class ConsoleDock;
 class DocumentManager;
 class Editor;
 class IssuesDock;
+class LocatorSource;
 class LocatorWidget;
 class MapDocument;
 class MapDocumentActionHandler;
 class MapEditor;
 class MapScene;
 class MapView;
-class ObjectTypesEditor;
 class PropertyTypesEditor;
 class ProjectDock;
 class ProjectModel;
 class TilesetDocument;
 class TilesetEditor;
+class WorldDocument;
 class Zoomable;
 
 /**
@@ -71,7 +73,7 @@ class Zoomable;
  * Represents the main user interface, including the menu bar. It keeps track
  * of the current file and is also the entry point of all menu actions.
  */
-class MainWindow : public QMainWindow
+class TILED_EDITOR_EXPORT MainWindow : public QMainWindow
 {
     Q_OBJECT
 
@@ -97,6 +99,7 @@ public:
     bool addRecentProjectsActions(QMenu *menu) const;
 
     static MainWindow *instance();
+    static MainWindow *maybeInstance();
 
 protected:
     bool event(QEvent *event) override;
@@ -104,9 +107,6 @@ protected:
     void closeEvent(QCloseEvent *event) override;
     void changeEvent(QEvent *event) override;
 
-    void keyPressEvent(QKeyEvent *) override;
-    void keyReleaseEvent(QKeyEvent *) override;
-
     void dragEnterEvent(QDragEnterEvent *) override;
     void dropEvent(QDropEvent *) override;
 
@@ -116,6 +116,8 @@ private:
     void newMap();
     void openFileDialog();
     void openFileInProject();
+    void searchActions();
+    void showLocatorWidget(LocatorSource *source);
     bool saveFile();
     bool saveFileAs();
     void saveAll();
@@ -127,11 +129,10 @@ private:
     void closeFile();
     bool closeAllFiles();
 
-    void openProject();
     bool openProjectFile(const QString &fileName);
-    void saveProjectAs();
-    void closeProject();
-    bool switchProject(Project project);
+    void newProject();
+    bool closeProject();
+    bool switchProject(std::unique_ptr<Project> project);
     void restoreSession();
     void projectProperties();
 
@@ -160,6 +161,7 @@ private:
     bool newTileset(const QString &path = QString());
     void reloadTilesetImages();
     void addExternalTileset();
+    void addAutomappingRulesTileset();
     void resizeMap();
     void offsetMap();
     void editMapProperties();
@@ -188,7 +190,6 @@ private:
     void autoMappingError(bool automatic);
     void autoMappingWarning(bool automatic);
 
-    void onObjectTypesEditorClosed();
     void onPropertyTypesEditorClosed();
     void ensureHasBorderInFullScreen();
 
@@ -212,10 +213,11 @@ private:
       */
     bool confirmAllSave();
 
-    bool confirmSaveWorld(const QString &fileName);
+    bool confirmSaveWorld(WorldDocument *worldDocument);
 
     void writeSettings();
     void readSettings();
+    void restoreLayout();
 
     void updateRecentFilesMenu();
     void updateRecentProjectsMenu();
@@ -236,7 +238,6 @@ private:
     ConsoleDock *mConsoleDock;
     ProjectDock *mProjectDock;
     IssuesDock *mIssuesDock;
-    ObjectTypesEditor *mObjectTypesEditor;
     PropertyTypesEditor *mPropertyTypesEditor;
     QPointer<LocatorWidget> mLocatorWidget;
     QPointer<QWidget> mPopupWidget;
@@ -249,7 +250,6 @@ private:
     QMenu *mGroupLayerMenu;
     QMenu *mViewsAndToolbarsMenu;
     QAction *mViewsAndToolbarsAction;
-    QAction *mShowObjectTypesEditor;
     QAction *mShowPropertyTypesEditor;
     QAction *mResetToDefaultLayout;
     QAction *mLockLayout;
@@ -261,10 +261,12 @@ private:
     MapEditor *mMapEditor;
     TilesetEditor *mTilesetEditor;
     QList<QWidget*> mEditorStatusBarWidgets;
+    QToolButton *mNewsButton;
 
     QPointer<PreferencesDialog> mPreferencesDialog;
 
     QMap<QMainWindow*, QByteArray> mMainWindowStates;
+    bool mHasRestoredLayout = false;
 
     SessionOption<QStringList> mLoadedWorlds { "loadedWorlds" };
 
@@ -277,4 +279,9 @@ inline MainWindow *MainWindow::instance()
     return mInstance;
 }
 
+inline MainWindow *MainWindow::maybeInstance()
+{
+    return mInstance;
+}
+
 } // namespace Tiled
