@@ -25,6 +25,7 @@
 #include "addremovemapobject.h"
 #include "addremovetileset.h"
 #include "brokenlinks.h"
+#include "changeevents.h"
 #include "changelayer.h"
 #include "changemapobject.h"
 #include "changemapobjectsorder.h"
@@ -32,7 +33,9 @@
 #include "changeselectedarea.h"
 #include "containerhelpers.h"
 #include "editablemap.h"
+#include "editor.h"
 #include "flipmapobjects.h"
+#include "geometry.h"
 #include "grouplayer.h"
 #include "imagelayer.h"
 #include "issuesmodel.h"
@@ -42,9 +45,9 @@
 #include "mapobjectmodel.h"
 #include "maprenderer.h"
 #include "movelayer.h"
-#include "movemapobject.h"
 #include "movemapobjecttogroup.h"
 #include "objectgroup.h"
+#include "objectreferenceshelper.h"
 #include "objecttemplate.h"
 #include "offsetlayer.h"
 #include "painttilelayer.h"
@@ -52,21 +55,39 @@
 #include "reparentlayers.h"
 #include "resizemap.h"
 #include "resizetilelayer.h"
-#include "rotatemapobject.h"
 #include "templatemanager.h"
 #include "tile.h"
 #include "tilelayer.h"
 #include "tilesetdocument.h"
+#include "transformmapobjects.h"
 
 #include <QFileInfo>
 #include <QRect>
+#include <QSet>
+#include <QString>
 #include <QUndoStack>
 
-#include "changeevents.h"
-#include "qtcompat_p.h"
-
 using namespace Tiled;
 
+class ReloadMap : public QUndoCommand
+{
+public:
+    ReloadMap(MapDocument *mapDocument, std::unique_ptr<Map> map)
+        : mMapDocument(mapDocument)
+        , mMap(std::move(map))
+    {
+        setText(QCoreApplication::translate("Undo Commands", "Reload Map"));
+    }
+
+    void undo() override { mMapDocument->swapMap(mMap); }
+    void redo() override { mMapDocument->swapMap(mMap); }
+
+private:
+    MapDocument *mMapDocument;
+    std::unique_ptr<Map> mMap;
+};
+
+
 MapDocument::MapDocument(std::unique_ptr<Map> map)
     : Document(MapDocumentType, map->fileName)
     , mMap(std::move(map))
@@ -125,7 +146,7 @@ bool MapDocument::save(const QString &fileName, QString *error)
     MapFormat *mapFormat = writerFormat();
     if (!mapFormat) {
         if (error)
-            *error = tr("Map format '%s' not found").arg(mWriterFormat);
+            *error = tr("Map format '%1' not found").arg(mWriterFormat);
         return false;
     }
 
@@ -156,6 +177,42 @@ bool MapDocument::save(const QString &fileName, QString *error)
     return true;
 }
 
+bool MapDocument::canReload() const
+{
+    return !fileName().isEmpty() && !mReaderFormat.isEmpty();
+}
+
+bool MapDocument::reload(QString *error)
+{
+    if (!canReload())
+        return false;
+
+    auto format = findFileFormat<MapFormat>(mReaderFormat, FileFormat::Read);
+    if (!format) {
+        if (error)
+            *error = tr("Map format '%s' not found").arg(mReaderFormat);
+        return false;
+    }
+
+    auto map = format->read(fileName());
+
+    if (!map) {
+        if (error)
+            *error = format->errorString();
+        return false;
+    }
+
+    map->fileName = fileName();
+
+    undoStack()->push(new ReloadMap(this, std::move(map)));
+    undoStack()->setClean();
+
+    mLastSaved = QFileInfo(fileName()).lastModified();
+    setChangedOnDisk(false);
+
+    return true;
+}
+
 MapDocumentPtr MapDocument::load(const QString &fileName,
                                  MapFormat *format,
                                  QString *error)
@@ -269,6 +326,9 @@ void MapDocument::setSelectedLayers(const QList<Layer *> &layers)
 
     mSelectedLayers = layers;
     emit selectedLayersChanged();
+
+    if (currentObject() && currentObject()->typeId() == Object::LayerType)
+        emit currentObjectsChanged();
 }
 
 void MapDocument::switchCurrentLayer(Layer *layer)
@@ -338,6 +398,7 @@ void MapDocument::resizeMap(QSize size, QPoint offset, bool removeObjects)
     QUndoCommand *command = new QUndoCommand(tr("Resize Map"));
 
     QList<MapObject *> objectsToRemove;
+    QList<MapObject *> objectsToMove;
 
     LayerIterator iterator(map());
     while (Layer *layer = iterator.next()) {
@@ -351,14 +412,11 @@ void MapDocument::resizeMap(QSize size, QPoint offset, bool removeObjects)
             ObjectGroup *objectGroup = static_cast<ObjectGroup*>(layer);
 
             for (MapObject *o : objectGroup->objects()) {
-                if (removeObjects && !visibleIn(visibleArea, o, *renderer())) {
-                    // Remove objects that will fall outside of the map
+                // Remove objects that will fall outside of the map
+                if (removeObjects && !visibleIn(visibleArea, o, *renderer()))
                     objectsToRemove.append(o);
-                } else {
-                    QPointF oldPos = o->position();
-                    QPointF newPos = oldPos + pixelOffset;
-                    new MoveMapObject(this, o, newPos, oldPos, command);
-                }
+                else if (!pixelOffset.isNull())
+                    objectsToMove.append(o);
             }
             break;
         }
@@ -380,6 +438,15 @@ void MapDocument::resizeMap(QSize size, QPoint offset, bool removeObjects)
     if (!objectsToRemove.isEmpty())
         new RemoveMapObjects(this, objectsToRemove, command);
 
+    if (!objectsToMove.isEmpty()) {
+        QVector<TransformState> states;
+        for (MapObject *o : std::as_const(objectsToMove)) {
+            states.append(TransformState(o));
+            states.last().setPosition(o->position() + pixelOffset);
+        }
+        new TransformMapObjects(this, objectsToMove, states, command);
+    }
+
     new ResizeMap(this, size, command);
     new ChangeSelectedArea(this, movedSelection, command);
 
@@ -405,6 +472,7 @@ void MapDocument::autocropMap()
 void MapDocument::offsetMap(const QList<Layer*> &layers,
                             const QPoint offset,
                             const QRect &bounds,
+                            bool wholeMap,
                             bool wrapX, bool wrapY)
 {
     if (layers.empty())
@@ -413,20 +481,30 @@ void MapDocument::offsetMap(const QList<Layer*> &layers,
     undoStack()->beginMacro(tr("Offset Map"));
     for (auto layer : layers) {
         undoStack()->push(new OffsetLayer(this, layer, offset,
-                                          bounds, wrapX, wrapY));
+                                          bounds, wholeMap, wrapX, wrapY));
     }
     undoStack()->endMacro();
 }
 
 /**
- * Flips the selected objects in the given \a direction.
+ * Flips the selected objects in the given \a direction, around the given
+ * \a origin.
  */
 void MapDocument::flipSelectedObjects(FlipDirection direction)
 {
     if (mSelectedObjects.isEmpty())
         return;
 
-    undoStack()->push(new FlipMapObjects(this, mSelectedObjects, direction));
+    QRectF sharedBounds;
+
+    for (const MapObject *object : std::as_const(mSelectedObjects)) {
+        QPointF screenPos = mRenderer->pixelToScreenCoords(object->position());
+        QRectF bounds = object->screenBounds(*mRenderer);
+        sharedBounds |= rotateAt(screenPos, object->rotation()).mapRect(bounds);
+    }
+
+    const QPointF origin = sharedBounds.center();
+    undoStack()->push(new FlipMapObjects(this, mSelectedObjects, direction, origin));
 }
 
 /**
@@ -437,12 +515,11 @@ void MapDocument::rotateSelectedObjects(RotateDirection direction)
     if (mSelectedObjects.isEmpty())
         return;
 
-    undoStack()->beginMacro(tr("Rotate %n Object(s)", "",
-                               mSelectedObjects.size()));
+    QVector<TransformState> states;
+    states.reserve(mSelectedObjects.size());
 
     // TODO: Rotate them properly as a group
-    const auto &selectedObjects = mSelectedObjects;
-    for (MapObject *mapObject : selectedObjects) {
+    for (MapObject *mapObject : std::as_const(mSelectedObjects)) {
         const qreal oldRotation = mapObject->rotation();
         qreal newRotation = oldRotation;
 
@@ -456,10 +533,12 @@ void MapDocument::rotateSelectedObjects(RotateDirection direction)
                 newRotation -= 360;
         }
 
-        undoStack()->push(new RotateMapObject(this, mapObject,
-                                              newRotation, oldRotation));
+        states.append(TransformState(mapObject));
+        auto &state = states.last();
+        state.setRotation(newRotation);
     }
-    undoStack()->endMacro();
+
+    undoStack()->push(new TransformMapObjects(this, mSelectedObjects, states));
 }
 
 /**
@@ -469,23 +548,20 @@ void MapDocument::rotateSelectedObjects(RotateDirection direction)
 Layer *MapDocument::addLayer(Layer::TypeFlag layerType)
 {
     Layer *layer = nullptr;
-    QString name;
+    const QString name = newLayerName(layerType);
+    Q_ASSERT(!name.isEmpty());
 
     switch (layerType) {
     case Layer::TileLayerType:
-        name = tr("Tile Layer %1").arg(mMap->tileLayerCount() + 1);
         layer = new TileLayer(name, 0, 0, mMap->width(), mMap->height());
         break;
     case Layer::ObjectGroupType:
-        name = tr("Object Layer %1").arg(mMap->objectGroupCount() + 1);
         layer = new ObjectGroup(name, 0, 0);
         break;
     case Layer::ImageLayerType:
-        name = tr("Image Layer %1").arg(mMap->imageLayerCount() + 1);
         layer = new ImageLayer(name, 0, 0);
         break;
     case Layer::GroupLayerType:
-        name = tr("Group %1").arg(mMap->groupLayerCount() + 1);
         layer = new GroupLayer(name, 0, 0);
         break;
     }
@@ -593,37 +669,55 @@ void MapDocument::duplicateLayers(const QList<Layer *> &layers)
         if (layers.contains(layer))
             layersToDuplicate.append(layer);
 
+    struct Duplication {
+        Layer *original;
+        Layer *clone;
+    };
+    QVector<Duplication> duplications;
+    ObjectReferencesHelper objectRefs(map());
+
+    // Duplicate the layers, reassigning any layer and object IDs
+    while (!layersToDuplicate.isEmpty()) {
+        Duplication dup;
+        dup.original = layersToDuplicate.takeFirst();
+        dup.clone = dup.original->clone();
+
+        // If a group layer gets duplicated, make sure any children are removed
+        // from the remaining list of layers to duplicate
+        if (dup.original->isGroupLayer()) {
+            layersToDuplicate.erase(std::remove_if(layersToDuplicate.begin(),
+                                                   layersToDuplicate.end(),
+                                                   [&] (Layer *layer) {
+                                        return layer->isParentOrSelf(dup.original);
+                                    }), layersToDuplicate.end());
+        }
+
+        objectRefs.reassignIds(dup.clone);
+        dup.clone->setName(Editor::nameOfDuplicate(dup.clone->name()));
+
+        duplications.append(dup);
+    }
+
+    objectRefs.rewire();
+
+    // Actually add each duplicated layer
     QList<Layer *> newLayers;
     GroupLayer *previousParentLayer = nullptr;
     int previousIndex = 0;
 
-    while (!layersToDuplicate.isEmpty()) {
-        Layer *layer = layersToDuplicate.takeFirst();
-
-        // If a group layer gets duplicated, make sure any children are removed
-        // from the remaining list of layers to duplicate
-        if (layer->isGroupLayer()) {
-            for (int i = layersToDuplicate.size() - 1; i >= 0; --i)
-                if (layersToDuplicate.at(i)->isParentOrSelf(layer))
-                    layersToDuplicate.removeAt(i);
-        }
-
-        Layer *duplicate = layer->clone();
-        duplicate->resetIds();
-        duplicate->setName(tr("Copy of %1").arg(duplicate->name()));
-
-        auto parentLayer = layer->parentLayer();
+    for (const auto &dup : std::as_const(duplications)) {
+        auto parentLayer = dup.original->parentLayer();
 
         int index = previousIndex;
         if (newLayers.isEmpty() || previousParentLayer != parentLayer)
-            index = layer->siblingIndex() + 1;
+            index = dup.original->siblingIndex() + 1;
 
-        undoStack()->push(new AddLayer(this, index, duplicate, parentLayer));
+        undoStack()->push(new AddLayer(this, index, dup.clone, parentLayer));
 
         previousParentLayer = parentLayer;
         previousIndex = index;
 
-        newLayers.append(duplicate);
+        newLayers.append(dup.clone);
     }
 
     undoStack()->endMacro();
@@ -706,7 +800,7 @@ void MapDocument::moveLayersUp(const QList<Layer *> &layers)
         return;
 
     undoStack()->beginMacro(QCoreApplication::translate("Undo Commands", "Raise %n Layer(s)", "", layersToMove.size()));
-    for (Layer *layer : qAsConst(layersToMove))
+    for (Layer *layer : std::as_const(layersToMove))
         undoStack()->push(new MoveLayer(this, layer, MoveLayer::Up));
     undoStack()->endMacro();
 }
@@ -733,7 +827,7 @@ void MapDocument::moveLayersDown(const QList<Layer *> &layers)
         return;
 
     undoStack()->beginMacro(QCoreApplication::translate("Undo Commands", "Lower %n Layer(s)", "", layersToMove.size()));
-    for (Layer *layer : qAsConst(layersToMove))
+    for (Layer *layer : std::as_const(layersToMove))
         undoStack()->push(new MoveLayer(this, layer, MoveLayer::Down));
     undoStack()->endMacro();
 }
@@ -957,8 +1051,8 @@ void MapDocument::paintTileLayers(const Map &map, bool mergeable,
         auto source = sourceLayers[i];
         auto target = targetLayers[i];
 
-        const QRegion editedRegion = source->region();
-        if (editedRegion.isEmpty())
+        const QRegion paintRegion = source->modifiedRegion();
+        if (paintRegion.isEmpty())
             continue;
 
         std::unique_ptr<TileLayer> newLayer;
@@ -983,10 +1077,10 @@ void MapDocument::paintTileLayers(const Map &map, bool mergeable,
                                                           source->x(),
                                                           source->y(),
                                                           source,
-                                                          editedRegion);
+                                                          paintRegion);
 
         if (missingTilesets && !missingTilesets->isEmpty()) {
-            for (const SharedTileset &tileset : qAsConst(*missingTilesets)) {
+            for (const SharedTileset &tileset : std::as_const(*missingTilesets)) {
                 if (!mMap->tilesets().contains(tileset))
                     new AddTileset(this, tileset, paintCommand);
             }
@@ -1023,7 +1117,7 @@ void MapDocument::paintTileLayers(const Map &map, bool mergeable,
         paintCommand->setMergeable(mergeable);
         undoStack()->push(paintCommand);
 
-        regions[target] |= editedRegion;
+        regions[target] |= paintRegion;
 
         mergeable = true; // further paints are always mergeable
     }
@@ -1040,6 +1134,55 @@ void MapDocument::paintTileLayers(const Map &map, bool mergeable,
     }
 }
 
+void MapDocument::eraseTileLayers(const QRegion &region,
+                                  bool allLayers,
+                                  bool mergeable,
+                                  const QString &customName)
+{
+    QList<QPair<QRegion, TileLayer*>> erasedRegions;
+
+    auto eraseCommand = std::make_unique<PaintTileLayer>(this);
+    eraseCommand->setText(customName.isEmpty() ? QCoreApplication::translate("Undo Commands", "Erase")
+                                               : customName);
+    eraseCommand->setMergeable(mergeable);
+
+    auto eraseOnLayer = [&] (TileLayer *tileLayer) {
+        if (!tileLayer->isUnlocked())
+            return;
+
+        QRegion eraseRegion = region.intersected(tileLayer->bounds());
+        if (eraseRegion.isEmpty())
+            return;
+
+        eraseCommand->erase(tileLayer, eraseRegion);
+
+        erasedRegions.append({ eraseRegion, tileLayer });
+    };
+
+    if (allLayers) {
+        for (Layer *layer : map()->tileLayers())
+            eraseOnLayer(static_cast<TileLayer*>(layer));
+    } else if (!selectedLayers().isEmpty()) {
+        for (Layer *layer : selectedLayers())
+            if (TileLayer *tileLayer = layer->asTileLayer())
+                eraseOnLayer(tileLayer);
+    } else if (auto tileLayer = dynamic_cast<TileLayer*>(currentLayer())) {
+        eraseOnLayer(tileLayer);
+    }
+
+    if (!erasedRegions.isEmpty())
+        undoStack()->push(eraseCommand.release());
+
+    for (auto &[region, tileLayer] : std::as_const(erasedRegions)) {
+        // Sanity check needed because a script might respond to the below
+        // signal by removing the layer from the map.
+        if (tileLayer->map() != map())
+            continue;
+
+        emit regionEdited(region, tileLayer);
+    }
+}
+
 void MapDocument::replaceObjectTemplate(const ObjectTemplate *oldObjectTemplate,
                                         const ObjectTemplate *newObjectTemplate)
 {
@@ -1137,8 +1280,10 @@ void MapDocument::setSelectedObjects(const QList<MapObject *> &selectedObjects)
     // Make sure the current object is one of the selected ones
     if (!selectedObjects.isEmpty()) {
         if (currentObject() && currentObject()->typeId() == Object::MapObjectType) {
-            if (selectedObjects.contains(static_cast<MapObject*>(currentObject())))
+            if (selectedObjects.contains(static_cast<MapObject*>(currentObject()))) {
+                emit currentObjectsChanged();
                 return;
+            }
         }
 
         setCurrentObject(selectedObjects.first());
@@ -1251,10 +1396,10 @@ void MapDocument::unifyTilesets(Map &map)
  * and adds tilesets to \a missingTilesets whenever there is a tileset without
  * replacement in this map.
  */
-void MapDocument::unifyTilesets(Map &map, QVector<SharedTileset> &missingTilesets)
+void MapDocument::unifyTilesets(Map &map, QVector<SharedTileset> &missingTilesets) const
 {
     QVector<SharedTileset> availableTilesets = mMap->tilesets();
-    for (const SharedTileset &tileset : qAsConst(missingTilesets))
+    for (const SharedTileset &tileset : std::as_const(missingTilesets))
         if (!availableTilesets.contains(tileset))
             availableTilesets.append(tileset);
 
@@ -1416,6 +1561,43 @@ void MapDocument::onLayerRemoved(Layer *layer)
     emit layerRemoved(layer);
 }
 
+QString MapDocument::newLayerName(Layer::TypeFlag layerType) const
+{
+    const char *parametricName = nullptr;
+    switch (layerType)
+    {
+        case Layer::TypeFlag::TileLayerType:
+            parametricName = QT_TR_NOOP("Tile Layer %1");
+            break;
+        case Layer::TypeFlag::ObjectGroupType:
+            parametricName = QT_TR_NOOP("Object Layer %1");
+            break;
+        case Layer::TypeFlag::ImageLayerType:
+            parametricName = QT_TR_NOOP("Image Layer %1");
+            break;
+        case Layer::TypeFlag::GroupLayerType:
+            parametricName = QT_TR_NOOP("Group Layer %1");
+            break;
+        default:
+            // invalid layer type
+            return {};
+    }
+
+    QSet<QString> layerNames;
+    int number = 0;
+    for (const auto layer : mMap->allLayers(layerType)) {
+        ++number;
+        layerNames.insert(layer->name());
+    }
+
+    QString candidateName;
+    do {
+        candidateName = tr(parametricName).arg(++number);
+    } while (layerNames.contains(candidateName));
+
+    return candidateName;
+}
+
 void MapDocument::checkIssues()
 {
     // Clear any previously found issues in this document
@@ -1459,6 +1641,54 @@ void MapDocument::checkIssues()
     }
 }
 
+void MapDocument::swapMap(std::unique_ptr<Map> &other)
+{
+    // Store previous state
+    const int currentLayerId = currentLayer() ? currentLayer()->id() : -1;
+
+    QVector<int> selectedLayerIds;
+    for (const Layer *layer : selectedLayers())
+        selectedLayerIds.append(layer->id());
+
+    QVector<int> selectedObjectIds;
+    for (const MapObject *object : selectedObjects())
+        selectedObjectIds.append(object->id());
+
+    // Bring pointers to safety
+    setSelectedLayers({});
+    setSelectedObjects({});
+    setAboutToBeSelectedObjects({});
+    setHoveredMapObject(nullptr);
+    setCurrentLayer(nullptr);
+    setCurrentObject(nullptr);
+
+    emit changed(AboutToReloadEvent());
+
+    mMap.swap(other);
+    createRenderer();
+
+    emit changed(ReloadEvent());
+
+    // Restore previous state
+    QList<MapObject*> selectedObjects;
+    for (int id : selectedObjectIds) {
+        if (MapObject *object = mMap->findObjectById(id))
+            selectedObjects.append(object);
+    }
+    setSelectedObjects(selectedObjects);
+
+    if (currentLayerId != -1)
+        if (auto layer = mMap->findLayerById(currentLayerId))
+            switchCurrentLayer(layer);
+
+    QList<Layer*> selectedLayers;
+    for (int id : selectedLayerIds) {
+        if (Layer *layer = mMap->findLayerById(id))
+            selectedLayers.append(layer);
+    }
+    switchSelectedLayers(selectedLayers);
+}
+
 void MapDocument::updateTemplateInstances(const ObjectTemplate *objectTemplate)
 {
     QList<MapObject*> objectList;
@@ -1505,8 +1735,11 @@ void MapDocument::deselectObjects(const QList<MapObject *> &objects)
         removedAboutToBeSelectedObjects += mAboutToBeSelectedObjects.removeAll(object);
     }
 
-    if (removedSelectedObjects > 0)
+    if (removedSelectedObjects > 0) {
         emit selectedObjectsChanged();
+        if (mCurrentObject && mCurrentObject->typeId() == Object::MapObjectType)
+            emit currentObjectsChanged();
+    }
     if (removedAboutToBeSelectedObjects > 0)
         emit aboutToBeSelectedObjectsChanged(mAboutToBeSelectedObjects);
 }
@@ -1519,13 +1752,18 @@ void MapDocument::duplicateObjects(const QList<MapObject *> &objects)
     QVector<AddMapObjects::Entry> objectsToAdd;
     objectsToAdd.reserve(objects.size());
 
+    ObjectReferencesHelper objectRefs(map());
+
     for (MapObject *mapObject : objects) {
         MapObject *clone = mapObject->clone();
-        clone->resetId();
+        clone->setName(Editor::nameOfDuplicate(clone->name()));
+        objectRefs.reassignId(clone);
         objectsToAdd.append(AddMapObjects::Entry { clone, mapObject->objectGroup() });
         objectsToAdd.last().index = mapObject->objectGroup()->objects().indexOf(mapObject) + 1;
     }
 
+    objectRefs.rewire();
+
     auto command = new AddMapObjects(this, objectsToAdd);
     command->setText(tr("Duplicate %n Object(s)", "", objects.size()));
 
