@@ -27,10 +27,12 @@
 #include "map.h"
 #include "mapformat.h"
 #include "tiled.h"
+#include "tilededitor_global.h"
 #include "tileset.h"
 
 #include <QList>
 #include <QRegion>
+#include <QSet>
 
 #include <memory>
 
@@ -65,7 +67,7 @@ using MapDocumentPtr = QSharedPointer<MapDocument>;
  * selected layer and provides an API for adding and removing map objects. It
  * also owns the QUndoStack.
  */
-class MapDocument : public Document
+class TILED_EDITOR_EXPORT MapDocument final : public Document
 {
     Q_OBJECT
 
@@ -82,7 +84,7 @@ public:
     /**
      * Constructs a map document around the given map.
      */
-    MapDocument(std::unique_ptr<Map> map);
+    explicit MapDocument(std::unique_ptr<Map> map);
 
     ~MapDocument() override;
 
@@ -90,6 +92,9 @@ public:
 
     bool save(const QString &fileName, QString *error = nullptr) override;
 
+    bool canReload() const override;
+    bool reload(QString *error);
+
     /**
      * Loads a map and returns a MapDocument instance on success. Returns null
      * on error and sets the \a error message.
@@ -149,6 +154,7 @@ public:
     void offsetMap(const QList<Layer *> &layers,
                    QPoint offset,
                    const QRect &bounds,
+                   bool wholeMap,
                    bool wrapX, bool wrapY);
 
     void flipSelectedObjects(FlipDirection direction);
@@ -175,6 +181,10 @@ public:
     void paintTileLayers(const Map &map, bool mergeable = false,
                          QVector<SharedTileset> *missingTilesets = nullptr,
                          QHash<TileLayer *, QRegion> *paintedRegions = nullptr);
+    void eraseTileLayers(const QRegion &region,
+                         bool allLayers = false,
+                         bool mergeable = false,
+                         const QString &customName = QString());
 
     void replaceObjectTemplate(const ObjectTemplate *oldObjectTemplate,
                                const ObjectTemplate *newObjectTemplate);
@@ -245,7 +255,7 @@ public:
     void setHoveredMapObject(MapObject *object);
 
     void unifyTilesets(Map &map);
-    void unifyTilesets(Map &map, QVector<SharedTileset> &missingTilesets);
+    void unifyTilesets(Map &map, QVector<SharedTileset> &missingTilesets) const;
 
     bool allowHidingObjects() const { return mAllowHidingObjects; }
     void setAllowHidingObjects(bool value) { mAllowHidingObjects = value; }
@@ -257,6 +267,11 @@ public:
 
     void checkIssues() override;
 
+    void swapMap(std::unique_ptr<Map> &other);
+
+    QSet<int> expandedGroupLayers;
+    QSet<int> expandedObjectLayers;
+
 signals:
     /**
      * Emitted when the selected tile region changes. Sends the currently
@@ -357,7 +372,6 @@ signals:
     // emitted from the TilesetDocument
     void tilesetNameChanged(Tileset *tileset);
     void tilesetTilePositioningChanged(Tileset *tileset);
-    void tileTypeChanged(Tile *tile);
     void tileImageSourceChanged(Tile *tile);
     void tileProbabilityChanged(Tile *tile);
     void tileObjectGroupChanged(Tile *tile);
@@ -384,6 +398,8 @@ private:
 
     void moveObjectIndex(const MapObject *object, int count);
 
+    QString newLayerName(Layer::TypeFlag layerType) const;
+
     /*
      * QString is used since the formats referenced here may be dynamically
      * added by a plugin, and can also be removed again.
