@@ -72,11 +72,11 @@ class MapEditor final : public Editor
 {
     Q_OBJECT
 
-    Q_PROPERTY(Tiled::TilesetDock *tilesetsView READ tilesetDock)
-    Q_PROPERTY(Tiled::EditableMap *currentBrush READ currentBrush WRITE setCurrentBrush)
+    Q_PROPERTY(Tiled::TilesetDock *tilesetsView READ tilesetDock CONSTANT)
+    Q_PROPERTY(Tiled::EditableMap *currentBrush READ currentBrush WRITE setCurrentBrush NOTIFY currentBrushChanged)
     Q_PROPERTY(Tiled::EditableWangSet *currentWangSet READ currentWangSet NOTIFY currentWangSetChanged)
     Q_PROPERTY(int currentWangColorIndex READ currentWangColorIndex NOTIFY currentWangColorIndexChanged)
-    Q_PROPERTY(Tiled::MapView *currentMapView READ currentMapView)
+    Q_PROPERTY(Tiled::MapView *currentMapView READ currentMapView CONSTANT)
 
 public:
     explicit MapEditor(QObject *parent = nullptr);
@@ -130,6 +130,7 @@ public:
     AbstractTool *selectedTool() const;
 
 signals:
+    void currentBrushChanged();
     void currentWangSetChanged();
     void currentWangColorIndexChanged(int colorIndex);
 
@@ -159,6 +160,7 @@ private:
     void updateLayerComboIndex();
 
     void setupQuickStamps();
+    void setUseOpenGL(bool useOpenGL);
     void retranslateUi();
     void showTileCollisionShapesChanged(bool enabled);
     void parallaxEnabledChanged(bool enabled);
