@@ -26,7 +26,6 @@
 #include "mainwindow.h"
 #include "stylehelper.h"
 #include "tiledproxystyle.h"
-#include "utils.h"
 
 #include <QAction>
 #include <QApplication>
@@ -41,15 +40,15 @@ NoEditorWidget::NoEditorWidget(QWidget *parent) :
 {
     ui->setupUi(this);
 
+    ui->logo->setPixmap(QPixmap(QString::fromUtf8(":/images/about-tiled-logo.png")));
+
     auto opacityEffect = new QGraphicsOpacityEffect(this);
     opacityEffect->setOpacity(0.25);
     ui->logo->setGraphicsEffect(opacityEffect);
 
     ui->versionLabel->setText(QStringLiteral("%1 %2").arg(QGuiApplication::applicationDisplayName(), QGuiApplication::applicationVersion()));
 
-    connect(ui->openProjectButton, &QToolButton::clicked, ActionManager::action("OpenProject"), &QAction::trigger);
-    connect(ui->saveProjectButton, &QToolButton::clicked, ActionManager::action("SaveProjectAs"), &QAction::trigger);
-    connect(ui->addFolderToProjectButton, &QToolButton::clicked, ActionManager::action("AddFolderToProject"), &QAction::trigger);
+    connect(ui->newProjectButton, &QToolButton::clicked, ActionManager::action("NewProject"), &QAction::trigger);
 
     connect(ui->newMapButton, &QToolButton::clicked, this, &NoEditorWidget::newMap);
     connect(ui->newTilesetButton, &QToolButton::clicked, this, &NoEditorWidget::newTileset);
@@ -100,14 +99,13 @@ void NoEditorWidget::openFile()
 
 void NoEditorWidget::retranslateUi()
 {
-    ui->openProjectButton->setText(ActionManager::action("OpenProject")->text());
-    ui->saveProjectButton->setText(ActionManager::action("SaveProjectAs")->text());
-    ui->addFolderToProjectButton->setText(ActionManager::action("AddFolderToProject")->text());
+    ui->newProjectButton->setText(ActionManager::action("NewProject")->text());
+    ui->openFileButton->setText(ActionManager::action("Open")->text());
 }
 
 void NoEditorWidget::updateRecentProjectsMenu()
 {
-    auto menu = ui->openProjectButton->menu();
+    auto menu = ui->recentProjectsButton->menu();
     if (!menu)
         menu = new QMenu(this);
 
@@ -115,15 +113,8 @@ void NoEditorWidget::updateRecentProjectsMenu()
 
     bool enabled = MainWindow::instance()->addRecentProjectsActions(menu);
 
-    if (enabled) {
-        ui->openProjectButton->setMenu(menu);
-    } else {
-        ui->openProjectButton->setMenu(nullptr);
-        delete menu;
-    }
-
-    ui->openProjectButton->setPopupMode(enabled ? QToolButton::MenuButtonPopup
-                                                : QToolButton::DelayedPopup);
+    ui->recentProjectsButton->setMenu(menu);
+    ui->recentProjectsButton->setEnabled(enabled);
 }
 
 void NoEditorWidget::adjustToStyle()
