@@ -85,11 +85,12 @@ public:
     QRectF boundingRect() const override { return QRectF(); }
     void paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *) override {}
 
+protected:
+    QVariant itemChange(GraphicsItemChange change, const QVariant &value) override;
+
 private:
     void changeEvent(const ChangeEvent &event);
-    void propertyAdded(Object *object, const QString &name);
     void propertyRemoved(Object *object, const QString &name);
-    void propertyChanged(Object *object, const QString &name);
     void propertiesChanged(Object *object);
     void selectedObjectsChanged();
     void aboutToBeSelectedObjectsChanged();
@@ -100,6 +101,7 @@ private:
     void layerChanged(const LayerChangeEvent &event);
     void syncOverlayItems(const QList<MapObject *> &objects);
     void updateItemColors() const;
+    void updateItemColorsForObject(MapObject *mapObject) const;
     void objectsAdded(const QList<MapObject*> &objects);
     void objectsAboutToBeRemoved(const QList<MapObject*> &objects);
     void tilesetTilePositioningChanged(Tileset *tileset);
@@ -109,6 +111,8 @@ private:
     void showObjectReferencesChanged();
     void objectLineWidthChanged();
 
+    void sceneFontChanged();
+
     void addRemoveObjectLabels();
     void addRemoveObjectOutlines();
     void addRemoveObjectHoverItems();
