@@ -104,7 +104,7 @@ private:
     void startMoving(const QPointF &pos, Qt::KeyboardModifiers modifiers);
     void updateMovingItems(const QPointF &pos,
                            Qt::KeyboardModifiers modifiers);
-    void finishMoving(const QPointF &pos);
+    void finishMoving();
 
     void startMovingOrigin(const QPointF &pos);
     void updateMovingOrigin(const QPointF &pos, Qt::KeyboardModifiers modifiers);
@@ -113,7 +113,7 @@ private:
     void startRotating(const QPointF &pos);
     void updateRotatingItems(const QPointF &pos,
                              Qt::KeyboardModifiers modifiers);
-    void finishRotating(const QPointF &pos);
+    void finishRotating();
 
     void startResizing();
     void updateResizingItems(const QPointF &pos,
@@ -121,18 +121,18 @@ private:
     void updateResizingSingleItem(const QPointF &resizingOrigin,
                                   const QPointF &screenPos,
                                   Qt::KeyboardModifiers modifiers);
-    void finishResizing(const QPointF &pos);
+    void finishResizing();
 
     void setMode(Mode mode);
     void saveSelectionState();
 
     enum AbortReason {
         UserInteraction,
+        ObjectsRemoved,
         Deactivated
     };
 
-    void abortCurrentAction(AbortReason reason = UserInteraction,
-                            const QList<MapObject *> &removedObjects = QList<MapObject*>());
+    void abortCurrentAction(AbortReason reason);
 
     void refreshCursor();
 
