@@ -29,19 +29,12 @@
 
 using namespace Tiled;
 
-PaintTileLayer::PaintTileLayer(MapDocument *mapDocument,
-                               TileLayer *target,
-                               int x,
-                               int y,
-                               const TileLayer *source,
-                               QUndoCommand *parent)
-    : PaintTileLayer(mapDocument,
-                     target,
-                     x, y,
-                     source,
-                     source->region().translated(QPoint(x, y) - source->position()),
-                     parent)
+PaintTileLayer::PaintTileLayer(MapDocument *mapDocument, QUndoCommand *parent)
+    : QUndoCommand(parent)
+    , mMapDocument(mapDocument)
+    , mMergeable(false)
 {
+    setText(QCoreApplication::translate("Undo Commands", "Paint"));
 }
 
 PaintTileLayer::PaintTileLayer(MapDocument *mapDocument,
@@ -51,31 +44,43 @@ PaintTileLayer::PaintTileLayer(MapDocument *mapDocument,
                                const TileLayer *source,
                                const QRegion &paintRegion,
                                QUndoCommand *parent)
-    : QUndoCommand(parent)
-    , mMapDocument(mapDocument)
-    , mMergeable(false)
+    : PaintTileLayer(mapDocument, parent)
 {
-    auto &data = mLayerData[target];
-
-    data.mSource.reset(source->clone());
-    data.mErased.reset(new TileLayer);
-    data.mErased->setCells(target->x(), target->y(), target, paintRegion);
-    data.mX = x;
-    data.mY = y;
-    data.mPaintedRegion = paintRegion;
-
-    setText(QCoreApplication::translate("Undo Commands", "Paint"));
+    paint(target, x, y, source, paintRegion);
 }
 
 PaintTileLayer::~PaintTileLayer()
 {
 }
 
+void PaintTileLayer::paint(TileLayer *target,
+                           int x,
+                           int y,
+                           const TileLayer *source,
+                           const QRegion &paintRegion)
+{
+    PaintTileLayer::LayerData data;
+    data.mSource = std::make_unique<TileLayer>();
+    data.mSource->setCells(x + target->x(),
+                           y + target->y(), source, paintRegion);
+    data.mErased = std::make_unique<TileLayer>();
+    data.mErased->setCells(target->x(),
+                           target->y(), target, paintRegion);
+    data.mPaintedRegion = paintRegion;
+
+    mLayerData[target].mergeWith(std::move(data));
+}
+
+void PaintTileLayer::erase(TileLayer *target, const QRegion &eraseRegion)
+{
+    TileLayer empty;
+    paint(target, 0, 0, &empty, eraseRegion);
+}
+
 void PaintTileLayer::undo()
 {
-    for (const std::pair<TileLayer* const, LayerData> &entry : mLayerData) {
-        const LayerData &data = entry.second;
-        TilePainter painter(mMapDocument, entry.first);
+    for (const auto& [tileLayer, data] : mLayerData) {
+        TilePainter painter(mMapDocument, tileLayer);
         painter.setCells(0, 0, data.mErased.get(), data.mPaintedRegion);
     }
 
@@ -86,44 +91,38 @@ void PaintTileLayer::redo()
 {
     QUndoCommand::redo(); // redo child commands
 
-    for (const std::pair<TileLayer* const, LayerData> &entry : mLayerData) {
-        const LayerData &data = entry.second;
-        TilePainter painter(mMapDocument, entry.first);
-        painter.setCells(data.mX, data.mY, data.mSource.get(), data.mPaintedRegion);
+    for (const auto& [tileLayer, data] : mLayerData) {
+        TilePainter painter(mMapDocument, tileLayer);
+        painter.setCells(0, 0, data.mSource.get(), data.mPaintedRegion);
     }
 }
 
-void PaintTileLayer::LayerData::mergeWith(const PaintTileLayer::LayerData &o)
+void PaintTileLayer::LayerData::mergeWith(const LayerData &o)
 {
     if (!mSource) {
         mSource.reset(o.mSource->clone());
         mErased.reset(o.mErased->clone());
-        mX = o.mX;
-        mY = o.mY;
         mPaintedRegion = o.mPaintedRegion;
         return;
     }
 
-    const QRegion combinedRegion = mPaintedRegion.united(o.mPaintedRegion);
-    const QRegion newRegion = combinedRegion.subtracted(mPaintedRegion);
+    copy(o);
+}
 
-    // Copy the painted tiles from the other command over
-    mPaintedRegion = combinedRegion;
-    mSource->setCells(o.mX - mSource->x(),
-                      o.mY - mSource->y(),
-                      o.mSource.get(),
-                      o.mPaintedRegion.translated(-mSource->position()));
+void PaintTileLayer::LayerData::mergeWith(LayerData &&o)
+{
+    if (!mSource)
+        *this = std::move(o);
+    else
+        copy(o);
+}
 
-    // Copy the newly erased tiles from the other command over
-#if QT_VERSION >= 0x050800
-    for (const QRect &rect : newRegion)
-#else
-    const auto rects = newRegion.rects();
-    for (const QRect &rect : rects)
-#endif
-        for (int y = rect.top(); y <= rect.bottom(); ++y)
-            for (int x = rect.left(); x <= rect.right(); ++x)
-                mErased->setCell(x, y, o.mErased->cellAt(x, y));
+void PaintTileLayer::LayerData::copy(const LayerData &o)
+{
+    // Copy the newly painted tiles as well as the newly erased tiles over
+    mSource->setCells(0, 0, o.mSource.get(), o.mPaintedRegion);
+    mErased->setCells(0, 0, o.mErased.get(), o.mPaintedRegion - mPaintedRegion);
+    mPaintedRegion |= o.mPaintedRegion;
 }
 
 bool PaintTileLayer::mergeWith(const QUndoCommand *other)
@@ -134,8 +133,8 @@ bool PaintTileLayer::mergeWith(const QUndoCommand *other)
     if (!cloneChildren(other, this))
         return false;
 
-    for (const std::pair<TileLayer* const, LayerData> &entry : o->mLayerData)
-        mLayerData[entry.first].mergeWith(entry.second);
+    for (const auto& [tileLayer, data] : o->mLayerData)
+        mLayerData[tileLayer].mergeWith(data);
 
     return true;
 }
