@@ -29,7 +29,6 @@ class PreferencesDialog;
 }
 
 namespace Tiled {
-namespace Internal {
 
 /**
  * The preferences dialog. Allows the user to configure some general behaviour
@@ -41,28 +40,23 @@ class PreferencesDialog : public QDialog
 
 public:
     PreferencesDialog(QWidget *parent = nullptr);
-    ~PreferencesDialog();
+    ~PreferencesDialog() override;
 
 protected:
     void changeEvent(QEvent *e) override;
 
-private slots:
+private:
     void languageSelected(int index);
 
-private:
     void fromPreferences();
 
     void retranslateUi();
 
     void styleComboChanged();
 
-    void autoUpdateToggled(bool checked);
-    void checkForUpdates();
-
     Ui::PreferencesDialog *mUi;
     QStringList mLanguages;
 };
 
 
-} // namespace Internal
 } // namespace Tiled
