@@ -1,6 +1,6 @@
 /*
  * propertytypeseditor.cpp
- * Copyright 2016-2021, Thorbjørn Lindeijer <bjorn@lindeijer.nl>>
+ * Copyright 2016-2022, Thorbjørn Lindeijer <bjorn@lindeijer.nl>>
  *
  * This file is part of Tiled.
  *
@@ -22,8 +22,9 @@
 #include "ui_propertytypeseditor.h"
 
 #include "addpropertydialog.h"
+#include "colorbutton.h"
 #include "custompropertieshelper.h"
-#include "object.h"
+#include "objecttypes.h"
 #include "preferences.h"
 #include "project.h"
 #include "projectmanager.h"
@@ -31,8 +32,6 @@
 #include "savefile.h"
 #include "session.h"
 #include "utils.h"
-#include "varianteditorfactory.h"
-#include "variantpropertymanager.h"
 
 #include <QCheckBox>
 #include <QCloseEvent>
@@ -41,14 +40,18 @@
 #include <QFormLayout>
 #include <QInputDialog>
 #include <QJsonDocument>
+#include <QMenu>
 #include <QMessageBox>
+#include <QPushButton>
 #include <QScopedValueRollback>
 #include <QStringListModel>
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
+#include <QStylePainter>
+#endif
 #include <QToolBar>
 
 #include <QtTreePropertyBrowser>
-
-#include "qtcompat_p.h"
+#include <QtVariantProperty>
 
 namespace Tiled {
 
@@ -67,17 +70,79 @@ static bool confirm(const QString &title, const QString& text, QWidget *parent)
                                 QMessageBox::No) == QMessageBox::Yes;
 }
 
+class DropDownPushButton : public QPushButton
+{
+public:
+    using QPushButton::QPushButton;
+
+    QSize sizeHint() const override
+    {
+        QStyleOptionButton option;
+        initStyleOption(&option);
+
+        QSize hint = QPushButton::sizeHint();
+        hint.rwidth() += style()->pixelMetric(QStyle::PM_MenuButtonIndicator, &option, this);
+        return hint;
+    }
+
+protected:
+#if QT_VERSION < QT_VERSION_CHECK(6,0,0)
+    void paintEvent(QPaintEvent *) override
+    {
+        QStyleOptionButton option;
+        initStyleOption(&option);
+
+        QStylePainter p(this);
+        p.drawControl(QStyle::CE_PushButton, option);
+    }
+
+    void initStyleOption(QStyleOptionButton *option) const
+#else
+    void initStyleOption(QStyleOptionButton *option) const override
+#endif
+    {
+        QPushButton::initStyleOption(option);
+        option->features |= QStyleOptionButton::HasMenu;
+    }
+};
+
+
+PropertyTypesFilter::PropertyTypesFilter(const QString &lastPath)
+    : propertyTypesFilter(QCoreApplication::translate("File Types", "Custom Types files (*.json)"))
+    , objectTypesJsonFilter(QCoreApplication::translate("File Types", "Object Types JSON (*.json)"))
+    , objectTypesXmlFilter(QCoreApplication::translate("File Types", "Object Types XML (*.xml)"))
+{
+    filters = QStringList { propertyTypesFilter, objectTypesJsonFilter, objectTypesXmlFilter }.join(QStringLiteral(";;"));
+    selectedFilter = lastPath.endsWith(QLatin1String(".xml"), Qt::CaseInsensitive) ? objectTypesXmlFilter
+                                                                                   : propertyTypesFilter;
+}
+
+
 PropertyTypesEditor::PropertyTypesEditor(QWidget *parent)
     : QDialog(parent)
     , mUi(new Ui::PropertyTypesEditor)
     , mPropertyTypesModel(new PropertyTypesModel(this))
+    , mDetailsLayout(new QFormLayout)
     , mValuesModel(new QStringListModel(this))
 {
     mUi->setupUi(this);
 
+    mDetailsLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
+
     resize(Utils::dpiScaled(size()));
 
     mUi->propertyTypesView->setModel(mPropertyTypesModel);
+    mUi->horizontalLayout->addLayout(mDetailsLayout);
+
+    mFlagsWithNames = {
+        { ClassPropertyType::MapClass,          tr("Map") },
+        { ClassPropertyType::LayerClass,        tr("Layer") },
+        { ClassPropertyType::MapObjectClass,    tr("Object") },
+        { ClassPropertyType::TileClass,         tr("Tile") },
+        { ClassPropertyType::TilesetClass,      tr("Tileset") },
+        { ClassPropertyType::WangColorClass,    tr("Terrain") },
+        { ClassPropertyType::WangSetClass,      tr("Terrain Set") },
+    };
 
     mAddEnumPropertyTypeAction = new QAction(this);
     mAddClassPropertyTypeAction = new QAction(this);
@@ -185,7 +250,7 @@ PropertyTypesEditor::PropertyTypesEditor(QWidget *parent)
 
     Preferences *prefs = Preferences::instance();
 
-    auto &project = ProjectManager::instance()->project();
+    const auto &project = ProjectManager::instance()->project();
     mPropertyTypesModel->setPropertyTypes(project.propertyTypes());
 
     connect(prefs, &Preferences::propertyTypesChanged,
@@ -232,9 +297,9 @@ void PropertyTypesEditor::retranslateUi()
     mRenameMemberAction->setText(tr("Rename Member"));
 
     mExportAction->setText(tr("Export..."));
-    mExportAction->setToolTip(tr("Export Property Types"));
+    mExportAction->setToolTip(tr("Export Types"));
     mImportAction->setText(tr("Import..."));
-    mImportAction->setToolTip(tr("Import Property Types"));
+    mImportAction->setToolTip(tr("Import Types"));
 }
 
 void PropertyTypesEditor::addPropertyType(PropertyType::Type type)
@@ -282,7 +347,7 @@ void PropertyTypesEditor::removeSelectedPropertyType()
 
 /**
  * Returns the index of the currently selected property type, or an invalid
- * index if no or multiple property types are selected.
+ * index if no or multiple types are selected.
  */
 QModelIndex PropertyTypesEditor::selectedPropertyTypeIndex() const
 {
@@ -296,6 +361,15 @@ PropertyType *PropertyTypesEditor::selectedPropertyType() const
     return mPropertyTypesModel->propertyTypeAt(selectedPropertyTypeIndex());
 }
 
+ClassPropertyType *PropertyTypesEditor::selectedClassPropertyType() const
+{
+    PropertyType *propertyType = selectedPropertyType();
+    if (!propertyType || !propertyType->isClass())
+        return nullptr;
+
+    return static_cast<ClassPropertyType*>(propertyType);
+}
+
 void PropertyTypesEditor::currentMemberItemChanged(QtBrowserItem *item)
 {
     mRemoveMemberAction->setEnabled(item);
@@ -314,7 +388,7 @@ void PropertyTypesEditor::propertyTypeNameChanged(const QModelIndex &index, cons
 void PropertyTypesEditor::applyMemberToSelectedType(const QString &name, const QVariant &value)
 {
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Class)
+    if (!propertyType || !propertyType->isClass())
         return;
 
     auto &classType = static_cast<ClassPropertyType&>(*propertyType);
@@ -338,7 +412,7 @@ void PropertyTypesEditor::propertyTypesChanged()
     if (mSettingPrefPropertyTypes)
         return;
 
-    auto &project = ProjectManager::instance()->project();
+    const auto &project = ProjectManager::instance()->project();
     mPropertyTypesModel->setPropertyTypes(project.propertyTypes());
 
     selectedPropertyTypesChanged();
@@ -350,7 +424,7 @@ void PropertyTypesEditor::setStorageType(EnumPropertyType::StorageType storageTy
         return;
 
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Enum)
+    if (!propertyType || !propertyType->isEnum())
         return;
 
     auto &enumType = static_cast<EnumPropertyType&>(*propertyType);
@@ -367,7 +441,7 @@ void PropertyTypesEditor::setValuesAsFlags(bool flags)
         return;
 
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Enum)
+    if (!propertyType || !propertyType->isEnum())
         return;
 
     auto &enumType = static_cast<EnumPropertyType&>(*propertyType);
@@ -402,7 +476,7 @@ static QString nextValueText(const EnumPropertyType &propertyType)
 void PropertyTypesEditor::addValue()
 {
     const PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Enum)
+    if (!propertyType || !propertyType->isEnum())
         return;
 
     const auto &enumType = *static_cast<const EnumPropertyType*>(propertyType);
@@ -425,7 +499,7 @@ void PropertyTypesEditor::addValue()
 void PropertyTypesEditor::removeValues()
 {
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Enum)
+    if (!propertyType || !propertyType->isEnum())
         return;
 
     if (!confirm(tr("Remove Values"),
@@ -441,19 +515,57 @@ void PropertyTypesEditor::removeValues()
 
 bool PropertyTypesEditor::checkValueCount(int count)
 {
-    if (count > 32) {
+    if (count > 31) {
         QMessageBox::critical(this,
                               tr("Too Many Values"),
-                              tr("Too many values for enum with values stored as flags. Maximum number of bit flags is 32."));
+                              tr("Too many values for enum with values stored as flags. Maximum number of bit flags is %1.").arg(31));
         return false;
     }
     return true;
 }
 
+void PropertyTypesEditor::openClassOfPopup()
+{
+    ClassPropertyType *classType = selectedClassPropertyType();
+    if (!classType)
+        return;
+
+    QFrame *popup = new QFrame(this, Qt::Popup);
+    popup->setAttribute(Qt::WA_DeleteOnClose);
+    popup->setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
+
+    QVBoxLayout *layout = new QVBoxLayout(popup);
+    const int space = Utils::dpiScaled(4);
+    layout->setSpacing(space);
+    layout->setContentsMargins(space, space, space, space);
+
+    for (auto &entry : mFlagsWithNames) {
+        auto checkBox = new QCheckBox(entry.name);
+        checkBox->setChecked(classType->usageFlags & entry.flag);
+        layout->addWidget(checkBox);
+
+        connect(checkBox, &QCheckBox::toggled,
+                this, [this, flag = entry.flag] (bool checked) {
+            setUsageFlags(flag, checked);
+        });
+    }
+
+    // Focus the first checkbox for convenient keyboard navigation
+    layout->itemAt(0)->widget()->setFocus();
+
+    const QSize size = popup->sizeHint();
+    popup->setGeometry(Utils::popupGeometry(mClassOfButton, size));
+    popup->show();
+
+    connect(popup, &QWidget::destroyed, this, [this] {
+        mClassOfButton->setDown(false);
+    });
+}
+
 void PropertyTypesEditor::openAddMemberDialog()
 {
     const PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Class)
+    if (!propertyType || !propertyType->isClass())
         return;
 
     AddPropertyDialog dialog(static_cast<const ClassPropertyType*>(propertyType), this);
@@ -469,7 +581,7 @@ void PropertyTypesEditor::addMember(const QString &name, const QVariant &value)
         return;
 
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Class)
+    if (!propertyType || !propertyType->isClass())
         return;
 
     auto &classType = static_cast<ClassPropertyType&>(*propertyType);
@@ -503,7 +615,7 @@ void PropertyTypesEditor::removeMember()
         return;
 
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Class)
+    if (!propertyType || !propertyType->isClass())
         return;
 
     const QString name = item->property()->propertyName();
@@ -563,7 +675,7 @@ void PropertyTypesEditor::renameMemberTo(const QString &name)
         return;
 
     auto propertyType = selectedPropertyType();
-    if (propertyType->type != PropertyType::PT_Class)
+    if (!propertyType || !propertyType->isClass())
         return;
 
     auto &classType = *static_cast<ClassPropertyType*>(propertyType);
@@ -586,36 +698,72 @@ void PropertyTypesEditor::renameMemberTo(const QString &name)
 void PropertyTypesEditor::importPropertyTypes()
 {
     Session &session = Session::current();
-    const QString lastPath = session.lastPath(Session::ObjectTypesFile);
+    const QString lastPath = session.lastPath(Session::PropertyTypesFile);
+
+    PropertyTypesFilter filter(lastPath);
+    // When importing, we don't use the "objectTypesJsonFilter". Instead, we
+    // will auto-detect the format of the JSON file.
+    const QString filters = QStringList { filter.propertyTypesFilter, filter.objectTypesXmlFilter }.join(QStringLiteral(";;"));
     const QString fileName =
-            QFileDialog::getOpenFileName(this, tr("Import Property Types"),
+            QFileDialog::getOpenFileName(this, tr("Import Types"),
                                          lastPath,
-                                         QCoreApplication::translate("File Types", "Property Types files (*.json)"));
+                                         filters,
+                                         &filter.selectedFilter);
     if (fileName.isEmpty())
         return;
 
-    session.setLastPath(Session::ObjectTypesFile, fileName);
+    session.setLastPath(Session::PropertyTypesFile, fileName);
 
-    QFile file(fileName);
-    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        const auto error = QCoreApplication::translate("File Errors", "Could not open file for reading.");
-        QMessageBox::critical(this, tr("Error Reading Property Types"), error);
-        return;
+    ObjectTypes objectTypes;
+    const ExportContext context(*mPropertyTypesModel->propertyTypes(),
+                                QFileInfo(fileName).path());
+
+    if (filter.selectedFilter == filter.objectTypesXmlFilter) {
+        ObjectTypesSerializer serializer(ObjectTypesSerializer::Xml);
+
+        if (!serializer.readObjectTypes(fileName, objectTypes, context)) {
+            QMessageBox::critical(this, tr("Error Reading Object Types"),
+                                  serializer.errorString());
+            return;
+        }
+    } else {
+        QFile file(fileName);
+        if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            const auto error = QCoreApplication::translate("File Errors", "Could not open file for reading.");
+            QMessageBox::critical(this, tr("Error Reading Types"), error);
+            return;
+        }
+
+        QJsonParseError jsonError;
+        const QJsonDocument document = QJsonDocument::fromJson(file.readAll(), &jsonError);
+        if (document.isNull()) {
+            QMessageBox::critical(this, tr("Error Reading Types"),
+                                  Utils::Error::jsonParseError(jsonError));
+            return;
+        }
+
+        // We can detect the old format by the absence of an "id" property:
+        //
+        //  Object Types: "[ { color, name, properties } ]"
+        //  Custom Types: "[ { id, name, type, ... } ]"
+        //
+        const QJsonArray array = document.array();
+        const bool oldFormat = array.first().toObject().value(QLatin1String("id")).isUndefined();
+        if (oldFormat) {
+            fromJson(array, objectTypes, context);
+        } else {
+            PropertyTypes typesToImport;
+            typesToImport.loadFromJson(array, QFileInfo(fileName).path());
+
+            if (typesToImport.count() > 0) {
+                mPropertyTypesModel->importPropertyTypes(std::move(typesToImport));
+                applyPropertyTypes();
+            }
+        }
     }
 
-    QJsonParseError jsonError;
-    const QJsonDocument document = QJsonDocument::fromJson(file.readAll(), &jsonError);
-    if (document.isNull()) {
-        QMessageBox::critical(this, tr("Error Reading Property Types"),
-                              Utils::Error::jsonParseError(jsonError));
-        return;
-    }
-
-    PropertyTypes typesToImport;
-    typesToImport.loadFromJson(document.array(), QFileInfo(fileName).path());
-
-    if (typesToImport.count() > 0) {
-        mPropertyTypesModel->importPropertyTypes(std::move(typesToImport));
+    if (!objectTypes.isEmpty()) {
+        mPropertyTypesModel->importObjectTypes(objectTypes);
         applyPropertyTypes();
     }
 }
@@ -623,33 +771,47 @@ void PropertyTypesEditor::importPropertyTypes()
 void PropertyTypesEditor::exportPropertyTypes()
 {
     Session &session = Session::current();
-    QString lastPath = session.lastPath(Session::ObjectTypesFile);
+    QString lastPath = session.lastPath(Session::PropertyTypesFile);
 
-    if (!lastPath.endsWith(QLatin1String(".json")))
+    if (!QFileInfo(lastPath).isFile())
         lastPath.append(QStringLiteral("/propertytypes.json"));
 
+    PropertyTypesFilter filter(lastPath);
     const QString fileName =
-            QFileDialog::getSaveFileName(this, tr("Export Property Types"),
+            QFileDialog::getSaveFileName(this, tr("Export Types"),
                                          lastPath,
-                                         QCoreApplication::translate("File Types", "Property Types files (*.json)"));
+                                         filter.filters,
+                                         &filter.selectedFilter);
     if (fileName.isEmpty())
         return;
 
-    session.setLastPath(Session::ObjectTypesFile, fileName);
-
-    SaveFile file(fileName);
-
-    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-        const auto error = QCoreApplication::translate("File Errors", "Could not open file for writing.");
-        QMessageBox::critical(this, tr("Error Writing Property Types"), error);
-        return;
-    }
+    session.setLastPath(Session::PropertyTypesFile, fileName);
 
     const auto types = mPropertyTypesModel->propertyTypes();
-    file.device()->write(QJsonDocument(types->toJson()).toJson());
 
-    if (!file.commit())
-        QMessageBox::critical(this, tr("Error Writing Property Types"), file.errorString());
+    if (filter.selectedFilter == filter.objectTypesJsonFilter ||
+            filter.selectedFilter == filter.objectTypesXmlFilter) {
+        ObjectTypesSerializer serializer;
+        const ObjectTypes objectTypes = toObjectTypes(*types);
+
+        if (!serializer.writeObjectTypes(fileName, objectTypes)) {
+            QMessageBox::critical(this, tr("Error Writing Object Types"),
+                                  serializer.errorString());
+        }
+    } else {
+        SaveFile file(fileName);
+
+        if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+            const auto error = QCoreApplication::translate("File Errors", "Could not open file for writing.");
+            QMessageBox::critical(this, tr("Error Writing Types"), error);
+            return;
+        }
+
+        file.device()->write(QJsonDocument(types->toJson()).toJson());
+
+        if (!file.commit())
+            QMessageBox::critical(this, tr("Error Writing Types"), file.errorString());
+    }
 }
 
 void PropertyTypesEditor::updateDetails()
@@ -666,10 +828,16 @@ void PropertyTypesEditor::updateDetails()
 
     switch (propertyType->type) {
     case PropertyType::PT_Invalid:
+        Q_UNREACHABLE();
         break;
     case PropertyType::PT_Class: {
         const auto &classType = *static_cast<const ClassPropertyType*>(propertyType);
 
+        mColorButton->setColor(classType.color);
+        mUseAsPropertyCheckBox->setChecked(classType.isPropertyValueType());
+        mDrawFillCheckBox->setChecked(classType.drawFill);
+        updateClassUsageDetails(classType);
+
         mPropertiesHelper->clear();
 
         QMapIterator<QString, QVariant> it(classType.members);
@@ -699,24 +867,34 @@ void PropertyTypesEditor::updateDetails()
     mNameEdit->setText(propertyType->name);
 }
 
+void PropertyTypesEditor::updateClassUsageDetails(const ClassPropertyType &classType)
+{
+    QScopedValueRollback<bool> updatingDetails(mUpdatingDetails, true);
+
+    mClassOfCheckBox->setChecked(classType.usageFlags & ClassPropertyType::AnyObjectClass);
+
+    QStringList selectedTypes;
+    for (const NamedFlag &namedFlag : std::as_const(mFlagsWithNames)) {
+        if (classType.usageFlags & namedFlag.flag)
+            selectedTypes.append(namedFlag.name);
+    }
+
+    if (selectedTypes.isEmpty()) {
+        mClassOfButton->setText(tr("Select Types"));
+    } else {
+        if (selectedTypes.size() > 3) {
+            selectedTypes.erase(selectedTypes.begin() + 3, selectedTypes.end());
+            selectedTypes.append(QStringLiteral("..."));
+        }
+        mClassOfButton->setText(selectedTypes.join(QStringLiteral(", ")));
+    }
+}
+
 void PropertyTypesEditor::selectedValuesChanged(const QItemSelection &selected)
 {
     mRemoveValueAction->setEnabled(!selected.isEmpty());
 }
 
-void deleteAllFromLayout(QLayout *layout)
-{
-    for (int i = layout->count() - 1; i >= 0; --i) {
-        QLayoutItem *item = layout->takeAt(i);
-        delete item->widget();
-
-        if (auto layout = item->layout())
-            deleteAllFromLayout(layout);
-
-        delete item;
-    }
-}
-
 void PropertyTypesEditor::setCurrentPropertyType(PropertyType::Type type)
 {
     if (mCurrentPropertyType == type)
@@ -727,15 +905,11 @@ void PropertyTypesEditor::setCurrentPropertyType(PropertyType::Type type)
     delete mPropertiesHelper;
     mPropertiesHelper = nullptr;
 
-    if (mDetailsLayout) {
-        deleteAllFromLayout(mDetailsLayout);
-        delete mDetailsLayout;
-    }
-
-    mDetailsLayout = new QFormLayout;
-    mUi->horizontalLayout->addLayout(mDetailsLayout);
+    while (mDetailsLayout->rowCount() > 0)
+        mDetailsLayout->removeRow(0);
 
     mNameEdit = nullptr;
+    mColorButton = nullptr;
     mStorageTypeComboBox = nullptr;
     mValuesAsFlagsCheckBox = nullptr;
     mValuesView = nullptr;
@@ -753,73 +927,117 @@ void PropertyTypesEditor::setCurrentPropertyType(PropertyType::Type type)
     connect(mNameEdit, &QLineEdit::editingFinished,
             this, &PropertyTypesEditor::nameEditingFinished);
 
-    mDetailsLayout->addRow(tr("Name"), mNameEdit);
-
     switch (type) {
     case PropertyType::PT_Invalid:
+        Q_UNREACHABLE();
         break;
-    case PropertyType::PT_Class: {
-        mMembersView = new QtTreePropertyBrowser(this);
-        mPropertiesHelper = new CustomPropertiesHelper(mMembersView, this);
-
-        connect(mPropertiesHelper, &CustomPropertiesHelper::propertyValueChanged,
-                this, &PropertyTypesEditor::memberValueChanged);
-
-        connect(mMembersView, &QtTreePropertyBrowser::currentItemChanged,
-                this, &PropertyTypesEditor::currentMemberItemChanged);
-
-        QToolBar *membersToolBar = createSmallToolBar(mUi->groupBox);
-        membersToolBar->addAction(mAddMemberAction);
-        membersToolBar->addAction(mRemoveMemberAction);
-        membersToolBar->addAction(mRenameMemberAction);
-
-        auto membersWithToolBarLayout = new QVBoxLayout;
-        membersWithToolBarLayout->setSpacing(0);
-        membersWithToolBarLayout->setContentsMargins(0, 0, 0, 0);
-        membersWithToolBarLayout->addWidget(mMembersView);
-        membersWithToolBarLayout->addWidget(membersToolBar);
-
-        mDetailsLayout->addRow(tr("Members"), membersWithToolBarLayout);
+    case PropertyType::PT_Class:
+        addClassProperties();
+        break;
+    case PropertyType::PT_Enum:
+        addEnumProperties();
         break;
     }
-    case PropertyType::PT_Enum: {
-        mStorageTypeComboBox = new QComboBox(mUi->groupBox);
-        mStorageTypeComboBox->addItems({ tr("String"), tr("Number") });
+}
 
-        connect(mStorageTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
-                this, [this] (int index) { if (index != -1) setStorageType(static_cast<EnumPropertyType::StorageType>(index)); });
+void PropertyTypesEditor::addClassProperties()
+{
+    mColorButton = new ColorButton(mUi->groupBox);
+    mColorButton->setToolTip(tr("Color"));
+    mColorButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
+    connect(mColorButton, &ColorButton::colorChanged,
+            this, &PropertyTypesEditor::colorChanged);
 
-        mValuesAsFlagsCheckBox = new QCheckBox(tr("Allow multiple values (flags)"), mUi->groupBox);
+    mDrawFillCheckBox = new QCheckBox(tr("Draw fill"));
+    connect(mDrawFillCheckBox, &QCheckBox::toggled,
+            this, &PropertyTypesEditor::setDrawFill);
+    auto nameAndColor = new QHBoxLayout;
+    nameAndColor->addWidget(mNameEdit);
+    nameAndColor->addWidget(mColorButton);
+    nameAndColor->addWidget(mDrawFillCheckBox);
 
-        connect(mValuesAsFlagsCheckBox, &QCheckBox::toggled,
-                this, [this] (bool checked) { setValuesAsFlags(checked); });
+    mMembersView = new QtTreePropertyBrowser(this);
+    mPropertiesHelper = new CustomPropertiesHelper(mMembersView, this);
 
-        mValuesView = new QTreeView(this);
-        mValuesView->setRootIsDecorated(false);
-        mValuesView->setUniformRowHeights(true);
-        mValuesView->setHeaderHidden(true);
-        mValuesView->setSelectionMode(QAbstractItemView::ExtendedSelection);
-        mValuesView->setModel(mValuesModel);
+    connect(mPropertiesHelper, &CustomPropertiesHelper::propertyMemberValueChanged,
+            this, &PropertyTypesEditor::memberValueChanged);
 
-        connect(mValuesView->selectionModel(), &QItemSelectionModel::selectionChanged,
-                this, &PropertyTypesEditor::selectedValuesChanged);
+    connect(mMembersView, &QtTreePropertyBrowser::currentItemChanged,
+            this, &PropertyTypesEditor::currentMemberItemChanged);
 
-        QToolBar *valuesToolBar = createSmallToolBar(mUi->groupBox);
-        valuesToolBar->addAction(mAddValueAction);
-        valuesToolBar->addAction(mRemoveValueAction);
+    mUseAsPropertyCheckBox = new QCheckBox(tr("Property value"));
 
-        auto valuesWithToolBarLayout = new QVBoxLayout;
-        valuesWithToolBarLayout->setSpacing(0);
-        valuesWithToolBarLayout->setContentsMargins(0, 0, 0, 0);
-        valuesWithToolBarLayout->addWidget(mValuesView);
-        valuesWithToolBarLayout->addWidget(valuesToolBar);
+    connect(mUseAsPropertyCheckBox, &QCheckBox::toggled,
+            this, [this] (bool checked) { setUsageFlags(ClassPropertyType::PropertyValueType, checked); });
 
-        mDetailsLayout->addRow(tr("Save as"), mStorageTypeComboBox);
-        mDetailsLayout->addRow(QString(), mValuesAsFlagsCheckBox);
-        mDetailsLayout->addRow(tr("Values"), valuesWithToolBarLayout);
-        break;
-    }
-    }
+    mClassOfButton = new DropDownPushButton(tr("Select Types"));
+    mClassOfButton->setAutoDefault(false);
+    mClassOfCheckBox = new QCheckBox(tr("Class of"));
+
+    connect(mClassOfButton, &QToolButton::pressed, this, &PropertyTypesEditor::openClassOfPopup);
+    connect(mClassOfCheckBox, &QCheckBox::toggled,
+            this, [this] (bool checked) { setUsageFlags(ClassPropertyType::AnyObjectClass, checked); });
+
+    auto usageOptions = new QHBoxLayout;
+    usageOptions->addWidget(mUseAsPropertyCheckBox);
+    usageOptions->addSpacing(Utils::dpiScaled(20));
+    usageOptions->addWidget(mClassOfCheckBox);
+    usageOptions->addWidget(mClassOfButton);
+    usageOptions->addStretch();
+
+    QToolBar *membersToolBar = createSmallToolBar(mUi->groupBox);
+    membersToolBar->addAction(mAddMemberAction);
+    membersToolBar->addAction(mRemoveMemberAction);
+    membersToolBar->addAction(mRenameMemberAction);
+
+    auto membersWithToolBarLayout = new QVBoxLayout;
+    membersWithToolBarLayout->setSpacing(0);
+    membersWithToolBarLayout->setContentsMargins(0, 0, 0, 0);
+    membersWithToolBarLayout->addWidget(mMembersView);
+    membersWithToolBarLayout->addWidget(membersToolBar);
+
+    mDetailsLayout->addRow(tr("Name"), nameAndColor);
+    mDetailsLayout->addRow(tr("Use as"), usageOptions);
+    mDetailsLayout->addRow(tr("Members"), membersWithToolBarLayout);
+}
+
+void PropertyTypesEditor::addEnumProperties()
+{
+    mStorageTypeComboBox = new QComboBox(mUi->groupBox);
+    mStorageTypeComboBox->addItems({ tr("String"), tr("Number") });
+
+    connect(mStorageTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, [this] (int index) { if (index != -1) setStorageType(static_cast<EnumPropertyType::StorageType>(index)); });
+
+    mValuesAsFlagsCheckBox = new QCheckBox(tr("Allow multiple values (flags)"), mUi->groupBox);
+
+    connect(mValuesAsFlagsCheckBox, &QCheckBox::toggled,
+            this, [this] (bool checked) { setValuesAsFlags(checked); });
+
+    mValuesView = new QTreeView(this);
+    mValuesView->setRootIsDecorated(false);
+    mValuesView->setUniformRowHeights(true);
+    mValuesView->setHeaderHidden(true);
+    mValuesView->setSelectionMode(QAbstractItemView::ExtendedSelection);
+    mValuesView->setModel(mValuesModel);
+
+    connect(mValuesView->selectionModel(), &QItemSelectionModel::selectionChanged,
+            this, &PropertyTypesEditor::selectedValuesChanged);
+
+    QToolBar *valuesToolBar = createSmallToolBar(mUi->groupBox);
+    valuesToolBar->addAction(mAddValueAction);
+    valuesToolBar->addAction(mRemoveValueAction);
+
+    auto valuesWithToolBarLayout = new QVBoxLayout;
+    valuesWithToolBarLayout->setSpacing(0);
+    valuesWithToolBarLayout->setContentsMargins(0, 0, 0, 0);
+    valuesWithToolBarLayout->addWidget(mValuesView);
+    valuesWithToolBarLayout->addWidget(valuesToolBar);
+
+    mDetailsLayout->addRow(tr("Name"), mNameEdit);
+    mDetailsLayout->addRow(tr("Save as"), mStorageTypeComboBox);
+    mDetailsLayout->addRow(QString(), mValuesAsFlagsCheckBox);
+    mDetailsLayout->addRow(tr("Values"), valuesWithToolBarLayout);
 }
 
 void PropertyTypesEditor::selectFirstPropertyType()
@@ -841,7 +1059,7 @@ void PropertyTypesEditor::valuesChanged()
         return;
 
     PropertyType *propertyType = selectedPropertyType();
-    if (!propertyType || propertyType->type != PropertyType::PT_Enum)
+    if (!propertyType || !propertyType->isEnum())
         return;
 
     const QStringList newValues = mValuesModel->stringList();
@@ -853,6 +1071,13 @@ void PropertyTypesEditor::valuesChanged()
 
 void PropertyTypesEditor::nameEditingFinished()
 {
+    // The dialog that might pop up when setPropertyTypeName fails can cause
+    // another "editingFinished" signal to get emitted from the QLineEdit due
+    // to losing focus. This needs to be ignored to prevent lockup in the UI
+    // with two modal dialogs (issue #3380).
+    if (mSettingName)
+        return;
+
     const auto index = selectedPropertyTypeIndex();
     if (!index.isValid())
         return;
@@ -865,12 +1090,63 @@ void PropertyTypesEditor::nameEditingFinished()
         mNameEdit->setText(type->name);
 }
 
-void PropertyTypesEditor::memberValueChanged(const QString &name, const QVariant &value)
+void PropertyTypesEditor::colorChanged(const QColor &color)
 {
     if (mUpdatingDetails)
         return;
 
-    applyMemberToSelectedType(name, value);
+    if (ClassPropertyType *classType = selectedClassPropertyType()) {
+        classType->color = color;
+        applyPropertyTypes();
+    }
+}
+
+void PropertyTypesEditor::setDrawFill(bool value)
+{
+    if (mUpdatingDetails)
+        return;
+
+    if (ClassPropertyType *classType = selectedClassPropertyType()) {
+        classType->drawFill = value;
+        applyPropertyTypes();
+    }
+}
+
+void PropertyTypesEditor::setUsageFlags(int flags, bool value)
+{
+    if (mUpdatingDetails)
+        return;
+
+    if (ClassPropertyType *classType = selectedClassPropertyType()) {
+        classType->setUsageFlags(flags, value);
+        updateClassUsageDetails(*classType);
+        applyPropertyTypes();
+    }
+}
+
+void PropertyTypesEditor::memberValueChanged(const QStringList &path, const QVariant &value)
+{
+    if (mUpdatingDetails)
+        return;
+
+    ClassPropertyType *classType = selectedClassPropertyType();
+    if (!classType)
+        return;
+
+    if (!setPropertyMemberValue(classType->members, path, value))
+        return;
+
+    // When a nested property was changed, we need to update the value of the
+    // top-level property to match.
+    if (path.size() > 1) {
+        auto &topLevelName = path.first();
+        if (auto property = mPropertiesHelper->property(topLevelName)) {
+            QScopedValueRollback<bool> updatingDetails(mUpdatingDetails, true);
+            property->setValue(mPropertiesHelper->toDisplayValue(classType->members.value(topLevelName)));
+        }
+    }
+
+    applyPropertyTypes();
 }
 
 } // namespace Tiled
