@@ -20,9 +20,9 @@
 
 #pragma once
 
-#include "documentmanager.h"
 #include "id.h"
 #include "issuesdock.h"
+#include "properties.h"
 
 #include <QJSValue>
 #include <QObject>
@@ -34,6 +34,7 @@ class QAction;
 
 namespace Tiled {
 
+class Document;
 class EditableAsset;
 class MapEditor;
 class ScriptImage;
@@ -53,10 +54,13 @@ class ScriptModule : public QObject
     Q_OBJECT
 
     Q_PROPERTY(QString version READ version)
+    Q_PROPERTY(QString qtVersion READ qtVersion)
     Q_PROPERTY(QString platform READ platform)
     Q_PROPERTY(QString arch READ arch)
     Q_PROPERTY(QString extensionsPath READ extensionsPath)
     Q_PROPERTY(QString applicationDirPath READ applicationDirPath)
+    Q_PROPERTY(QString projectFilePath READ projectFilePath)
+    Q_PROPERTY(QStringList scriptArguments READ scriptArguments)
 
     Q_PROPERTY(QStringList actions READ actions)
     Q_PROPERTY(QStringList menus READ menus)
@@ -65,19 +69,26 @@ class ScriptModule : public QObject
 
     Q_PROPERTY(Tiled::EditableAsset *activeAsset READ activeAsset WRITE setActiveAsset NOTIFY activeAssetChanged)
     Q_PROPERTY(QList<QObject*> openAssets READ openAssets)
+    Q_PROPERTY(Tiled::EditableAsset *project READ project)
 
     Q_PROPERTY(Tiled::MapEditor *mapEditor READ mapEditor)
     Q_PROPERTY(Tiled::TilesetEditor *tilesetEditor READ tilesetEditor)
+    Q_PROPERTY(QList<QObject*> worlds READ worlds)
 
 public:
     ScriptModule(QObject *parent = nullptr);
     ~ScriptModule() override;
 
     QString version() const;
+    QString qtVersion() const;
     QString platform() const;
     QString arch() const;
     QString extensionsPath() const;
     QString applicationDirPath() const;
+    QString projectFilePath() const;
+
+    QStringList scriptArguments() const;
+    void setScriptArguments(const QStringList &arguments);
 
     QStringList actions() const;
     QStringList menus() const;
@@ -89,12 +100,21 @@ public:
 
     QList<QObject*> openAssets() const;
 
+    EditableAsset *project();
+
     TilesetEditor *tilesetEditor() const;
     MapEditor *mapEditor() const;
 
+    Q_INVOKABLE QColor color(const QString &name) const;
+    Q_INVOKABLE QColor color(float r, float g, float b, float a = 1.0f) const;
     Q_INVOKABLE Tiled::FilePath filePath(const QUrl &path) const;
     Q_INVOKABLE Tiled::ObjectRef objectRef(int id) const;
-    Q_INVOKABLE QVariant propertyValue(const QString &typeName, const QVariant &value) const;
+    Q_INVOKABLE QVariant propertyValue(const QString &typeName, const QJSValue &value) const;
+    Q_INVOKABLE QCursor cursor(Qt::CursorShape shape);
+    Q_INVOKABLE QCursor cursor(Tiled::ScriptImage *image, int hotX = -1, int hotY = -1);
+
+    Q_INVOKABLE bool versionLessThan(const QString &a);
+    Q_INVOKABLE bool versionLessThan(const QString &a, const QString &b);
 
     Q_INVOKABLE Tiled::EditableAsset *open(const QString &fileName) const;
     Q_INVOKABLE bool close(Tiled::EditableAsset *asset) const;
@@ -112,15 +132,38 @@ public:
 
     Q_INVOKABLE void extendMenu(const QByteArray &idName, QJSValue items);
 
+    // Synchronized with Tiled::CompressionMethod
+    enum CompressionMethod {
+        Gzip,
+        Zlib,
+        Zstandard
+    };
+    Q_ENUM(CompressionMethod)
+
+    Q_INVOKABLE QByteArray compress(const QByteArray &data, CompressionMethod method = Zlib, int compressionLevel = -1);
+    Q_INVOKABLE QByteArray decompress(const QByteArray &data, CompressionMethod method = Zlib);
+    Q_INVOKABLE QString promptDirectory(const QString &defaultDir = QString(), const QString &title = QString()) const;
+    Q_INVOKABLE QStringList promptOpenFiles(const QString &defaultDir = QString(),  const QString &filters = QString(), const QString &title = QString()) const;
+    Q_INVOKABLE QString promptOpenFile(const QString &defaultDir = QString(), const QString &filters = QString(), const QString &title = QString()) const;
+    Q_INVOKABLE QString promptSaveFile(const QString &defaultDir = QString(), const QString &filters = QString(),  const QString &title = QString()) const;
+
+    QList<QObject*> worlds() const;
+    Q_INVOKABLE void loadWorld(const QString &fileName) const;
+    Q_INVOKABLE void unloadWorld(const QString &fileName) const;
+    Q_INVOKABLE void unloadAllWorlds() const;
+
 signals:
     void assetCreated(Tiled::EditableAsset *asset);
     void assetOpened(Tiled::EditableAsset *asset);
+    void assetReloaded(Tiled::EditableAsset *asset);
     void assetAboutToBeSaved(Tiled::EditableAsset *asset);
     void assetSaved(Tiled::EditableAsset *asset);
     void assetAboutToBeClosed(Tiled::EditableAsset *asset);
 
     void activeAssetChanged(Tiled::EditableAsset *asset);
 
+    void worldsChanged();
+
 public slots:
     void trigger(const QByteArray &actionName) const;
     void executeCommand(const QString &name, bool inTerminal = false) const;
@@ -137,6 +180,7 @@ public slots:
 private:
     void documentCreated(Document *document);
     void documentOpened(Document *document);
+    void documentReloaded(Document *document);
     void documentAboutToBeSaved(Document *document);
     void documentSaved(Document *document);
     void documentAboutToClose(Document *document);
@@ -148,6 +192,13 @@ private:
     std::map<QString, std::unique_ptr<ScriptedMapFormat>> mRegisteredMapFormats;
     std::map<QString, std::unique_ptr<ScriptedTilesetFormat>> mRegisteredTilesetFormats;
     std::map<Id, std::unique_ptr<ScriptedTool>> mRegisteredTools;
+
+    QStringList mScriptArguments;
 };
 
+inline bool ScriptModule::versionLessThan(const QString &a)
+{
+    return versionLessThan(version(), a);
+}
+
 } // namespace Tiled
