@@ -33,6 +33,9 @@ class ShapeFillTool : public AbstractTileFillTool
 public:
     ShapeFillTool(QObject *parent = nullptr);
 
+    void activate(MapScene *scene) override;
+    void deactivate(MapScene *scene) override;
+
     void mousePressed(QGraphicsSceneMouseEvent *event) override;
     void mouseReleased(QGraphicsSceneMouseEvent *event) override;
 
@@ -67,6 +70,8 @@ private:
     QAction *mRectFill;
     QAction *mCircleFill;
 
+    void setActionsEnabled(bool enabled);
+
     void setCurrentShape(Shape shape);
     void updateFillOverlay();
 };
