@@ -20,12 +20,17 @@
 
 #pragma once
 
+#include "tilededitor_global.h"
+
+#include <QFont>
+#include <QObject>
 #include <QPalette>
-#include <QString>
+
+#include <optional>
 
 namespace Tiled {
 
-class StyleHelper : public QObject
+class TILED_EDITOR_EXPORT StyleHelper : public QObject
 {
     Q_OBJECT
 
@@ -36,6 +41,8 @@ public:
     const QString &defaultStyle() { return mDefaultStyle; }
     const QPalette &defaultPalette() { return mDefaultPalette; }
 
+    void applyFont();
+
 signals:
     void styleApplied();
 
@@ -44,8 +51,10 @@ private:
 
     void apply();
 
-    QString mDefaultStyle;
-    QPalette mDefaultPalette;
+    const QString mDefaultStyle;
+    const QPalette mDefaultPalette;
+    std::optional<QFont> mDefaultFont;
+    const bool mDefaultShowShortcutsInContextMenus = true;
 
     static StyleHelper *mInstance;
 };
