@@ -23,18 +23,17 @@
 
 #include "documentmanager.h"
 #include "editpolygontool.h"
-#include "mapdocumentactionhandler.h"
 #include "mapscene.h"
 #include "mapview.h"
 #include "objectgroup.h"
 #include "objectselectiontool.h"
+#include "objecttemplate.h"
 #include "propertiesdock.h"
 #include "replacetileset.h"
 #include "session.h"
 #include "templatemanager.h"
-#include "tilesetmanager.h"
 #include "tilesetdocument.h"
-#include "tmxmapformat.h"
+#include "tilesetmanager.h"
 #include "toolmanager.h"
 #include "utils.h"
 
@@ -64,7 +63,7 @@ TemplatesDock::TemplatesDock(QWidget *parent)
     , mUndoAction(new QAction(this))
     , mRedoAction(new QAction(this))
     , mMapScene(new MapScene(this))
-    , mMapView(new MapView(this, MapView::NoStaticContents))
+    , mMapView(new MapView(this))
     , mToolManager(new ToolManager(this))
 {
     setObjectName(QLatin1String("TemplatesDock"));
