@@ -38,9 +38,6 @@ TextEditorDialog::TextEditorDialog(QWidget *parent)
 {
     mUi->setupUi(this);
     resize(Utils::dpiScaled(size()));
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
-#endif
 
     connect(mUi->monospaceFont, &QAbstractButton::toggled, this, [this] (bool checked) {
         mUi->plainTextEdit->setFont(checked ? QFontDatabase::systemFont(QFontDatabase::FixedFont)
