@@ -21,12 +21,18 @@
 #pragma once
 
 #include "projectmanager.h"
+#include "tilededitor_global.h"
 
 #include <QtSingleApplication>
 
+#include <memory>
+
 namespace Tiled {
 
-class TiledApplication : public QtSingleApplication
+class NewVersionChecker;
+class NewsFeed;
+
+class TILED_EDITOR_EXPORT TiledApplication : public QtSingleApplication
 {
     Q_OBJECT
 
@@ -34,6 +40,9 @@ public:
     TiledApplication(int &argc, char **argv);
     ~TiledApplication() override;
 
+    NewVersionChecker &newVersionChecker();
+    NewsFeed &newsFeed();
+
 protected:
     bool event(QEvent *) override;
 
@@ -44,6 +53,14 @@ private:
     void onMessageReceived(const QString &message);
 
     ProjectManager mProjectManager;
+
+    NewVersionChecker *mNewVersionChecker = nullptr;
+    std::unique_ptr<NewsFeed> mNewsFeed;
 };
 
+inline TiledApplication *tiledApp()
+{
+    return static_cast<TiledApplication*>(QCoreApplication::instance());
+}
+
 } // namespace Tiled
