@@ -27,7 +27,6 @@
 #include "addremovetileset.h"
 #include "containerhelpers.h"
 #include "documentmanager.h"
-#include "editablemanager.h"
 #include "editabletile.h"
 #include "erasetiles.h"
 #include "map.h"
@@ -189,6 +188,7 @@ TilesetDock::TilesetDock(QWidget *parent)
     mSelectNextTileset->setShortcut(Qt::Key_BracketRight);
     mSelectPreviousTileset->setShortcut(Qt::Key_BracketLeft);
 
+    ActionManager::registerAction(mEditTileset, "EditTileset");
     ActionManager::registerAction(mSelectNextTileset, "SelectNextTileset");
     ActionManager::registerAction(mSelectPreviousTileset, "SelectPreviousTileset");
 
@@ -272,7 +272,7 @@ TilesetDock::TilesetDock(QWidget *parent)
     horizontal->addWidget(mZoomComboBox);
 
     connect(mViewStack, &QStackedWidget::currentChanged,
-            this, &TilesetDock::currentTilesetChanged);
+            this, &TilesetDock::onCurrentTilesetChanged);
 
     connect(TilesetManager::instance(), &TilesetManager::tilesetImagesChanged,
             this, &TilesetDock::tilesetChanged);
@@ -308,8 +308,8 @@ void TilesetDock::setMapDocument(MapDocument *mapDocument)
         return;
 
     // Hide while we update the tab bar, to avoid repeated layouting
-    // But, this causes problems on OS X (issue #1055)
-#ifndef Q_OS_OSX
+    // But, this causes problems on macOS (issue #1055)
+#ifndef Q_OS_MACOS
     widget()->hide();
 #endif
 
@@ -339,7 +339,7 @@ void TilesetDock::setMapDocument(MapDocument *mapDocument)
 
     updateActions();
 
-#ifndef Q_OS_OSX
+#ifndef Q_OS_MACOS
     widget()->show();
 #endif
 }
@@ -447,11 +447,13 @@ void TilesetDock::dropEvent(QDropEvent *e)
     }
 }
 
-void TilesetDock::currentTilesetChanged()
+void TilesetDock::onCurrentTilesetChanged()
 {
     TilesetView *view = currentTilesetView();
-    if (!view)
+    if (!view) {
+        emit currentTilesetChanged();
         return;
+    }
 
     if (!mSynchronizingSelection)
         updateCurrentTiles();
@@ -462,6 +464,8 @@ void TilesetDock::currentTilesetChanged()
         setCurrentTile(view->tilesetModel()->tileAt(s->currentIndex()));
 
     mDynamicWrappingToggle->setChecked(view->dynamicWrapping());
+
+    emit currentTilesetChanged();
 }
 
 void TilesetDock::selectionChanged()
@@ -597,6 +601,13 @@ void TilesetDock::createTilesetView(int index, TilesetDocument *tilesetDocument)
     mTabBar->insertTab(index, tileset->name());
     mTabBar->setTabToolTip(index, tileset->fileName());
 
+    // Workaround a bug that appears to have snug into Qt 6 which causes the
+    // tab bar to be entirely invisible if only a single tab exists.
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 4)
+    if (!mTabBar->isVisible())
+        mTabBar->updateGeometry();
+#endif
+
     connect(tilesetDocument, &TilesetDocument::fileNameChanged,
             this, &TilesetDock::tilesetFileNameChanged);
     connect(tilesetDocument, &TilesetDocument::tilesetChanged,
@@ -926,9 +937,8 @@ void TilesetDock::tabContextMenuRequested(const QPoint &pos)
     QMenu menu;
 
     auto tilesetDocument = mTilesetDocuments.at(index);
-    const QString fileName = tilesetDocument->fileName();
 
-    Utils::addFileManagerActions(menu, fileName);
+    Utils::addFileManagerActions(menu, tilesetDocument->fileName());
 
     menu.addSeparator();
     menu.addAction(mEditTileset->icon(), mEditTileset->text(), this, [tileset = tilesetDocument->tileset()] {
@@ -979,7 +989,7 @@ void TilesetDock::setCurrentEditableTileset(EditableTileset *tileset)
 
 EditableTileset *TilesetDock::currentEditableTileset() const
 {
-    const int index = mTabBar->currentIndex();
+    const int index = mViewStack->currentIndex();
     if (index == -1)
         return nullptr;
 
@@ -1021,10 +1031,9 @@ QList<QObject *> TilesetDock::selectedTiles() const
     EditableTileset *editableTileset = currentEditableTileset();
 
     const TilesetModel *model = view->tilesetModel();
-    auto &editableManager = EditableManager::instance();
     for (const QModelIndex &index : indexes)
         if (Tile *tile = model->tileAt(index))
-            result.append(editableManager.editableTile(editableTileset, tile));
+            result.append(EditableTile::get(editableTileset, tile));
 
     return result;
 }
@@ -1153,7 +1162,7 @@ void TilesetDock::embedTileset()
         return;
 
     // To embed a tileset we clone it, since further changes to that tileset
-    // should not affect the exteral tileset.
+    // should not affect the external tileset.
     SharedTileset embeddedTileset = tileset->clone();
 
     QUndoStack *undoStack = mMapDocument->undoStack();
