@@ -64,7 +64,7 @@ class TilesetDock : public QDockWidget
 {
     Q_OBJECT
 
-    Q_PROPERTY(Tiled::EditableTileset *currentTileset READ currentEditableTileset WRITE setCurrentEditableTileset)
+    Q_PROPERTY(Tiled::EditableTileset *currentTileset READ currentEditableTileset WRITE setCurrentEditableTileset NOTIFY currentTilesetChanged)
     Q_PROPERTY(QList<QObject*> selectedTiles READ selectedTiles WRITE setSelectedTiles)
 
 public:
@@ -98,6 +98,11 @@ public:
     QAction *actionSelectPreviousTileset() const { return mSelectPreviousTileset; }
 
 signals:
+    /**
+     * Emitted when the currently selected tileset changed.
+     */
+    void currentTilesetChanged();
+
     /**
      * Emitted when the current tile changed.
      */
@@ -121,7 +126,7 @@ protected:
     void dropEvent(QDropEvent *) override;
 
 private:
-    void currentTilesetChanged();
+    void onCurrentTilesetChanged();
     void selectionChanged();
     void currentChanged(const QModelIndex &index);
 
