@@ -27,8 +27,6 @@
 #include <QDebug>
 #include <QJsonArray>
 
-#include <qtcompat_p.h>
-
 namespace Tiled {
 
 class TileStampData : public QSharedData
@@ -62,7 +60,7 @@ TileStampData::TileStampData(const TileStampData &other)
 
 TileStampData::~TileStampData()
 {
-    for (const TileStampVariation &variation : qAsConst(variations))
+    for (const TileStampVariation &variation : std::as_const(variations))
         delete variation.map;
 }
 
@@ -135,10 +133,8 @@ void TileStamp::setProbability(int index, qreal probability)
 QSize TileStamp::maxSize() const
 {
     QSize size;
-    for (const TileStampVariation &variation : qAsConst(d->variations)) {
-        size.setWidth(qMax(size.width(), variation.map->width()));
-        size.setHeight(qMax(size.height(), variation.map->height()));
-    }
+    for (const TileStampVariation &variation : std::as_const(d->variations))
+        size = size.expandedTo(variation.map->size());
     return size;
 }
 
@@ -188,7 +184,7 @@ RandomPicker<Map *> TileStamp::randomVariations() const
     Q_ASSERT(!d->variations.isEmpty());
 
     RandomPicker<Map *> randomPicker;
-    for (const TileStampVariation &variation : qAsConst(d->variations))
+    for (const TileStampVariation &variation : std::as_const(d->variations))
         randomPicker.add(variation.map, variation.probability);
 
     return randomPicker;
@@ -295,7 +291,7 @@ QJsonObject TileStamp::toJson(const QDir &dir) const
         json.insert(QLatin1String("quickStampIndex"), d->quickStampIndex);
 
     QJsonArray variations;
-    for (const TileStampVariation &variation : qAsConst(d->variations)) {
+    for (const TileStampVariation &variation : std::as_const(d->variations)) {
         MapToVariantConverter converter;
         QVariant mapVariant = converter.toVariant(*variation.map, dir);
         QJsonValue mapJson = QJsonValue::fromVariant(mapVariant);
