@@ -39,7 +39,7 @@ UndoDock::UndoDock(QWidget *parent)
 
     QWidget *widget = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(widget);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->addWidget(mUndoView);
 
     setWidget(widget);
@@ -68,3 +68,5 @@ void UndoDock::retranslateUi()
     setWindowTitle(tr("History"));
     mUndoView->setEmptyLabel(tr("<empty>"));
 }
+
+#include "moc_undodock.cpp"
