@@ -1,6 +1,6 @@
 /*
  * undodock.h
- * Copyright 2009, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2009-2017, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  * Copyright 2010, Petr Viktorin <encukou@gmail.com>
  *
  * This file is part of Tiled.
@@ -19,16 +19,14 @@
  * this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef UNDODOCK_H
-#define UNDODOCK_H
+#pragma once
 
 #include <QDockWidget>
 
-class QUndoGroup;
+class QUndoStack;
 class QUndoView;
 
 namespace Tiled {
-namespace Internal {
 
 /**
  * A dock widget showing the undo stack. Mainly for debugging, but can also be
@@ -39,7 +37,9 @@ class UndoDock : public QDockWidget
     Q_OBJECT
 
 public:
-    UndoDock(QUndoGroup *undoGroup, QWidget *parent = nullptr);
+    UndoDock(QWidget *parent = nullptr);
+
+    void setStack(QUndoStack *stack);
 
 protected:
     void changeEvent(QEvent *e) override;
@@ -49,7 +49,4 @@ private:
     QUndoView *mUndoView;
 };
 
-} // namespace Internal
 } // namespace Tiled
-
-#endif // UNDODOCK_H
