@@ -31,6 +31,9 @@
 #include <QDBusMessage>
 #endif
 #include <QDesktopServices>
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+#include <QDesktopWidget>
+#endif
 #include <QDir>
 #include <QFileInfo>
 #include <QGuiApplication>
@@ -38,6 +41,8 @@
 #include <QImageWriter>
 #include <QJsonDocument>
 #include <QKeyEvent>
+#include <QLayout>
+#include <QLayoutItem>
 #include <QMainWindow>
 #include <QMenu>
 #include <QPainter>
@@ -48,7 +53,9 @@
 #endif
 #include <QScreen>
 
-#include "qtcompat_p.h"
+#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0) && QT_VERSION < QT_VERSION_CHECK(6, 5, 1)
+#include <QtCore/qapplicationstatic.h>
+#endif
 
 static QString toImageFileFilter(const QList<QByteArray> &formats)
 {
@@ -218,7 +225,7 @@ static int matchingScore(const QString &word, QStringRef string)
     int score = 1;  // empty word matches
     int previousIndex = -1;
 
-    for (const Match &match : qAsConst(indexes)) {
+    for (const Match &match : std::as_const(indexes)) {
         const int start = match.stringIndex == 0;
         const int sequential = match.stringIndex == previousIndex + 1;
 
@@ -238,7 +245,7 @@ static bool matchingRanges(const QString &word, QStringRef string, int offset, R
     if (!matchingIndexes(word, string, indexes))
         return false;
 
-    for (const Match &match : qAsConst(indexes))
+    for (const Match &match : std::as_const(indexes))
         result.insert(match.stringIndex + offset);
 
     return true;
@@ -280,8 +287,34 @@ RangeSet<int> matchingRanges(const QStringList &words, QStringRef string)
     return result;
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0) && QT_VERSION < QT_VERSION_CHECK(6, 5, 1)
+/*
+ * Caching icons here, since Qt no longer caches null icons in
+ * QIcon::fromTheme.
+ */
+using IconCache = QHash<QString, QIcon>;
+Q_APPLICATION_STATIC(IconCache, iconCache);
+
+QIcon themeIcon(const QString &name)
+{
+    IconCache *cache = iconCache();
+    auto it = cache->find(name);
+    if (it == cache->end())
+        it = cache->insert(name, QIcon::fromTheme(name));
+    return *it;
+}
+#else
+QIcon themeIcon(const QString &name)
+{
+    return QIcon::fromTheme(name);
+}
+#endif
+
 QIcon colorIcon(const QColor &color, QSize size)
 {
+    if (!color.isValid())
+        return QIcon();
+
     QPixmap pixmap(size);
     pixmap.fill(color);
 
@@ -292,6 +325,46 @@ QIcon colorIcon(const QColor &color, QSize size)
     return QIcon(pixmap);
 }
 
+/**
+ * Returns the available geometry of the screen containing the given \a widget.
+ */
+QRect screenRect(const QWidget *widget)
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    return QApplication::desktop()->availableGeometry(widget);
+#else
+    const QPoint center = widget->mapToGlobal(widget->rect().center());
+    const QScreen *screen = widget->screen()->virtualSiblingAt(center);
+    if (!screen)
+        screen = widget->screen();
+    return screen->availableGeometry();
+#endif
+}
+
+/**
+ * Returns the suitable geometry for a popup of the given \a popupSize,
+ * relative to the \a parent widget.
+ */
+QRect popupGeometry(const QWidget *parent, QSize popupSize)
+{
+    const QRect screen = screenRect(parent);
+    const QSize widgetSize = parent->size();
+    QPoint pos = parent->mapToGlobal(QPoint(0, widgetSize.height()));
+
+    // Move popup up when there is not enough space below
+    if (pos.y() + popupSize.height() > screen.bottom())
+        pos.ry() -= widgetSize.height() + popupSize.height();
+
+    // Align popup to the right when expected
+    if (parent->isRightToLeft())
+        pos.rx() += widgetSize.width() - popupSize.width();
+
+    // Make sure the popup is visible on the sides of the screen
+    pos.rx() = qBound(screen.left(), pos.x(), screen.right() - popupSize.width());
+
+    return QRect(pos, popupSize);
+}
+
 /**
  * Restores a widget's geometry.
  * Requires the widget to have its object name set.
@@ -485,7 +558,7 @@ void addFileManagerActions(QMenu &menu, const QString &fileName)
     if (fileName.isEmpty())
         return;
 
-    menu.addAction(QCoreApplication::translate("Utils", "Copy File Path"), [fileName] {
+    menu.addAction(QCoreApplication::translate("Utils", "Copy File Path"), &menu, [fileName] {
         QApplication::clipboard()->setText(QDir::toNativeSeparators(fileName));
     });
 
@@ -494,14 +567,14 @@ void addFileManagerActions(QMenu &menu, const QString &fileName)
 
 void addOpenContainingFolderAction(QMenu &menu, const QString &fileName)
 {
-    menu.addAction(QCoreApplication::translate("Utils", "Open Containing Folder..."), [fileName] {
+    menu.addAction(QCoreApplication::translate("Utils", "Open Containing Folder..."), &menu, [fileName] {
         showInFileManager(fileName);
     });
 }
 
 void addOpenWithSystemEditorAction(QMenu &menu, const QString &fileName)
 {
-    menu.addAction(QCoreApplication::translate("Utils", "Open with System Editor"), [=] {
+    menu.addAction(QCoreApplication::translate("Utils", "Open with System Editor"), &menu, [=] {
         QDesktopServices::openUrl(QUrl::fromLocalFile(fileName));
     });
 }
@@ -539,5 +612,20 @@ QString Error::jsonParseError(QJsonParseError error)
 
 }
 
+/**
+ * Recursively deletes all items and their widgets from the given layout.
+ */
+void deleteAllFromLayout(QLayout *layout)
+{
+    while (QLayoutItem *item = layout->takeAt(0)) {
+        delete item->widget();
+
+        if (QLayout *layout = item->layout())
+            deleteAllFromLayout(layout);
+
+        delete item;
+    }
+}
+
 } // namespace Utils
 } // namespace Tiled
