@@ -33,6 +33,7 @@
 
 class QAction;
 class QKeyEvent;
+class QLayout;
 class QMenu;
 
 namespace Tiled {
@@ -49,6 +50,8 @@ QString firstExtension(const QString &nameFilter);
 int matchingScore(const QStringList &words, QStringRef string);
 RangeSet<int> matchingRanges(const QStringList &words, QStringRef string);
 
+QIcon themeIcon(const QString &name);
+
 /**
  * Looks up the icon with the specified \a name from the system theme and set
  * it on the instance \a t when found.
@@ -62,9 +65,9 @@ template <class T>
 void setThemeIcon(T *t, const QString &name)
 {
 #ifdef Q_OS_LINUX
-    QIcon themeIcon = QIcon::fromTheme(name);
-    if (!themeIcon.isNull())
-        t->setIcon(themeIcon);
+    const QIcon icon = themeIcon(name);
+    if (!icon.isNull())
+        t->setIcon(icon);
 #else
     Q_UNUSED(t)
     Q_UNUSED(name)
@@ -79,6 +82,9 @@ void setThemeIcon(T *t, const char *name)
 
 QIcon colorIcon(const QColor &color, QSize size);
 
+QRect screenRect(const QWidget *widget);
+QRect popupGeometry(const QWidget *parent, QSize popupSize);
+
 void restoreGeometry(QWidget *widget);
 void saveGeometry(QWidget *widget);
 
@@ -106,5 +112,7 @@ namespace Error {
 QString jsonParseError(QJsonParseError error);
 } // namespace Error
 
+void deleteAllFromLayout(QLayout *layout);
+
 } // namespace Utils
 } // namespace Tiled
