@@ -67,6 +67,15 @@ VariantPropertyManager::VariantPropertyManager(QObject *parent)
             this, &VariantPropertyManager::slotPropertyDestroyed);
 }
 
+VariantPropertyManager::~VariantPropertyManager()
+{
+    // Disconnect early since these functions may be called during destruction
+    disconnect(this, &QtVariantPropertyManager::valueChanged,
+               this, &VariantPropertyManager::slotValueChanged);
+    disconnect(this, &QtAbstractPropertyManager::propertyDestroyed,
+               this, &VariantPropertyManager::slotPropertyDestroyed);
+}
+
 QVariant VariantPropertyManager::value(const QtProperty *property) const
 {
     if (mValues.contains(property))
@@ -168,13 +177,15 @@ int VariantPropertyManager::unstyledGroupTypeId()
 
 QString VariantPropertyManager::objectRefLabel(const MapObject &object)
 {
+    const QString &className = object.effectiveClassName();
+
     QString label = tr("%1: ").arg(object.id());
     if (!object.name().isEmpty()) {
         label.append(object.name());
-        if (!object.type().isEmpty())
-            label.append(tr(" (%1)").arg(object.type()));
-    } else if (!object.type().isEmpty())
-        label.append(tr("(%1)").arg(object.type()));
+        if (!className.isEmpty())
+            label.append(tr(" (%1)").arg(className));
+    } else if (!className.isEmpty())
+        label.append(tr("(%1)").arg(className));
     else
         label.append(tr("Unnamed object"));
 
@@ -336,12 +347,17 @@ void VariantPropertyManager::setAttribute(QtProperty *property,
     }
 
     if (mStringAttributes.contains(property)) {
+        StringAttributes &attributes = mStringAttributes[property];
         if (attribute == mSuggestionsAttribute) {
-            mStringAttributes[property].suggestions = val.toStringList();
+            const QStringList suggestions = val.toStringList();
+            if (attributes.suggestions == suggestions)
+                return;
+            attributes.suggestions = suggestions;
+            emit attributeChanged(property, attribute, val);
             return;
         }
         if (attribute == mMultilineAttribute) {
-            mStringAttributes[property].multiline = val.toBool();
+            attributes.multiline = val.toBool();
             return;
         }
     }
