@@ -20,7 +20,9 @@
 
 #include "wangcolormodel.h"
 
+#include "changeevents.h"
 #include "changewangcolordata.h"
+#include "mapdocument.h"
 #include "tileset.h"
 #include "tilesetdocument.h"
 #include "wangset.h"
@@ -74,7 +76,7 @@ QVariant WangColorModel::data(const QModelIndex &index, int role) const
         return wangColorAt(index)->name();
     case Qt::DecorationRole:
         if (Tile *tile =  mWangSet->tileset()->findTile(wangColorAt(index)->imageId()))
-            return tile->image();
+            return tile->image().copy(tile->imageRect());
         break;
     case ColorRole:
         return wangColorAt(index)->color();
@@ -130,24 +132,28 @@ void WangColorModel::setName(WangColor *wangColor, const QString &name)
 {
     wangColor->setName(name);
     emitDataChanged(wangColor);
+    emitToTilesetAndMaps(WangColorChangeEvent(wangColor, WangColorChangeEvent::NameProperty));
 }
 
 void WangColorModel::setImage(WangColor *wangColor, int imageId)
 {
     wangColor->setImageId(imageId);
     emitDataChanged(wangColor);
+    emitToTilesetAndMaps(WangColorChangeEvent(wangColor, WangColorChangeEvent::ImageProperty));
 }
 
 void WangColorModel::setColor(WangColor *wangColor, const QColor &color)
 {
     wangColor->setColor(color);
     emitDataChanged(wangColor);
+    emitToTilesetAndMaps(WangColorChangeEvent(wangColor, WangColorChangeEvent::ColorProperty));
 }
 
 void WangColorModel::setProbability(WangColor *wangColor, qreal probability)
 {
     wangColor->setProbability(probability);
     // no data changed signal because probability not exposed by model
+    emitToTilesetAndMaps(WangColorChangeEvent(wangColor, WangColorChangeEvent::ProbabilityProperty));
 }
 
 void WangColorModel::emitDataChanged(WangColor *wangColor)
@@ -156,4 +162,15 @@ void WangColorModel::emitDataChanged(WangColor *wangColor)
     emit dataChanged(i, i);
 }
 
+void WangColorModel::emitToTilesetAndMaps(const ChangeEvent &event)
+{
+    emit mTilesetDocument->changed(event);
+
+    // todo: this doesn't work reliably because it only reaches maps that use
+    // the tileset, whereas the Properties view can be showing stuff from any
+    // tileset.
+    for (MapDocument *mapDocument : mTilesetDocument->mapDocuments())
+        emit mapDocument->changed(event);
+}
+
 #include "moc_wangcolormodel.cpp"
