@@ -28,6 +28,7 @@
 
 #include "pluginmanager.h"
 #include "tmxrasterizer.h"
+#include "tmxmapformat.h"
 
 #include <QCommandLineParser>
 #include <QDebug>
@@ -43,6 +44,8 @@ static QString localFile(const QString &fileNameOrUrl)
 
 int main(int argc, char *argv[])
 {
+    Tiled::increaseImageAllocationLimit();
+
     QGuiApplication app(argc, argv);
 
     app.setOrganizationDomain(QLatin1String("mapeditor.org"));
@@ -51,38 +54,60 @@ int main(int argc, char *argv[])
 
     PluginManager::instance()->loadPlugins();
 
+    // Necessary to enable loading of object templates in XML format
+    XmlObjectTemplateFormat xmlObjectTemplateFormat;
+    PluginManager::addObject(&xmlObjectTemplateFormat);
+
     QCommandLineParser parser;
     parser.setApplicationDescription(QCoreApplication::translate("main", "Renders a Tiled map or world to an image."));
     parser.addHelpOption();
     parser.addVersionOption();
     parser.addOptions({
-                          { { "s", "scale" },
+                          { { QStringLiteral("s"), QStringLiteral("scale") },
                             QCoreApplication::translate("main", "The scale of the output image (default: 1)."),
                             QCoreApplication::translate("main", "scale") },
-                          { { "t", "tilesize" },
+                          { { QStringLiteral("t"), QStringLiteral("tilesize") },
                             QCoreApplication::translate("main", "The requested size in pixels at which a tile is rendered (overrides the --scale option)."),
                             QCoreApplication::translate("main", "size") },
-                          { "size",
+                          { QStringLiteral("size"),
                             QCoreApplication::translate("main", "The output image fits within a SIZE x SIZE square (overrides the --scale and --tilesize options)."),
                             QCoreApplication::translate("main", "size") },
-                          { { "a", "anti-aliasing" },
+                          { { QStringLiteral("a"), QStringLiteral("anti-aliasing") },
                             QCoreApplication::translate("main", "Antialias edges of primitives.") },
-                          { "no-smoothing",
+                          { QStringLiteral("no-smoothing"),
                             QCoreApplication::translate("main", "Use nearest neighbour instead of smooth blending of pixels.") },
-                          { "ignore-visibility",
+                          { QStringLiteral("ignore-visibility"),
                             QCoreApplication::translate("main", "Ignore all layer visibility flags in the map file, and render all layers in the output (default is to omit invisible layers).") },
-                          { "hide-layer",
+                          { QStringLiteral("hide-layer"),
                             QCoreApplication::translate("main", "Specifies a layer to omit from the output image. Can be repeated to hide multiple layers."),
                             QCoreApplication::translate("main", "name") },
-                          { "show-layer",
+                          { QStringLiteral("show-layer"),
                             QCoreApplication::translate("main", "If used only specified layers are shown. Can be repeated to show multiple specified layers only."),
                             QCoreApplication::translate("main", "name") },
-                          { "advance-animations",
-                            QCoreApplication::translate("main", "If used tile animations are advanced by the specified duration."),
-                            QCoreApplication::translate("main", "duration") }
+                          { QStringLiteral("hide-tile-layers"),
+                            QCoreApplication::translate("main", "Don't render tile layers.") },
+                          { QStringLiteral("hide-object-layers"),
+                            QCoreApplication::translate("main", "Don't render object layers.") },
+                          { QStringLiteral("hide-image-layers"),
+                            QCoreApplication::translate("main", "Don't render image layers.") },
+                          { QStringLiteral("hide-object"),
+                            QCoreApplication::translate("main", "Specifies an object to omit from the output image. Can be repeated to hide multiple objects. If multiple objects share the specified name they all will be hidden."),
+                            QCoreApplication::translate("main", "name") },
+                          { QStringLiteral("show-object"),
+                            QCoreApplication::translate("main", "If used only specified objects are shown. Can be repeated to show multiple specified objects only. If multiple objects share the specified name they all will be shown."),
+                            QCoreApplication::translate("main", "name") },
+                          { QStringLiteral("advance-animations"),
+                            QCoreApplication::translate("main", "If used, tile animations are advanced by the specified duration in milliseconds."),
+                            QCoreApplication::translate("main", "duration") },
+                          { QStringLiteral("frames"),
+                            QCoreApplication::translate("main", "Number of frames to export. This will add a frame number suffix to the image names. Animations are advanced by <frame-duration> for each frame."),
+                            QCoreApplication::translate("main", "number") },
+                          { QStringLiteral("frame-duration"),
+                            QCoreApplication::translate("main", "Duration of each frame in milliseconds, defaults to 100."),
+                            QCoreApplication::translate("main", "number") },
                       });
-    parser.addPositionalArgument("map|world", QCoreApplication::translate("main", "Map or world file to render."));
-    parser.addPositionalArgument("image", QCoreApplication::translate("main", "Image file to output."));
+    parser.addPositionalArgument(QStringLiteral("map|world"), QCoreApplication::translate("main", "Map or world file to render."));
+    parser.addPositionalArgument(QStringLiteral("image"), QCoreApplication::translate("main", "Image file to output."));
     parser.process(app);
 
     const QStringList args = parser.positionalArguments();
@@ -101,6 +126,11 @@ int main(int argc, char *argv[])
     w.setIgnoreVisibility(parser.isSet(QLatin1String("ignore-visibility")));
     w.setLayersToHide(parser.values(QLatin1String("hide-layer")));
     w.setLayersToShow(parser.values(QLatin1String("show-layer")));
+    w.setLayerTypeVisible(Layer::TileLayerType, !parser.isSet(QLatin1String("hide-tile-layers")));
+    w.setLayerTypeVisible(Layer::ObjectGroupType, !parser.isSet(QLatin1String("hide-object-layers")));
+    w.setLayerTypeVisible(Layer::ImageLayerType, !parser.isSet(QLatin1String("hide-image-layers")));
+    w.setObjectsToHide(parser.values(QLatin1String("hide-object")));
+    w.setObjectsToShow(parser.values(QLatin1String("show-object")));
 
     if (parser.isSet(QLatin1String("size"))) {
         bool ok;
@@ -138,5 +168,23 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (parser.isSet(QLatin1String("frames"))) {
+        bool ok;
+        w.setFrameCount(parser.value(QLatin1String("frames")).toInt(&ok));
+        if (!ok || w.frameCount() < 0) {
+            qWarning().noquote() << QCoreApplication::translate("main", "Invalid number of frames specified: \"%1\"").arg(parser.value(QLatin1String("frames")));
+            exit(1);
+        }
+    }
+
+    if (parser.isSet(QLatin1String("frame-duration"))) {
+        bool ok;
+        w.setFrameDuration(parser.value(QLatin1String("frame-duration")).toInt(&ok));
+        if (!ok || w.frameDuration() < 0) {
+            qWarning().noquote() << QCoreApplication::translate("main", "Invalid frame-duration specified: \"%1\"").arg(parser.value(QLatin1String("frame-duration")));
+            exit(1);
+        }
+    }
+
     return w.render(fileToOpen, fileToSave);
 }
