@@ -1,5 +1,5 @@
 <?xml version="1.0" encoding="UTF-8"?>
-<tileset name="perspective_walls" tilewidth="64" tileheight="64" tilecount="16" columns="4">
+<tileset version="1.2" tiledversion="1.2.3" name="perspective_walls" tilewidth="64" tileheight="64" tilecount="16" columns="4">
  <tileoffset x="-32" y="0"/>
  <image source="perspective_walls.png" width="256" height="256"/>
  <tile id="13">
